@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * PMain - main module (loads and inits all startup modules)
  *
@@ -61,23 +61,6 @@
 
 struct mainstruct *ocpmain = 0;
 
-static void plCloseAll(void)
-{
-	int i;
-
-	for (i=0;i<loadlist_n;i++)
-		if (loadlist[i].info->PreClose)
-			loadlist[i].info->PreClose();
-
-	for (i=0;i<loadlist_n;i++)
-		if (loadlist[i].info->Close)
-			loadlist[i].info->Close();
-
-	for (i=0;i<loadlist_n;i++)
-		if (loadlist[i].info->LateClose)
-			loadlist[i].info->LateClose();
-}
-
 static int cmdhlp(void)
 {
 	if (cfGetProfileString("commandline", "h", 0) || cfGetProfileString("commandline", "?", 0) || cfGetProfileString("commandline--", "help", 0))
@@ -134,7 +117,6 @@ extern char compiledate[], compiletime[]/*, compiledby[]*/;
 static int init_modules(int argc, char *argv[])
 {
 	int ret;
-	int i;
 
 	if ((ret=cmdhlp()))
 		return ret;
@@ -166,7 +148,7 @@ static int init_modules(int argc, char *argv[])
 
 	fprintf(stderr, "linking default objects...\n");
 
-	cfConfigSec="defaultconfig";
+	cfConfigSec=cfGetProfileString("CommandLine", "c", "defaultconfig");
 
 	{
 		int epoch = cfGetProfileInt("version", "epoch", 0, 10);
@@ -803,9 +785,9 @@ static int init_modules(int argc, char *argv[])
 		{
 			char *kernal, *basic, *chargen, *iter;
 			printf("ocp.ini update (0.2.95) libsidplayfp ROM paths changed into UNIX syntax\n");
-			kernal  = strdup (cfGetProfileString ("libsidplay", "kernal",  "KERNAL.ROM"));
-			basic   = strdup (cfGetProfileString ("libsidplay", "basic",   "BASIC.ROM"));
-			chargen = strdup (cfGetProfileString ("libsidplay", "chargen", "CHARGEN.ROM"));
+			kernal  = strdup (cfGetProfileString ("libsidplayfp", "kernal",  "KERNAL.ROM"));
+			basic   = strdup (cfGetProfileString ("libsidplayfp", "basic",   "BASIC.ROM"));
+			chargen = strdup (cfGetProfileString ("libsidplayfp", "chargen", "CHARGEN.ROM"));
 			for (iter = kernal; *iter; iter++)
 			{
 				if (*iter == '/')
@@ -836,9 +818,9 @@ static int init_modules(int argc, char *argv[])
 					*iter = '/';
 				}
 			}
-			cfSetProfileString ("libsidplay", "kernal",  kernal);
-			cfSetProfileString ("libsidplay", "basic",   basic);
-			cfSetProfileString ("libsidplay", "chargen", chargen);
+			cfSetProfileString ("libsidplayfp", "kernal",  kernal);
+			cfSetProfileString ("libsidplayfp", "basic",   basic);
+			cfSetProfileString ("libsidplayfp", "chargen", chargen);
 			free (kernal);
 			free (basic);
 			free (chargen);
@@ -853,26 +835,166 @@ static int init_modules(int argc, char *argv[])
 			cfRemoveProfile("devsNone");
 		}
 
-		if (epoch < 20220304)
+		if (epoch < 20220326)
 		{
-			cfSetProfileInt("version", "epoch", 20220304, 10);
+			printf("ocp.ini update (0.2.96) removed the not-used smpbufsize\n");
+			cfRemoveEntry("sound", "smpbufsize");
+		}
+
+		if (epoch < 20220326)
+		{
+			printf("ocp.ini update (0.2.96) appended stereo=on and 16bit=on to [devpDisk]\n");
+			cfSetProfileBool("devpDisk", "stereo", 1);
+			cfSetProfileBool("devpDisk", "16bit", 1);
+		}
+
+		if (epoch < 20220327)
+		{
+			printf("ocp.ini update (0.2.96) removed mix16bit, mixstereo, samp16bit and sampstereo\n");
+			cfRemoveEntry("sound", "mix16bit");
+			cfRemoveEntry("sound", "mixstereo");
+			cfRemoveEntry("sound", "samp16bit");
+			cfRemoveEntry("sound", "sampstereo");
+		}
+
+		if (epoch < 20220417)
+		{
+			printf("ocp.ini update (0.2.96) removed mixbufsize\n");
+			cfRemoveEntry("sound", "mixbufsize");
+		}
+
+		if (epoch < 20220919)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.100) removes [defaultconfig] link=medialib\n");
+			cfRemoveEntry("defaultconfig", "link");
+		}
+
+		if (epoch < 20221228)
+		{
+			char *tmp, *iter;
+
+			fprintf(stderr, "ocp.ini update (0.2.102) removes postprocadds\n");
+			cfRemoveEntry("devwMix", "postprocadds");
+			cfRemoveEntry("devwMixQ", "postprocadds");
+			cfRemoveEntry("devwMixF", "postprocadds");
+
+			fprintf(stderr, "ocp.ini update (0.2.102) _iReverb and _fReverb should be named iReverb and fReverb\n");
+
+			if (!strcmp (cfGetProfileString ("devwMix", "postprocs", ""),  "_iReverb")) cfSetProfileString ("devwMix",  "postprocs", "iReverb");
+			if (!strcmp (cfGetProfileString ("devwMixQ", "postprocs", ""), "_iReverb")) cfSetProfileString ("devwMixQ", "postprocs", "iReverb");
+			if (!strcmp (cfGetProfileString ("devwMixF", "postprocs", ""), "_fReverb")) cfSetProfileString ("devwMixF", "postprocs", "fReverb");
+
+			/* We silently repair the case-error in devwmixQ, this has no effect other than looking more consistent */
+			tmp = strdup (cfGetProfileString ("sound", "wavetabledevices", ""));
+			iter = strstr (tmp, "devwmixQ");
+			if (iter)
+			{
+				iter[4] = 'M';
+				cfSetProfileString ("sound", "wavetabledevices", tmp);
+			}
+			free (tmp);
+		}
+
+		if (epoch < 20230211)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpALSA] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpCA]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpDisk] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpMPx]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpNone]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpOSS] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpSDL]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devpSDL2]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwNone]\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMix] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMix] subtype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMixQ] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMixQ] subtype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMixF] link=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) removes [devwMixF] mixResample=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) defaults [devpALSA] card=default (former name was path=)\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) defaults [devpALSA] mixer=default\n");
+
+			cfRemoveEntry  ("devpALSA", "link");
+			cfRemoveProfile("devpCA"  );
+			cfRemoveEntry  ("devpDisk", "link");
+			cfRemoveProfile("devpMPx" );
+			cfRemoveProfile("devpNone");
+			cfRemoveEntry  ("devpOSS" , "link");
+			cfRemoveProfile("devpSDL" );
+			cfRemoveProfile("devpSDL2");
+			cfRemoveProfile("devwNone");
+			cfRemoveEntry  ("devwMix" , "link");
+			cfRemoveEntry  ("devwMix" , "subtype");
+			cfRemoveEntry  ("devwMixQ", "link");
+			cfRemoveEntry  ("devwMixQ", "subtype");
+			cfRemoveEntry  ("devwMixF", "link");
+			cfRemoveEntry  ("devwMixF", "mixResample");
+			cfSetProfileString ("devpALSA", "card", cfGetProfileString ("devpALSA", "path", "default"));
+			cfRemoveEntry  ("devpALSA", "path");
+			cfSetProfileString ("devpALSA", "mixer", cfGetProfileString ("devpALSA", "mixer", "default"));
+		}
+
+		if (epoch < 20230213)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.103) adds comment on [screen] mvoltype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) adds comment on [screen] insttype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) adds comment on [screen] channeltype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) adjust the default [screen] screentype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) replaces comment on [screen] screentype=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) move [x11] font= to [screen] fontsize=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) add [screen] winwidth=\n");
+			fprintf(stderr, "ocp.ini update (0.2.103) add [screen] winheight=\n");
+
+			cfSetProfileComment ("screen", "mvoltype", cfGetProfileComment ("screen", "mvoltype", "; 0=none, 1=big, 2=side (only in >132 column modes)"));
+			cfSetProfileComment ("screen", "insttype", cfGetProfileComment ("screen", "insttype", "; 0=none, 1=short, 2=long, 3=side (only in >132 column modes)"));
+			cfSetProfileComment ("screen", "channeltype", cfGetProfileComment ("screen", "channeltype", "; 0=none, 1=short, 2=long, 3=side (only in >132 column modes)"));
+			cfSetProfileInt     ("screen", "screentype", 5, 10);
+			cfSetProfileComment ("screen", "screentype", "; 0=80x25, 1=80x30, 2=80x50, 3=80x60, 4=132x25, 5=132x30, 6=132x50, 7=132x60, 8=custom");
+			cfSetProfileString  ("screen", "fontsize", cfGetProfileString ("x11", "font", "1"));
+			cfSetProfileComment ("screen", "fontsize", "; if screentype=8: 0=8x8, 1=8x16");
+			cfRemoveEntry       ("x11", "font");
+			cfSetProfileInt     ("screen", "winwidth", 1280, 10);
+			cfSetProfileComment ("screen", "winwidth", "; if screentype=8");
+			cfSetProfileInt     ("screen", "winheight", 1024, 10);
+			cfSetProfileComment ("screen", "winheight", "; if screentype=8");
+		}
+
+		if (epoch < 20230214)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.103), 0.2.91 update missed setting the color for XM files.\n");
+
+			cfSetProfileInt ("fscolors", "XM", cfGetProfileInt ("fscolors", "XM", 3, 10), 10);
+		}
+
+		if (epoch < 20230403)
+		{
+			fprintf(stderr, "ocp.ini update (0.2.104), added [adplug] emulator=nuked\n");
+
+			cfSetProfileString  ("adplug", "emulator", "nuked");
+			cfSetProfileComment ("adplug", "emulator", "; Possible values are ken, nuked, satoh and woody.");
+		}
+
+		if (epoch < 20230403)
+		{
+			cfSetProfileInt("version", "epoch", 20230403, 10);
 			cfStoreConfig();
 			if (isatty(2))
 			{
-				fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing this:\033[0m\nrm -f ~/.ocp/ocp.ini\n\n");
+				fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing either:\033[0m\nrm -f ~/.ocp/ocp.ini\033[1m\033[31m or \033[0m\nrm -f $XDG_CONFIG_HOME/ocp/ocp.ini\n\n");
 			} else {
-				fprintf(stderr,"\nWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing this:\nrm -f ~/.ocp/ocp.ini\n\n");
+				fprintf(stderr,"\nWARNING, ocp.ini has changed, have tried my best to update it. If OCP failes to start, please try to remove by doing either:\nrm -f ~/.ocp/ocp.ini or rm -f $XDG_CONFIG_HOME/ocp/ocp.ini\n\n");
 			}
 			sleep(5);
 		}
 	}
-	if (cfGetProfileInt("version", "epoch", 0, 10) != 20220304)
+	if (cfGetProfileInt("version", "epoch", 0, 10) != 20230403)
 	{
 		if (isatty(2))
 		{
-			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20220304\033[0m\n\n");
+			fprintf(stderr,"\n\033[1m\033[31mWARNING, ocp.ini [version] epoch != 20230403\033[0m\n\n");
 		} else {
-			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20220304\n\n");
+			fprintf(stderr,"\nWARNING, ocp.ini [version] epoch != 20230403\n\n");
 		}
 		sleep(5);
 	}
@@ -883,17 +1005,10 @@ static int init_modules(int argc, char *argv[])
 	lnkLink(cfGetProfileString2(cfConfigSec, "defaultconfig", "prelink", ""));
 	lnkLink(cfGetProfileString("general", "prelink", ""));
 
+	if (lnkLinkDir(cfProgramDirAutoload)<0)
 	{
-		char *buffer;
-		buffer = malloc (strlen (cfProgramDir) + 9 + 1);
-		sprintf(buffer, "%sautoload/", cfProgramDir);
-		if (lnkLinkDir(buffer)<0)
-		{
-			fprintf(stderr, "could not autoload directory: %s\n", buffer);
-			free (buffer);
-			return -1;
-		}
-		free (buffer);
+		fprintf(stderr, "could not autoload directory: %s\n", cfProgramDirAutoload);
+		return -1;
 	}
 
 	if (lnkLink(cfGetProfileString("general", "link", ""))<0)
@@ -910,20 +1025,10 @@ static int init_modules(int argc, char *argv[])
 
 	fprintf(stderr, "running initializers...\n");
 
-	for (i=0;i<loadlist_n;i++)
-		if (loadlist[i].info->PreInit)
-			if (loadlist[i].info->PreInit()<0)
-				return errGen;
-
-	for (i=0;i<loadlist_n;i++)
-		if (loadlist[i].info->Init)
-			if (loadlist[i].info->Init()<0)
-				return errGen;
-
-	for (i=0;i<loadlist_n;i++)
-		if (loadlist[i].info->LateInit)
-			if (loadlist[i].info->LateInit()<0)
-				return errGen;
+	if (lnkInitAll())
+	{
+		return errGen;
+	}
 
 	if (!ocpmain)
 	{
@@ -933,14 +1038,12 @@ static int init_modules(int argc, char *argv[])
 	if (ocpmain->main(argc, argv)<0)
 		return errGen;
 
-	plSetTextMode(255);
-
 	return 0;
 }
 
 void done_modules(void)
 {
-	plCloseAll();
+	lnkCloseAll();
 	lnkFree(0);
 }
 
@@ -953,7 +1056,7 @@ int failcheck(signed int source, signed int filter)
 }
 #endif
 
-static int _bootup(int argc, char *argv[], const char *ConfigDir, const char *DataDir, const char *ProgramDir)
+static int _bootup(int argc, char *argv[], const char *ConfigHomeDir, const char *ConfigDataDir, const char *DataDir, const char *ProgramDir)
 {
 	int result;
 	if (isatty(2))
@@ -983,16 +1086,21 @@ static int _bootup(int argc, char *argv[], const char *ConfigDir, const char *Da
 	fprintf(stderr, "pass\n");
 #endif
 
-	cfConfigDir = (char *)ConfigDir;
+	cfConfigHomeDir = (char *)ConfigHomeDir;
+	cfDataHomeDir = (char*)ConfigDataDir;
 	cfDataDir = strdup (DataDir);
 	cfProgramDir = (char *)ProgramDir;
+	cfProgramDirAutoload = malloc (strlen (cfProgramDir) + 9 + 1);
+	sprintf(cfProgramDirAutoload, "%sautoload/", cfProgramDir);
 
 	if (cfGetConfig(argc, argv))
 	{
-		cfConfigDir = 0;
+		cfConfigHomeDir = 0;
+		cfDataHomeDir = 0;
 		free (cfDataDir); cfDataDir = 0;
 		cfProgramDir = 0;
 		free (cfTempDir); cfTempDir = 0;
+		free (cfProgramDirAutoload); cfProgramDirAutoload = 0;
 		return -1;
 	}
 
@@ -1005,10 +1113,13 @@ static int _bootup(int argc, char *argv[], const char *ConfigDir, const char *Da
 
 	cfCloseConfig();
 
-	cfConfigDir = 0;
+	cfConfigHomeDir = 0;
+	cfDataHomeDir = 0;
 	free (cfDataDir); cfDataDir = 0;
 	cfProgramDir = 0;
 	free (cfTempDir); cfTempDir = 0;
+	free (cfProgramDirAutoload); cfProgramDirAutoload = 0;
+
 
 	return 0;
 }
