@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * OCP.INI file and environment reading functions
  *
@@ -42,21 +42,21 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include "types.h"
 #include "psetting.h"
 #include "stuff/compat.h"
 
-char *cfConfigDir;
-char *cfDataDir;
 char *cfProgramDir;
-char *cfTempDir;
+char *cfProgramDirAutoload;
 
 #define KEYBUF_LEN 105
 #define STRBUF_LEN 405
 #define COMMENTBUF_LEN 256
+#define COMMENT_INDENT 26
 
-const char *cfConfigSec;
-const char *cfSoundSec;
-const char *cfScreenSec;
+#ifndef MAX
+# define MAX(x,y) ((x)>(y)?(x):(y))
+#endif
 
 struct profilekey
 {
@@ -92,8 +92,7 @@ static int readiniline(char *key, char *str, char *comment, const char *line)
   /* this line is a comment ?*/
 	if (((*line)==';')||((*line)=='#')||(!*line))
 	{
-		strncpy(comment, line, COMMENTBUF_LEN);
-		comment[COMMENTBUF_LEN-1]=0;
+		snprintf (comment, COMMENTBUF_LEN, "%.*s", COMMENTBUF_LEN-1, line);
 		return 0;
 	}
 
@@ -165,7 +164,7 @@ static int cfReadINIFile(int argc, char *argv[])
 	char linebuffer[1024];
 	/*  int curapp=-1;*/
 
-	makepath_malloc (&path, 0, cfConfigDir, "ocp.ini", 0);
+	makepath_malloc (&path, 0, cfConfigHomeDir, "ocp.ini", 0);
 
 	strcpy(keybuf, "");
 
@@ -200,7 +199,7 @@ static int cfReadINIFile(int argc, char *argv[])
 					int index=cfINIApps[cfINIApps_index].nkeys;
 					cfINIApps[cfINIApps_index].nkeys++;
 					memtmp = realloc(cfINIApps[cfINIApps_index].keys, sizeof(cfINIApps[cfINIApps_index].keys[0])*(index+1));
-					if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #1 (%ld)\n", sizeof(cfINIApps[cfINIApps_index].keys[0])*(index+1)); _exit(1); }
+					if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #1 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINIApps_index].keys[0])*(index+1))); _exit(1); }
 					cfINIApps[cfINIApps_index].keys = memtmp;
 					cfINIApps[cfINIApps_index].keys[index].key=NULL;
 					cfINIApps[cfINIApps_index].keys[index].str=NULL;
@@ -224,7 +223,7 @@ static int cfReadINIFile(int argc, char *argv[])
 					cfINIApps_index=cfINInApps;
 					cfINInApps++;
 					memtmp = realloc(cfINIApps, sizeof(cfINIApps[cfINInApps])*cfINInApps);
-					if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #2 (%ld)\n", sizeof(cfINIApps[cfINInApps])*cfINInApps); _exit(1); }
+					if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #2 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINInApps])*cfINInApps)); _exit(1); }
 					cfINIApps = memtmp;
 					cfINIApps[cfINIApps_index].app=strdup(strbuf);
 					cfINIApps[cfINIApps_index].keys=NULL;
@@ -239,7 +238,7 @@ static int cfReadINIFile(int argc, char *argv[])
 					int index=cfINIApps[cfINIApps_index].nkeys;
 					cfINIApps[cfINIApps_index].nkeys++;
 					memtmp = realloc(cfINIApps[cfINIApps_index].keys, sizeof(cfINIApps[cfINIApps_index].keys[0])*(index+1));
-					if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #3 (%ld)\n", sizeof(cfINIApps[cfINIApps_index].keys[0])*(index+1)); _exit(1); }
+					if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #3 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINIApps_index].keys[0])*(index+1))); _exit(1); }
 					cfINIApps[cfINIApps_index].keys = memtmp;
 					cfINIApps[cfINIApps_index].keys[index].key=strdup(keybuf);
 					cfINIApps[cfINIApps_index].keys[index].str=strdup(strbuf);
@@ -256,14 +255,15 @@ static int cfReadINIFile(int argc, char *argv[])
 		for (c=1;c<argc;c++)
 			if ((argv[c][0]=='-')&&argv[c][1])
 			{
-				if ((argv[c][1]=='-')&&(!argv[c][2])) /* Unix legacy */
+				if ((argv[c][1]=='-')&&(!argv[c][2])) /* Unix legacy: stop reading parameters if ran like       ./ocp -dcurses -- -filename.xm */
 					break;
-				if (argv[c][1]=='-')
+				if (argv[c][1]=='-') /* Ignore parameters that start with double dash like  ./ocp --help */
 					continue;
 
+				/* Generate a new section as ini file contained [commandline_x] and create keypairs for supporting v100,p80,c10,dcurses => v=100 p=80 c=10 d=curses */
 				cfINInApps++;
 				memtmp = realloc(cfINIApps, sizeof(cfINIApps[cfINInApps])*cfINInApps);
-				if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #4 (%ld)\n", sizeof(cfINIApps[cfINInApps])*cfINInApps); _exit(1); }
+				if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #4 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINInApps])*cfINInApps)); _exit(1); }
 				cfINIApps = memtmp;
 				cfINIApps[cfINInApps-1].app=strdup("commandline__");
 				cfINIApps[cfINInApps-1].app[12]=argv[c][1];
@@ -275,7 +275,7 @@ static int cfReadINIFile(int argc, char *argv[])
 				argvstat=argv[c]+2;
 				while (*argvstat)
 				{
-					char *temp=index(argvstat, ',');
+					char *temp=strchr(argvstat, ',');
 					int index=cfINIApps[cfINInApps-1].nkeys;
 
 					if (!temp)
@@ -283,7 +283,7 @@ static int cfReadINIFile(int argc, char *argv[])
 
 					cfINIApps[cfINInApps-1].nkeys++;
 					memtmp = realloc(cfINIApps[cfINInApps-1].keys, sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1));
-					if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #5 (%ld)\n", sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1)); _exit(1); }
+					if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #5 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1))); _exit(1); }
 					cfINIApps[cfINInApps-1].keys = memtmp;
 					cfINIApps[cfINInApps-1].keys[index].key=strdup("_");
 					cfINIApps[cfINInApps-1].keys[index].key[0]=*argvstat;
@@ -299,9 +299,10 @@ static int cfReadINIFile(int argc, char *argv[])
 				}
 			}
 
+		/* Generate a new section as ini file contained [CommandLine] and create keypairs for all arguments as-is:   v100,p10,c10,dcurses => v=100,p10,c10,dcurses */
 		cfINInApps++;
 		memtmp = realloc(cfINIApps, sizeof(cfINIApps[cfINInApps])*cfINInApps);
-		if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #6 (%ld)\n", sizeof(cfINIApps[cfINInApps])*cfINInApps); _exit(1); }
+		if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #6 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINInApps])*cfINInApps)); _exit(1); }
 		cfINIApps = memtmp;
 		cfINIApps[cfINInApps-1].app=strdup("CommandLine");
 		cfINIApps[cfINInApps-1].keys=NULL;
@@ -314,12 +315,12 @@ static int cfReadINIFile(int argc, char *argv[])
 			{
 				int index=cfINIApps[cfINInApps-1].nkeys;
 
-				if ((argv[c][1]=='-')&&(!argv[c][2])) /* Unix legacy */
+				if ((argv[c][1]=='-')&&(!argv[c][2])) /* Unix legacy: stop reading parameters if ran like       ./ocp -dcurses -- -filename.xm */
 					break;
 
 				cfINIApps[cfINInApps-1].nkeys++;
 				memtmp = realloc(cfINIApps[cfINInApps-1].keys, sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1));
-				if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #7 (%ld)\n", sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1)); _exit(1); }
+				if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #7 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1))); _exit(1); }
 				cfINIApps[cfINInApps-1].keys = memtmp;
 				cfINIApps[cfINInApps-1].keys[index].key=strdup("_");
 				cfINIApps[cfINInApps-1].keys[index].key[0]=argv[c][1];
@@ -328,9 +329,10 @@ static int cfReadINIFile(int argc, char *argv[])
 				cfINIApps[cfINInApps-1].keys[index].comment=NULL;
 			}
 
+		/* Generate a new section as ini file contained [CommandLine--] and create keypairs for all --arguments as-is --help  => help=1 */
 		cfINInApps++;
 		memtmp = realloc(cfINIApps, sizeof(cfINIApps[cfINInApps])*cfINInApps);
-		if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #8 (%ld)\n", sizeof(cfINIApps[cfINInApps])*cfINInApps); _exit(1); }
+		if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #8 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINInApps])*cfINInApps)); _exit(1); }
 		cfINIApps = memtmp;
 		cfINIApps[cfINInApps-1].app=strdup("CommandLine--");
 		cfINIApps[cfINInApps-1].keys=NULL;
@@ -343,12 +345,12 @@ static int cfReadINIFile(int argc, char *argv[])
 			{
 				int index=cfINIApps[cfINInApps-1].nkeys;
 
-				if (!argv[c][2]) /* Unix legacy */
+				if (!argv[c][2]) /* Unix legacy: stop reading parameters if ran like       ./ocp -dcurses -- -filename.xm */
 					break;
 
 				cfINIApps[cfINInApps-1].nkeys++;
 				memtmp = realloc(cfINIApps[cfINInApps-1].keys, sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1));
-				if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #9 (%ld)\n", sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1)); _exit(1); }
+				if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #9 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1))); _exit(1); }
 				cfINIApps[cfINInApps-1].keys = memtmp;
 				cfINIApps[cfINInApps-1].keys[index].key=strdup(argv[c]+2);
 				cfINIApps[cfINInApps-1].keys[index].str=strdup("1");
@@ -358,7 +360,7 @@ static int cfReadINIFile(int argc, char *argv[])
 
 		cfINInApps++;
 		memtmp = realloc(cfINIApps, sizeof(cfINIApps[cfINInApps])*cfINInApps);
-		if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #10 (%ld)\n", sizeof(cfINIApps[cfINInApps])*cfINInApps); _exit(1); }
+		if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #10 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINInApps])*cfINInApps)); _exit(1); }
 		cfINIApps = memtmp;
 		cfINIApps[cfINInApps-1].app=strdup("CommandLine_Files");
 		cfINIApps[cfINInApps-1].keys=NULL;
@@ -395,7 +397,7 @@ static int cfReadINIFile(int argc, char *argv[])
 						sprintf(buffer, "playlist%d", playlists++);
 					cfINIApps[cfINInApps-1].nkeys++;
 					memtmp = realloc(cfINIApps[cfINInApps-1].keys, sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1));
-					if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #11 (%ld)\n", sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1)); _exit(1); }
+					if (!memtmp) { fprintf (stderr, "cfReadINIFile() realloc failed #11 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINInApps-1].keys[0])*(index+1))); _exit(1); }
 					cfINIApps[cfINInApps-1].keys = memtmp;
 					cfINIApps[cfINInApps-1].keys[index].key=strdup(buffer);
 					cfINIApps[cfINInApps-1].keys[index].str=strdup(argv[c]+(argv[c][0]=='@'?1:0));
@@ -438,7 +440,7 @@ void cfCloseConfig()
 	cfFreeINI();
 }
 
-const char *cfGetProfileString(const char *app, const char *key, const char *def)
+static const char *_cfGetProfileString(const char *app, const char *key, const char *def)
 {
 	int i,j;
 	for (i=0; i<cfINInApps; i++)
@@ -450,12 +452,12 @@ const char *cfGetProfileString(const char *app, const char *key, const char *def
 	return def;
 }
 
-const char *cfGetProfileString2(const char *app, const char *app2, const char *key, const char *def)
+static const char *_cfGetProfileString2(const char *app, const char *app2, const char *key, const char *def)
 {
 	return cfGetProfileString(app, key, cfGetProfileString(app2, key, def));
 }
 
-void cfSetProfileString(const char *app, const char *key, const char *str)
+static void _cfSetProfileString(const char *app, const char *key, const char *str)
 {
 	int i, j;
 	void *memtmp;
@@ -466,6 +468,7 @@ void cfSetProfileString(const char *app, const char *key, const char *str)
 				if (cfINIApps[i].keys[j].key)
 					if (!strcasecmp(cfINIApps[i].keys[j].key, key))
 					{
+						if (cfINIApps[i].keys[j].str == str) return;
 						free(cfINIApps[i].keys[j].str);
 						cfINIApps[i].keys[j].str = strdup (str);
 						return;
@@ -474,7 +477,7 @@ doappend:
 			j=cfINIApps[i].nkeys;
 			cfINIApps[i].nkeys++;
 			memtmp = realloc(cfINIApps[i].keys, sizeof(cfINIApps[i].keys[0])*(j+1));
-			if (!memtmp) { fprintf (stderr, "cfSetProfileString() realloc failed #1 (%ld)\n", sizeof(cfINIApps[i].keys[0])*(j+1)); _exit(1); }
+			if (!memtmp) { fprintf (stderr, "cfSetProfileString() realloc failed #1 (%lu)\n", (unsigned long)(sizeof(cfINIApps[i].keys[0])*(j+1))); _exit(1); }
 			cfINIApps[i].keys = memtmp;
 			cfINIApps[i].keys[j].key=strdup(key);
 			cfINIApps[i].keys[j].str=strdup(str);
@@ -484,7 +487,7 @@ doappend:
 		}
 	cfINInApps++;
 	memtmp = realloc(cfINIApps, sizeof(cfINIApps[cfINInApps])*cfINInApps);
-	if (!memtmp) { fprintf (stderr, "cfSetProfileString() realloc failed #2 (%ld)\n", sizeof(cfINIApps[cfINInApps])*cfINInApps); _exit(1); }
+	if (!memtmp) { fprintf (stderr, "cfSetProfileString() realloc failed #2 (%lu)\n", (unsigned long)(sizeof(cfINIApps[cfINInApps])*cfINInApps)); _exit(1); }
 	cfINIApps = memtmp;
 	cfINIApps[i].app=strdup(app);
 	cfINIApps[i].keys=NULL;
@@ -494,32 +497,32 @@ doappend:
 	goto doappend;
 }
 
-int cfGetProfileInt(const char *app, const char *key, int def, int radix)
+static int _cfGetProfileInt(const char *app, const char *key, int def, int radix)
 {
-	const char *s=cfGetProfileString(app, key, "");
+	const char *s=_cfGetProfileString(app, key, "");
 	if (!*s)
 		return def;
 	return strtol(s, 0, radix);
 }
 
-int cfGetProfileInt2(const char *app, const char *app2, const char *key, int def, int radix)
+static int _cfGetProfileInt2(const char *app, const char *app2, const char *key, int def, int radix)
 {
-	return cfGetProfileInt(app, key, cfGetProfileInt(app2, key, def, radix), radix);
+	return _cfGetProfileInt(app, key, _cfGetProfileInt(app2, key, def, radix), radix);
 }
 
-void cfSetProfileInt(const char *app, const char *key, int str, int radix)
+static void _cfSetProfileInt(const char *app, const char *key, int str, int radix)
 {
 	char buffer[64];
 	if (radix==16)
 		snprintf(buffer, sizeof(buffer), "0x%x", str);
 	else
 		snprintf(buffer, sizeof(buffer), "%d", str);
-	cfSetProfileString(app, key, buffer);
+	_cfSetProfileString(app, key, buffer);
 }
 
-int cfGetProfileBool(const char *app, const char *key, int def, int err)
+static int _cfGetProfileBool(const char *app, const char *key, int def, int err)
 {
-	const char *s=cfGetProfileString(app, key, 0);
+	const char *s=_cfGetProfileString(app, key, 0);
 	if (!s)
 		return def;
 	if (!*s)
@@ -531,17 +534,53 @@ int cfGetProfileBool(const char *app, const char *key, int def, int err)
 	return err;
 }
 
-int cfGetProfileBool2(const char *app, const char *app2, const char *key, int def, int err)
+static int _cfGetProfileBool2(const char *app, const char *app2, const char *key, int def, int err)
 {
-	return cfGetProfileBool(app, key, cfGetProfileBool(app2, key, def, err), err);
+	return _cfGetProfileBool(app, key, _cfGetProfileBool(app2, key, def, err), err);
 }
 
-void cfSetProfileBool(const char *app, const char *key, const int str)
+static void _cfSetProfileBool(const char *app, const char *key, const int str)
 {
-	cfSetProfileString(app, key, (str?"on":"off"));
+	_cfSetProfileString(app, key, (str?"on":"off"));
 }
 
-void cfRemoveEntry(const char *app, const char *key)
+static const char *_cfGetProfileComment(const char *app, const char *key, const char *def)
+{
+	int i,j;
+	for (i=0; i<cfINInApps; i++)
+		if (!strcasecmp(cfINIApps[i].app, app))
+			for (j=0; j<cfINIApps[i].nkeys; j++)
+				if (cfINIApps[i].keys[j].key)
+					if (!strcasecmp(cfINIApps[i].keys[j].key, key))
+						return cfINIApps[i].keys[j].comment ? cfINIApps[i].keys[j].comment : def;
+	return def;
+}
+
+static void _cfSetProfileComment(const char *app, const char *key, const char *comment)
+{
+	int i, j;
+	for (i=0; i<cfINInApps; i++)
+	{
+		if (!strcasecmp(cfINIApps[i].app, app))
+		{
+			for (j=0; j<cfINIApps[i].nkeys; j++)
+			{
+				if (cfINIApps[i].keys[j].key)
+				{
+					if (!strcasecmp(cfINIApps[i].keys[j].key, key))
+					{
+						if (cfINIApps[i].keys[j].comment == comment) return;
+						free(cfINIApps[i].keys[j].comment);
+						cfINIApps[i].keys[j].comment = strdup (comment);
+						return;
+					}
+				}
+			}
+		}
+	}
+}
+
+static void _cfRemoveEntry(const char *app, const char *key)
 {
 	int i, j;
 	for (i=0; i<cfINInApps; i++)
@@ -571,7 +610,7 @@ void cfRemoveEntry(const char *app, const char *key)
 		}
 }
 
-void cfRemoveProfile(const char *app)
+static void _cfRemoveProfile(const char *app)
 {
 	int i, j;
 	for (i=0; i<cfINInApps; i++)
@@ -599,7 +638,7 @@ void cfRemoveProfile(const char *app)
 	}
 }
 
-int cfCountSpaceList(const char *str, int maxlen)
+static int _cfCountSpaceList(const char *str, int maxlen)
 {
 	int i=0;
 	while (1)
@@ -618,7 +657,7 @@ int cfCountSpaceList(const char *str, int maxlen)
 	}
 }
 
-int cfGetSpaceListEntry(char *buf, const char **str, int maxlen)
+static int _cfGetSpaceListEntry(char *buf, const char **str, int maxlen)
 {
 	while (1)
 	{
@@ -647,18 +686,18 @@ int cfGetConfig(int argc, char *argv[])
 		return -1; /* no config at all pigs! */
 	if (cfReadINIFile(argc, argv))
 	{
-		fprintf(stderr, "Failed to read ocp.ini\nPlease put it in ~/.ocp/\n");
+		fprintf(stderr, "Failed to read ocp.ini\nPlease put it in ~/.ocp/ or $XDG_CONFIG_HOME/ocp/\n");
 		return -1;
 	}
 
-	t=cfGetProfileString("general", "datadir", NULL);
+	t=_cfGetProfileString("general", "datadir", NULL);
 	if (t)
 	{
 		free (cfDataDir);
 		cfDataDir = strdup (t);
 	}
 
-	if ((t=cfGetProfileString("general", "tempdir", t)))
+	if ((t=_cfGetProfileString("general", "tempdir", t)))
 	{
 		cfTempDir = strdup (t);
 	} else if ((t=getenv("TEMP")))
@@ -714,14 +753,13 @@ int cfGetConfig(int argc, char *argv[])
 	return 0;
 }
 
-int cfStoreConfig(void)
+static int _cfStoreConfig(void)
 {
 	char *path;
 	FILE *f;
 	int i, j;
-	char buffer[2+KEYBUF_LEN+1+STRBUF_LEN+COMMENTBUF_LEN+32+1+1];
 
-	makepath_malloc (&path, 0, cfConfigDir, "ocp.ini", 0);
+	makepath_malloc (&path, 0, cfConfigHomeDir, "ocp.ini", 0);
 
 	if (!(f=fopen(path, "w")))
 	{
@@ -732,42 +770,73 @@ int cfStoreConfig(void)
 	free (path); path=0;
 
 	for (i=0;i<cfINInApps;i++)
+	{
 		if (cfINIApps[i].linenum>=0)
 		{
-			strcpy(buffer, "[");
-			strcat(buffer, cfINIApps[i].app);
-			strcat(buffer, "]");
+			if (i)
+			{
+				fprintf (f, "\n");
+			}
+			fprintf (f, "[%.*s]", KEYBUF_LEN, cfINIApps[i].app);
 			if (cfINIApps[i].comment)
 			{
-				int n=strlen(buffer)-32;
-				if (n>0)
-					strncat(buffer, "                                ", n);
-				strcat(buffer, cfINIApps[i].comment);
+				fprintf (f, "%*s%.*s",
+			                MAX (0, COMMENT_INDENT - 2 - (int)strlen(cfINIApps[i].app)), "",
+					COMMENTBUF_LEN, cfINIApps[i].comment);
 			}
-			strcat(buffer, "\n");
+			fprintf (f, "\n");
 
-			fprintf(f, "%s", buffer);
 			for (j=0;j<cfINIApps[i].nkeys;j++)
+			{
 				if (cfINIApps[i].keys[j].linenum>=0)
 				{
 					if (cfINIApps[i].keys[j].key)
 					{
-						strcpy(buffer, "  ");
-						strcat(buffer, cfINIApps[i].keys[j].key);
-						strcat(buffer, "=");
-						strcat(buffer, cfINIApps[i].keys[j].str);
+						fprintf (f, "  %.*s=%.*s",
+							KEYBUF_LEN, cfINIApps[i].keys[j].key,
+							STRBUF_LEN, cfINIApps[i].keys[j].str);
 						if (cfINIApps[i].keys[j].comment)
 						{
-							while (strlen(buffer)<32)
-								strcat(buffer, " ");
-							strcat(buffer, cfINIApps[i].keys[j].comment);
+							fprintf (f, "%*s%.*s",
+								MAX (0, COMMENT_INDENT - 3 - (int)strlen(cfINIApps[i].keys[j].key) - (int)strlen(cfINIApps[i].keys[j].str)), "",
+								COMMENTBUF_LEN, cfINIApps[i].keys[j].comment);
 						}
-					} else
-						strcpy(buffer, cfINIApps[i].keys[j].comment);
-					strcat(buffer, "\n");
-					fprintf(f, "%s", buffer);
+						fprintf (f, "\n");
+					} else if (cfINIApps[i].keys[j].comment)
+					{
+						fprintf (f, "%.*s\n", COMMENTBUF_LEN, cfINIApps[i].keys[j].comment);
+					}
 				}
+			}
 		}
+	}
 	fclose(f);
 	return 0;
 }
+
+struct configAPI_t configAPI =
+{
+	_cfStoreConfig,
+	_cfGetProfileString,
+	_cfGetProfileString2,
+	_cfSetProfileString,
+	_cfGetProfileBool,
+	_cfGetProfileBool2,
+	_cfSetProfileBool,
+	_cfGetProfileInt,
+	_cfGetProfileInt2,
+	_cfSetProfileInt,
+	_cfGetProfileComment,
+	_cfSetProfileComment,
+	_cfRemoveEntry,
+	_cfRemoveProfile,
+	0,
+	0,
+	0,
+	0,
+	0,
+	0,
+	0,
+	_cfCountSpaceList,
+	_cfGetSpaceListEntry
+};
