@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2011-'20 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2011-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * CPIface text track/pattern display
  *
@@ -39,6 +39,7 @@
 #include "stuff/poutput.h"
 #include "boot/psetting.h"
 #include "cpiface.h"
+#include "cpiface-private.h"
 
 #define COLPTNOTE 0x0A
 #define COLNOTE 0x0F
@@ -63,17 +64,17 @@
 
 static int plTrackActive;
 
-static int (*getcurpos)();
-static int (*getpatlen)(int n);
-static const char *(*getpatname)(int n);
-static void (*seektrack)(int n, int c);
-static int (*startrow)();
-static int (*getnote)(uint16_t *bp, int small);
-static int (*getins)(uint16_t *bp);
-static int (*getvol)(uint16_t *bp);
-static int (*getpan)(uint16_t *bp);
-static void (*getfx)(uint16_t *bp, int n);
-static void (*getgcmd)(uint16_t *bp, int n);
+static int         (*getcurpos) (struct cpifaceSessionAPI_t *cpifaceSession);
+static int         (*getpatlen) (struct cpifaceSessionAPI_t *cpifaceSession, int n);
+static const char *(*getpatname)(struct cpifaceSessionAPI_t *cpifaceSession, int n);
+static void        (*seektrack) (struct cpifaceSessionAPI_t *cpifaceSession, int n, int c);
+static int         (*startrow)  (struct cpifaceSessionAPI_t *cpifaceSession);
+static int         (*getnote)   (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp, int small);
+static int         (*getins)    (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp);
+static int         (*getvol)    (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp);
+static int         (*getpan)    (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp);
+static void        (*getfx)     (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp, int n);
+static void        (*getgcmd)   (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp, int n);
 
 static int plPatternNum;
 static int plPrepdPat;
@@ -90,16 +91,19 @@ static char pattitle1[CONSOLE_MAX_X+1];
 static uint16_t pattitle2[CONSOLE_MAX_X];
 static int patwidth, patpad;
 
+#warning Make a better API for this
+static int overrideplNLChan;
+
 enum
 {
 	cpiTrkFXIns=1,cpiTrkFXNote=2,cpiTrkFXVol=4,cpiTrkFXNoPan=8
 };
 
-static void getfx2(uint16_t *bp, int n, int o)
+static void getfx2 (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp, int n, int o)
 {
 	int p=0;
 	if (o&cpiTrkFXIns)
-		if (getins(bp+1))
+		if (getins (cpifaceSession, bp+1))
 		{
 			writestring(bp, 0, COLINS, "i", 1);
 			p++;
@@ -108,7 +112,7 @@ static void getfx2(uint16_t *bp, int n, int o)
 	if (p==n)
 		return;
 	if (o&cpiTrkFXNote)
-		if (getnote(bp, 0))
+		if (getnote (cpifaceSession, bp, 0))
 		{
 			p++;
 			bp+=3;
@@ -116,7 +120,7 @@ static void getfx2(uint16_t *bp, int n, int o)
 	if (p==n)
 		return;
 	if (o&cpiTrkFXVol)
-		if (getvol(bp+1))
+		if (getvol (cpifaceSession, bp+1))
 		{
 			writestring(bp, 0, COLVOL, "v", 1);
 			p++;
@@ -125,7 +129,7 @@ static void getfx2(uint16_t *bp, int n, int o)
 	if (p==n)
 		return;
 	if (!(o&cpiTrkFXNoPan))
-		if (getpan(bp+1))
+		if (getpan (cpifaceSession, bp+1))
 		{
 			writestring(bp, 0, COLPAN, "p", 1);
 			p++;
@@ -133,24 +137,24 @@ static void getfx2(uint16_t *bp, int n, int o)
 		}
 	if (p==n)
 		return;
-	getfx(bp, n-p);
+	getfx (cpifaceSession, bp, n-p);
 }
 
-static void getscrollpos(int tr, int *firstchan, int *chnn)
+static void getscrollpos (struct cpifaceSessionAPI_t *cpifaceSession, int tr, int *firstchan, int *chnn)
 {
-	if (plNLChan>tr)
+	if (overrideplNLChan>tr)
 	{
-		if (plSelCh<(tr>>1))
+		if (cpifaceSession->SelectedChannel < (tr>>1))
 			*firstchan=0;
 		else
-			if (plSelCh>=(plNLChan-(tr>>1)))
-				*firstchan=plNLChan-tr;
+			if (cpifaceSession->SelectedChannel >= (overrideplNLChan-(tr>>1)))
+				*firstchan=overrideplNLChan-tr;
 			else
-				*firstchan=plSelCh-((tr>>1));
+				*firstchan=cpifaceSession->SelectedChannel - ((tr>>1));
 		*chnn=tr;
 	} else {
 		*firstchan=0;
-		*chnn=plNLChan;
+		*chnn=overrideplNLChan;
 	}
 }
 
@@ -165,83 +169,85 @@ static void setattrgrey(uint16_t *buf, int len)
 	}
 }
 
-static void preparetrack1(uint16_t *bp)
+static void preparetrack1 (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getnote(bp, 2);
+	getnote (cpifaceSession, bp, 2);
 }
 
-static void preparetrack2(uint16_t *bp)
+static void preparetrack2 (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getnote(bp, 1);
+	getnote (cpifaceSession, bp, 1);
 }
 
-static void preparetrack3(uint16_t *bp)
+static void preparetrack3 (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getnote(bp, 0);
+	getnote (cpifaceSession, bp, 0);
 }
 
-static void preparetrack3f(uint16_t *bp)
+static void preparetrack3f (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	if (!getnote(bp, 0))
-		getfx2(bp, 1, cpiTrkFXVol);
+	if (!getnote (cpifaceSession, bp, 0))
+	{
+		getfx2 (cpifaceSession, bp, 1, cpiTrkFXVol);
+	}
 }
 
 
-static void preparetrack6nf(uint16_t *bp)
+static void preparetrack6nf (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getnote(bp, 0);
-	getfx2(bp+3, 1, cpiTrkFXVol);
+	getnote (cpifaceSession, bp    , 0);
+	getfx2  (cpifaceSession, bp + 3, 1, cpiTrkFXVol);
 }
 
-static void preparetrack17invff(uint16_t *bp)
+static void preparetrack17invff (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getins(bp);
-	getnote(bp+3, 0);
-	getvol(bp+7);
-	getfx2(bp+10, 2, 0);
+	getins  (cpifaceSession, bp);
+	getnote (cpifaceSession, bp +  3, 0);
+	getvol  (cpifaceSession, bp +  7);
+	getfx2  (cpifaceSession, bp + 10, 2, 0);
 }
 
-static void preparetrack14invff(uint16_t *bp)
+static void preparetrack14invff (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getins(bp);
-	getnote(bp+2, 0);
-	getvol(bp+5);
-	getfx2(bp+7, 2, 0);
+	getins  (cpifaceSession, bp);
+	getnote (cpifaceSession, bp + 2, 0);
+	getvol  (cpifaceSession, bp + 5);
+	getfx2  (cpifaceSession, bp + 7, 2, 0);
 }
 
-static void preparetrack14nvff(uint16_t *bp)
+static void preparetrack14nvff (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getnote(bp, 0);
-	getvol(bp+4);
-	getfx2(bp+7, 2, 0);
+	getnote (cpifaceSession, bp    , 0);
+	getvol  (cpifaceSession, bp + 4);
+	getfx2  (cpifaceSession, bp + 7, 2, 0);
 }
 
-static void preparetrack26invpffff(uint16_t *bp)
+static void preparetrack26invpffff (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getins(bp);
-	getnote(bp+3, 0);
-	getvol(bp+7);
-	getpan(bp+10);
-	getfx2(bp+13, 4, cpiTrkFXNoPan);
+	getins  (cpifaceSession, bp);
+	getnote (cpifaceSession, bp +  3, 0);
+	getvol  (cpifaceSession, bp +  7);
+	getpan  (cpifaceSession, bp + 10);
+	getfx2  (cpifaceSession, bp + 13, 4, cpiTrkFXNoPan);
 }
 
-static void preparetrack16fffff(uint16_t *bp)
+static void preparetrack16fffff (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getfx2(bp, 5, cpiTrkFXIns|cpiTrkFXNote|cpiTrkFXVol);
+	getfx2 (cpifaceSession, bp, 5, cpiTrkFXIns|cpiTrkFXNote|cpiTrkFXVol);
 }
 
-static void preparetrack8nvf(uint16_t *bp)
+static void preparetrack8nvf (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getnote(bp, 0);
-	getvol(bp+3);
-	getfx2(bp+5, 1, 0);
+	getnote (cpifaceSession, bp    , 0);
+	getvol  (cpifaceSession, bp + 3);
+	getfx2  (cpifaceSession, bp + 5, 1, 0);
 }
 
-static void preparetrack8inf(uint16_t *bp)
+static void preparetrack8inf (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp)
 {
-	getins(bp);
-	getnote(bp+2, 0);
-	getfx2(bp+5, 1, cpiTrkFXVol);
+	getins  (cpifaceSession, bp);
+	getnote (cpifaceSession, bp + 2, 0);
+	getfx2  (cpifaceSession, bp + 5, 1, cpiTrkFXVol);
 }
 
 struct patviewtype
@@ -254,10 +260,10 @@ struct patviewtype
 	const char *paused;
 	const char *normal;
 	const char *selected;
-	void (*putcmd)(uint16_t *bp);
+	void (*putcmd)(struct cpifaceSessionAPI_t *cpifaceSession, uint16_t *bp);
 };
 
-static void preparepatgen(int pat, const struct patviewtype *pt)
+static void preparepatgen (struct cpifaceSessionAPI_t *cpifaceSession, int pat, const struct patviewtype *pt)
 {
 	int i;
 	int firstchan,chnn;
@@ -270,27 +276,17 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 
 	int maxch=(plPatWidth-pt->gcmd*4-3)/pt->width;
 
-	if (maxch>plNLChan)
-		maxch=plNLChan;
+	if (maxch>overrideplNLChan)
+		maxch=overrideplNLChan;
 
 	patpad=(plPatWidth-maxch*pt->width-pt->gcmd*4-3)>=4;
 
 	plPrepdPat=pat;
 
-	getscrollpos(maxch, &firstchan, &chnn);
+	getscrollpos (cpifaceSession, maxch, &firstchan, &chnn);
 
-	strcpy(pattitle1, "   pattern view:  order ");
-	convnum(pat, pattitle1+strlen(pattitle1), 16, 3, 0);
-	strcat(pattitle1, ", ");
-	convnum(maxch, pattitle1+strlen(pattitle1), 10, 2, 1);
-	strcat(pattitle1, " channels,  ");
-	strcat(pattitle1, pt->title);
-	pname=getpatname(pat);
-	if (pname&&*pname)
-	{
-		strcat(pattitle1, ": ");
-		strcat(pattitle1, pname);
-	}
+	pname = getpatname (cpifaceSession, pat);
+	snprintf (pattitle1, sizeof (pattitle1), "   pattern view:  order %03X, %2d channels,  %s%s%s", pat, maxch, pt->title, (pname&&*pname)?": ":"", (pname&&*pname)?pname:"");
 
 	p0=4+4*pt->gcmd;
 	patwidth=p0+pt->width*chnn+4;
@@ -302,16 +298,16 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 		case 0:
 			break;
 		case 1:
-			writestring(pattitle2, 4, plPause?COLTITLE2M:COLTITLE2, "gbl", 3);
+			writestring(pattitle2, 4, cpifaceSession->InPause?COLTITLE2M:COLTITLE2, "gbl", 3);
 			break;
 		case 2:
-			writestring(pattitle2, 5, plPause?COLTITLE2M:COLTITLE2, "global", 6);
+			writestring(pattitle2, 5, cpifaceSession->InPause?COLTITLE2M:COLTITLE2, "global", 6);
 			break;
 		case 3:
-			writestring(pattitle2, 5, plPause?COLTITLE2M:COLTITLE2, "global cmd", 10);
+			writestring(pattitle2, 5, cpifaceSession->InPause?COLTITLE2M:COLTITLE2, "global cmd", 10);
 			break;
 		default:
-			writestring(pattitle2, pt->gcmd*2-4, plPause?COLTITLE2M:COLTITLE2, "global commands", 15);
+			writestring(pattitle2, pt->gcmd*2-4, cpifaceSession->InPause?COLTITLE2M:COLTITLE2, "global commands", 15);
 			break;
 	}
 
@@ -321,7 +317,7 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 		writestring(patmask, patwidth-3, COLLNUM, "00", 2);
 	writestring(patmask, 3, COLBACK, "\xba", 1);
 	writestring(patmask, 3+pt->gcmd*4, COLBACK, "\xba", 1);
-	if (!plPause)
+	if (!cpifaceSession->InPause)
 		for (i=0; i<pt->gcmd; i++)
 			writestring(patmask, 4+4*i, COLBACK, "\xfa\xfa\xfa", 3);
 
@@ -330,8 +326,8 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 		n0--;
 	for (i=0; i<chnn; i++)
 	{
-		char chpaus=plMuteCh[i+firstchan];
-		char sel=((i+firstchan)==plSelCh);
+		uint8_t chpaus = cpifaceSession->MuteChannel[i+firstchan];
+		uint8_t sel=((i+firstchan)==cpifaceSession->SelectedChannel);
 		writenum(pattitle2, n0+pt->width*i, sel?COLTITLE2H:chpaus?COLTITLE2M:COLLNUM, i+firstchan+1, 10, (pt->width==1)?1:2, pt->width>2);
 		writestring(patmask, p0+pt->width*i, COLBACK, chpaus?pt->paused:sel?pt->selected:pt->normal, pt->width);
 	}
@@ -342,7 +338,7 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 	while (firstpat)
 	{
 		firstpat--;
-		firstrow-=getpatlen(firstpat);
+		firstrow -= getpatlen (cpifaceSession, firstpat);
 		if (firstrow<=0)
 		{
 			firstprow-=firstrow;
@@ -365,7 +361,7 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 		int curlen;
 		int lastprow;
 
-		if (!(curlen=getpatlen(firstpat)))
+		if (!(curlen = getpatlen (cpifaceSession, firstpat)))
 		{
 			/* skip zerolength patterns */
 			firstpat++;
@@ -373,7 +369,9 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 		}
 		lastprow=curlen;
 		if ((firstrow+curlen-firstprow)>=plPatBufH)
+		{
 			lastprow=plPatBufH-firstrow-firstprow-1;
+		}
 
 		for (i=firstprow; i<lastprow; i++)
 		{
@@ -385,17 +383,17 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 
 		if (pt->gcmd)
 		{
-			seektrack(firstpat, -1);
+			seektrack (cpifaceSession, firstpat, -1);
 			while (1)
 			{
-				int currow=startrow();
+				int currow = startrow (cpifaceSession);
 				if (currow==-1)
 					break;
 				if ((currow>=firstprow)&&(currow<lastprow))
 				{
 					uint16_t *bp=plPatBuf[currow+firstrow-firstprow]+4;
-					getgcmd(bp, pt->gcmd);
-					if (plPause)
+					getgcmd (cpifaceSession, bp, pt->gcmd);
+					if (cpifaceSession->InPause)
 						setattrgrey(bp, pt->gcmd*4);
 				}
 			}
@@ -404,17 +402,17 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 		for (i=0; i<chnn; i++)
 		{
 			int chpaus;
-			seektrack(firstpat, i+firstchan);
-			chpaus=plMuteCh[i+firstchan];
+			seektrack (cpifaceSession, firstpat, i+firstchan);
+			chpaus = cpifaceSession->MuteChannel[i+firstchan];
 			while (1)
 			{
-				int currow=startrow();
+				int currow = startrow (cpifaceSession);
 				if (currow==-1)
 					break;
 				if ((currow>=firstprow)&&(currow<lastprow))
 				{
 					uint16_t *bp=plPatBuf[currow+firstrow-firstprow]+p0+i*pt->width;
-					pt->putcmd(bp);
+					pt->putcmd (cpifaceSession, bp);
 					if (chpaus)
 						setattrgrey(bp, pt->width);
 				}
@@ -425,8 +423,10 @@ static void preparepatgen(int pat, const struct patviewtype *pt)
 		firstprow=0;
 		firstpat++;
 
-		if (firstrow>=plPatBufH)
+		if (firstrow >= (plPatBufH-1))
+		{
 			break;
+		}
 	}
 	for (i=firstrow; i<plPatBufH; i++)
 		writestring(plPatBuf[i], 0, COLBACK, "", CONSOLE_MAX_X);
@@ -543,7 +543,7 @@ static void calcPatType(void)
 	for (i=0; i<6 /* the sixth entry we do not care to probe, since it will be the default */; i++)
 	{
 		int maxch=(plPatWidth-Probes[i].ref->gcmd*4-3)/Probes[i].ref->width;
-		if (maxch >= plNLChan) /* all channels can fit in this mode */
+		if (maxch >= overrideplNLChan) /* all channels can fit in this mode */
 		{
 			break;
 		}
@@ -552,10 +552,9 @@ static void calcPatType(void)
 	plPatType=Probes[i].PatType;
 }
 
-
-static void gmdDrawPattern(int sel)
+static void TrakDraw (struct cpifaceSessionAPI_t *cpifaceSession, int focus)
 {
-	int pos=getcurpos();
+	int pos = getcurpos (cpifaceSession);
 	int pat=pos>>8;
 	int currow=pos&0xFF;
 	int curpat=pat;
@@ -573,7 +572,7 @@ static void gmdDrawPattern(int sel)
 		pat=plPatManualPat;
 		crow=plPatManualRow;
 	}
-	while (!getpatlen(pat))
+	while (!getpatlen (cpifaceSession, pat))
 	{
 		pat++;
 		crow=0;
@@ -581,50 +580,50 @@ static void gmdDrawPattern(int sel)
 			pat=0;
 	}
 
-	if ((plPrepdPat!=pat)||plChanChanged)
+	if ((plPrepdPat!=pat) || cpifaceSession->SelectedChannelChanged)
 	{
 		if (plPatWidth<128)
 			switch (plPatType)
 			{
-				case 0: preparepatgen(pat, &pat6480); break;
-				case 1: preparepatgen(pat, &pat6480); break;
-				case 2: preparepatgen(pat, &pat4880); break;
-				case 3: preparepatgen(pat, &pat4880); break;
-				case 4: preparepatgen(pat, &pat3280); break;
-				case 5: preparepatgen(pat, &pat3280); break;
-				case 6: preparepatgen(pat, &pat2480); break;
-				case 7: preparepatgen(pat, &pat2480f); break;
-				case 8: preparepatgen(pat, &pat1680); break;
-				case 9: preparepatgen(pat, &pat1680f); break;
-				case 10: preparepatgen(pat, &pat880); break;
-				case 11: preparepatgen(pat, &pat880f); break;
-				case 12: preparepatgen(pat, &pat480f); break;
-				case 13: preparepatgen(pat, &pat480); break;
+				case 0: preparepatgen (cpifaceSession, pat, &pat6480); break;
+				case 1: preparepatgen (cpifaceSession, pat, &pat6480); break;
+				case 2: preparepatgen (cpifaceSession, pat, &pat4880); break;
+				case 3: preparepatgen (cpifaceSession, pat, &pat4880); break;
+				case 4: preparepatgen (cpifaceSession, pat, &pat3280); break;
+				case 5: preparepatgen (cpifaceSession, pat, &pat3280); break;
+				case 6: preparepatgen (cpifaceSession, pat, &pat2480); break;
+				case 7: preparepatgen (cpifaceSession, pat, &pat2480f); break;
+				case 8: preparepatgen (cpifaceSession, pat, &pat1680); break;
+				case 9: preparepatgen (cpifaceSession, pat, &pat1680f); break;
+				case 10: preparepatgen (cpifaceSession, pat, &pat880); break;
+				case 11: preparepatgen (cpifaceSession, pat, &pat880f); break;
+				case 12: preparepatgen (cpifaceSession, pat, &pat480f); break;
+				case 13: preparepatgen (cpifaceSession, pat, &pat480); break;
 			} else switch (plPatType)
 			{
-				case 0: preparepatgen(pat, &pat64132m); break;
-				case 1: preparepatgen(pat, &pat64132); break;
-				case 2: preparepatgen(pat, &pat48132); break;
-				case 3: preparepatgen(pat, &pat48132); break;
-				case 4: preparepatgen(pat, &pat32132); break;
-				case 5: preparepatgen(pat, &pat32132f); break;
-				case 6: preparepatgen(pat, &pat24132); break;
-				case 7: preparepatgen(pat, &pat24132f); break;
-				case 8: preparepatgen(pat, &pat16132); break;
-				case 9: preparepatgen(pat, &pat16132); break;
-				case 10: preparepatgen(pat, &pat8132f); break;
-				case 11: preparepatgen(pat, &pat8132); break;
-				case 12: preparepatgen(pat, &pat4132); break;
-				case 13: preparepatgen(pat, &pat4132); break;
+				case 0: preparepatgen (cpifaceSession, pat, &pat64132m); break;
+				case 1: preparepatgen (cpifaceSession, pat, &pat64132); break;
+				case 2: preparepatgen (cpifaceSession, pat, &pat48132); break;
+				case 3: preparepatgen (cpifaceSession, pat, &pat48132); break;
+				case 4: preparepatgen (cpifaceSession, pat, &pat32132); break;
+				case 5: preparepatgen (cpifaceSession, pat, &pat32132f); break;
+				case 6: preparepatgen (cpifaceSession, pat, &pat24132); break;
+				case 7: preparepatgen (cpifaceSession, pat, &pat24132f); break;
+				case 8: preparepatgen (cpifaceSession, pat, &pat16132); break;
+				case 9: preparepatgen (cpifaceSession, pat, &pat16132); break;
+				case 10: preparepatgen (cpifaceSession, pat, &pat8132f); break;
+				case 11: preparepatgen (cpifaceSession, pat, &pat8132); break;
+				case 12: preparepatgen (cpifaceSession, pat, &pat4132); break;
+				case 13: preparepatgen (cpifaceSession, pat, &pat4132); break;
 			}
 	}
 
 	/* show the two header lines*/
-	displaystr(plPatFirstLine-2, 0, sel?COLTITLE1H:COLTITLE1, pattitle1, plPatWidth);
+	displaystr(plPatFirstLine-2, 0, focus?COLTITLE1H:COLTITLE1, pattitle1, plPatWidth);
 	displaystrattr(plPatFirstLine-1, 0, pattitle2, plPatWidth);
 	/* done */
 
-	/* plen=getpatlen(pat); */
+	/* plen = getpatlen(cpifaceSession, pat); */
 	for (i=0, row=crow-min(plPatHeight/3, 20); i<plPatHeight; i++, row++)
 	{
 		/* if no manual position, and we are not at the current position, display the buffer normal */
@@ -653,7 +652,7 @@ static void gmdDrawPattern(int sel)
 	}
 }
 
-static int gmdTrkProcessKey(uint16_t key)
+static int gmdTrkProcessKey (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t key)
 {
 	if (plPatType < 0)
 	{ /* more an assertion... should not be able to get so far without this beeing calculated. Usually we would draw atleast one frame before processing keyboard presses */
@@ -684,10 +683,14 @@ static int gmdTrkProcessKey(uint16_t key)
 				{
 					plPatManualPat--;
 					if (plPatManualPat<0)
+					{
 						plPatManualPat=plPatternNum-1;
-					while (!getpatlen(plPatManualPat))
+					}
+					while (!getpatlen (cpifaceSession, plPatManualPat))
+					{
 						plPatManualPat--;
-					plPatManualRow=getpatlen(plPatManualPat)-1;
+					}
+					plPatManualRow = getpatlen (cpifaceSession, plPatManualPat)-1;
 				}
 			}
 			break;
@@ -701,13 +704,17 @@ static int gmdTrkProcessKey(uint16_t key)
 				}
 			} else {
 				plPatManualRow+=8;
-				if (plPatManualRow>=getpatlen(plPatManualPat))
+				if (plPatManualRow >= getpatlen (cpifaceSession, plPatManualPat))
 				{
 					plPatManualPat++;
-					while ((plPatManualPat<plPatternNum)&&!getpatlen(plPatManualPat))
+					while ((plPatManualPat < plPatternNum) && !getpatlen(cpifaceSession, plPatManualPat))
+					{
 						plPatManualPat++;
+					}
 					if (plPatManualPat>=plPatternNum)
+					{
 						plPatManualPat=0;
+					}
 					plPatManualRow=0;
 				}
 			}
@@ -757,7 +764,7 @@ static int gmdTrkProcessKey(uint16_t key)
 		case ' ':
 			if (plPatManualPat==-1)
 			{
-				int p=getcurpos();
+				int p = getcurpos (cpifaceSession);
 				plPatManualPat=p>>8;
 				plPatManualRow=p&0xFF;
 			} else
@@ -769,14 +776,14 @@ static int gmdTrkProcessKey(uint16_t key)
 	return 1;
 }
 
-static void TrakSetWin(int _ignore, int wid, int ypos, int hgt)
+static void TrakSetWin (struct cpifaceSessionAPI_t *cpifaceSession, int _ignore, int wid, int ypos, int hgt)
 {
 	plPatFirstLine=ypos+2;
 	plPatHeight=hgt-2;
 	plPatWidth=wid;
 }
 
-static int TrakGetWin(struct cpitextmodequerystruct *q)
+static int TrakGetWin (struct cpifaceSessionAPI_t *cpifaceSession, struct cpitextmodequerystruct *q)
 {
 	if (!plTrackActive)
 		return 0;
@@ -791,12 +798,7 @@ static int TrakGetWin(struct cpitextmodequerystruct *q)
 	return 1;
 }
 
-static void TrakDraw(int focus)
-{
-	gmdDrawPattern(focus);
-}
-
-static int TrakIProcessKey(uint16_t key)
+static int TrakIProcessKey (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t key)
 {
 	switch (key)
 	{
@@ -806,7 +808,7 @@ static int TrakIProcessKey(uint16_t key)
 			break;
 		case 't': case 'T':
 			plTrackActive=1;
-			cpiTextSetMode("trak");
+			cpiTextSetMode (cpifaceSession, "trak");
 			calcPatType();
 			return 1;
 		case 'x': case 'X':
@@ -820,29 +822,26 @@ static int TrakIProcessKey(uint16_t key)
 	return 0;
 }
 
-static int TrakAProcessKey(uint16_t key)
+static int TrakAProcessKey (struct cpifaceSessionAPI_t *cpifaceSession, uint16_t key)
 {
 	switch (key)
 	{
 		case 't': case 'T':
 			plTrackActive=!plTrackActive;
-			cpiTextRecalc();
+			cpiTextRecalc (cpifaceSession);
 			break;
 		default:
-			return gmdTrkProcessKey(key);
+			return gmdTrkProcessKey (cpifaceSession, key);
 	}
 	return 1;
 }
 
-static int trkEvent(int ev)
+static int trkEvent (struct cpifaceSessionAPI_t *cpifaceSession, int ev)
 {
 	switch (ev)
 	{
-		case cpievInitAll:
-			plTrackActive=cfGetProfileBool2(cfScreenSec, "screen", "pattern", 1, 1);
-			return 0;
 		case cpievInit:
-			plPatBuf=calloc(sizeof(short), (plPatBufH/*don't ask where we get these from */)*CONSOLE_MAX_X);
+			plPatBuf=calloc(sizeof(uint16_t), (plPatBufH/*don't ask where we get these from */)*CONSOLE_MAX_X);
 			if (!plPatBuf)
 				return 0;
 			break;
@@ -856,32 +855,49 @@ static int trkEvent(int ev)
 
 static struct cpitextmoderegstruct cpiTModeTrack = {"trak", TrakGetWin, TrakSetWin, TrakDraw, TrakIProcessKey, TrakAProcessKey, trkEvent CPITEXTMODEREGSTRUCT_TAIL};
 
-void cpiTrkSetup(const struct cpitrakdisplaystruct *c, int npat)
+void cpiTrkSetup (struct cpifaceSessionAPI_t *cpifaceSession, const struct cpitrakdisplaystruct *c, int npat)
 {
-	plPatternNum=npat;
-	plPatManualPat=-1;
-	plPrepdPat=-1;
-	plPatType=-1;
-	getcurpos=c->getcurpos;
-	getpatlen=c->getpatlen;
-	getpatname=c->getpatname;
-	seektrack=c->seektrack;
-	startrow=c->startrow;
-	getnote=c->getnote;
-	getins=c->getins;
-	getvol=c->getvol;
-	getpan=c->getpan;
-	getfx=c->getfx;
-	getgcmd=c->getgcmd;
-	cpiTextRegisterMode(&cpiTModeTrack);
+	overrideplNLChan = cpifaceSession->LogicalChannelCount;
+	plPatternNum     = npat;
+	plPatManualPat   = -1;
+	plPrepdPat       = -1;
+	plPatType        = -1;
+	getcurpos        = c->getcurpos;
+	getpatlen        = c->getpatlen;
+	getpatname       = c->getpatname;
+	seektrack        = c->seektrack;
+	startrow         = c->startrow;
+	getnote          = c->getnote;
+	getins           = c->getins;
+	getvol           = c->getvol;
+	getpan           = c->getpan;
+	getfx            = c->getfx;
+	getgcmd          = c->getgcmd;
+	cpiTextRegisterMode (cpifaceSession, &cpiTModeTrack);
 }
 
-static void __attribute__((constructor))init(void)
+void cpiTrkSetup2 (struct cpifaceSessionAPI_t *cpifaceSession, const struct cpitrakdisplaystruct *c, int npat, int tracks)
 {
-	cpiTextRegisterDefMode(&cpiTModeTrack);
+	overrideplNLChan = tracks;
+	plPatternNum     = npat;
+	plPatManualPat   = -1;
+	plPrepdPat       = -1;
+	plPatType        = -1;
+	getcurpos        = c->getcurpos;
+	getpatlen        = c->getpatlen;
+	getpatname       = c->getpatname;
+	seektrack        = c->seektrack;
+	startrow         = c->startrow;
+	getnote          = c->getnote;
+	getins           = c->getins;
+	getvol           = c->getvol;
+	getpan           = c->getpan;
+	getfx            = c->getfx;
+	getgcmd          = c->getgcmd;
+	cpiTextRegisterMode (cpifaceSession, &cpiTModeTrack);
 }
 
-static void __attribute__((destructor))done(void)
+OCP_INTERNAL void cpiTrackInit (void)
 {
-	cpiTextUnregisterDefMode(&cpiTModeTrack);
+	plTrackActive=cfGetProfileBool2(cfScreenSec, "screen", "pattern", 1, 1);
 }
