@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2016-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2016-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Generic ringbuffer pointer handler
  *
@@ -20,8 +20,10 @@
 
 #include "config.h"
 #include <assert.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "types.h"
 #include "ringbuffer.h"
 
 struct ringbuffer_callback_hook_t
@@ -53,6 +55,13 @@ struct ringbuffer_t
 	struct ringbuffer_callback_hook_t *processing_callbacks;
 	int processing_callbacks_size;
 	int processing_callbacks_fill;
+
+	/*     |TAIL      |PROC            |HEAD
+         *     |----nonpause---|pause--|
+	 *     we first eat up the nonpause data, and then eat up pause-data. pause data causes the total tail NOT to count */
+	int      pause_fill;
+	int      nonpause_fill;
+	uint64_t total_tail; /* total of non-pause samples we have played so far in the current session */
 };
 
 void ringbuffer_reset (struct ringbuffer_t *self)
@@ -78,9 +87,13 @@ void ringbuffer_reset (struct ringbuffer_t *self)
 		self->tail_callbacks[i].callback (self->tail_callbacks[i].arg, 1 - self->tail_callbacks[i].samples);
 	}
 	self->tail_callbacks_fill = 0;
+
+	self->total_tail = 0;
+	self->pause_fill = 0;
+	self->nonpause_fill = 0;
 }
 
-void ringbuffer_static_initialize (struct ringbuffer_t *self, int flags, int buffersize_samples)
+static void ringbuffer_static_initialize (struct ringbuffer_t *self, int flags, int buffersize_samples)
 {
 	self->flags = flags;
 	self->cache_sample_shift = 0;
@@ -139,6 +152,31 @@ void ringbuffer_free(struct ringbuffer_t *self)
 void ringbuffer_tail_consume_samples(struct ringbuffer_t *self, int samples)
 {
 	assert (samples <= self->cache_read_available);
+
+	if (self->pause_fill)
+	{
+		int samples2 = samples;
+		if (self->nonpause_fill >= samples2)
+		{
+			self->nonpause_fill -= samples2;
+			self->total_tail += samples2;
+		} else {
+			self->total_tail += self->nonpause_fill;
+			samples2 -= self->nonpause_fill;
+			self->nonpause_fill = 0;
+
+			if (self->pause_fill >= samples2)
+			{
+				self->pause_fill -= samples2;
+			} else {
+				samples2 -= self->pause_fill;
+				self->total_tail += samples2;
+			}
+		}
+	} else {
+		self->total_tail += samples;
+	}
+
 	self->tail = (self->tail + samples) % self->buffersize;
 
 	self->cache_read_available -= samples;
@@ -167,14 +205,6 @@ void ringbuffer_tail_consume_samples(struct ringbuffer_t *self, int samples)
 	assert ((self->cache_read_available + self->cache_write_available + self->cache_processing_available + 1) == self->buffersize);
 }
 
-void ringbuffer_tail_set_samples(struct ringbuffer_t *self, int pos)
-{
-	int samples = (self->buffersize + pos - self->tail) % self->buffersize;
-
-	ringbuffer_tail_consume_samples (self, samples);
-}
-
-
 void ringbuffer_processing_consume_samples(struct ringbuffer_t *self, int samples)
 {
 	assert (self->flags & RINGBUFFER_FLAGS_PROCESS);
@@ -208,14 +238,7 @@ void ringbuffer_processing_consume_samples(struct ringbuffer_t *self, int sample
 	assert ((self->cache_read_available + self->cache_write_available + self->cache_processing_available + 1) == self->buffersize);
 }
 
-void ringbuffer_processing_set_samples(struct ringbuffer_t *self, int pos)
-{
-	int samples = (self->buffersize + pos - self->processing) % self->buffersize;
-
-	ringbuffer_processing_consume_samples (self, samples);
-}
-
-void ringbuffer_head_add_samples(struct ringbuffer_t *self, int samples)
+static void ringbuffer_head_add_samples_common (struct ringbuffer_t *self, int samples)
 {
 	assert (samples <= self->cache_write_available);
 
@@ -233,11 +256,21 @@ void ringbuffer_head_add_samples(struct ringbuffer_t *self, int samples)
 	assert ((self->cache_read_available + self->cache_write_available + self->cache_processing_available + 1) == self->buffersize);
 }
 
-void ringbuffer_head_set_samples(struct ringbuffer_t *self, int pos)
+void ringbuffer_head_add_samples(struct ringbuffer_t *self, int samples)
 {
-	int samples = (self->buffersize + pos - self->head) % self->buffersize;
+	ringbuffer_head_add_samples_common (self, samples);
+}
 
-	ringbuffer_head_add_samples (self, samples);
+void ringbuffer_head_add_pause_samples(struct ringbuffer_t *self, int samples)
+{
+	ringbuffer_head_add_samples_common (self, samples);
+
+	/* move the pause head up until HEAD, so if there were any earlier samples, they now join up into one chunk
+ */
+	self->pause_fill += samples;
+	self->nonpause_fill = self->cache_read_available + self->cache_processing_available - self->pause_fill;
+
+	assert ((self->pause_fill + self->nonpause_fill) <= self->buffersize);
 }
 
 int ringbuffer_get_tail_available_samples (struct ringbuffer_t *self)
@@ -271,8 +304,14 @@ void ringbuffer_get_tail_samples (struct ringbuffer_t *self, int *pos1, int *len
 
 	*length1 = self->buffersize - self->tail;
 
-	*pos2 = 0;
-	*length2 = self->cache_read_available - *length1;
+	if (pos2)
+	{
+		*pos2 = 0;
+	}
+	if (length2)
+	{
+		*length2 = self->cache_read_available - *length1;
+	}
 
 	return;
 
@@ -280,8 +319,14 @@ clear1:
 	*pos1 = -1;
 	*length1 = 0;
 clear2:
-	*pos2 = -1;
-	*length2 = 0;
+	if (pos2)
+	{
+		*pos2 = -1;
+	}
+	if (length2)
+	{
+		*length2 = 0;
+	}
 }
 
 void ringbuffer_get_processing_samples (struct ringbuffer_t *self, int *pos1, int *length1, int *pos2, int *length2)
@@ -302,8 +347,14 @@ void ringbuffer_get_processing_samples (struct ringbuffer_t *self, int *pos1, in
 
 	*length1 = self->buffersize - self->processing;
 
-	*pos2 = 0;
-	*length2 = self->cache_processing_available - *length1;
+	if (pos2)
+	{
+		*pos2 = 0;
+	}
+	if (length2)
+	{
+		*length2 = self->cache_processing_available - *length1;
+	}
 
 	return;
 
@@ -311,8 +362,58 @@ clear1:
 	*pos1 = -1;
 	*length1 = 0;
 clear2:
-	*pos2 = -1;
-	*length2 = 0;
+	if (pos2)
+	{
+		*pos2 = -1;
+	}
+	if (length2)
+	{
+		*length2 = 0;
+	}
+}
+
+void ringbuffer_get_tailandprocessing_samples (struct ringbuffer_t *self, int *pos1, int *length1, int *pos2, int *length2)
+{
+	int temp = self->cache_read_available + self->cache_processing_available;
+	assert (self->flags & RINGBUFFER_FLAGS_PROCESS);
+
+	if (!temp)
+	{
+		goto clear1;
+	}
+
+	*pos1 = self->tail;
+	if ((self->tail + temp) <= self->buffersize)
+	{
+		*length1 = temp;
+		goto clear2;
+	}
+
+	*length1 = self->buffersize - self->tail;
+
+	if (pos2)
+	{
+		*pos2 = 0;
+	}
+	if (length2)
+	{
+		*length2 = temp - *length1;
+	}
+
+	return;
+
+clear1:
+	*pos1 = -1;
+	*length1 = 0;
+clear2:
+	if (pos2)
+	{
+		*pos2 = -1;
+	}
+	if (length2)
+	{
+		*length2 = 0;
+	}
 }
 
 void ringbuffer_get_head_samples (struct ringbuffer_t *self, int *pos1, int *length1, int *pos2, int *length2)
@@ -331,8 +432,14 @@ void ringbuffer_get_head_samples (struct ringbuffer_t *self, int *pos1, int *len
 
 	*length1 = self->buffersize - self->head;
 
-	*pos2 = 0;
-	*length2 = self->cache_write_available - *length1;
+	if (pos2)
+	{
+		*pos2 = 0;
+	}
+	if (length2)
+	{
+		*length2 = self->cache_write_available - *length1;
+	}
 
 	return;
 
@@ -340,8 +447,14 @@ clear1:
 	*pos1 = -1;
 	*length1 = 0;
 clear2:
-	*pos2 = -1;
-	*length2 = 0;
+	if (pos2)
+	{
+		*pos2 = -1;
+	}
+	if (length2)
+	{
+		*length2 = 0;
+	}
 }
 
 void ringbuffer_tail_consume_bytes(struct ringbuffer_t *self, int bytes)
@@ -349,31 +462,22 @@ void ringbuffer_tail_consume_bytes(struct ringbuffer_t *self, int bytes)
 	ringbuffer_tail_consume_samples (self, bytes >> self->cache_sample_shift);
 }
 
-void ringbuffer_tail_set_bytes(struct ringbuffer_t *self, int pos)
-{
-	ringbuffer_tail_set_samples (self, pos >> self->cache_sample_shift);
-}
-
 void ringbuffer_processing_consume_bytes(struct ringbuffer_t *self, int bytes)
 {
 	ringbuffer_processing_consume_samples (self, bytes >> self->cache_sample_shift);
 }
 
-void ringbuffer_processing_set_bytes(struct ringbuffer_t *self, int pos)
-{
-	ringbuffer_processing_set_samples (self, pos >> self->cache_sample_shift);
-}
-
 void ringbuffer_head_add_bytes(struct ringbuffer_t *self, int bytes)
 {
 	ringbuffer_head_add_samples (self, bytes >> self->cache_sample_shift);
 }
 
-void ringbuffer_head_set_bytes(struct ringbuffer_t *self, int pos)
+void ringbuffer_head_add_pause_bytes(struct ringbuffer_t *self, int bytes)
 {
-	ringbuffer_head_set_samples (self, pos>>self->cache_sample_shift);
+	ringbuffer_head_add_pause_samples (self, bytes >> self->cache_sample_shift);
 }
 
+
 void ringbuffer_get_tail_bytes (struct ringbuffer_t *self, int *pos1, int *length1, int *pos2, int *length2)
 {
 	ringbuffer_get_tail_samples (self, pos1, length1, pos2, length2);
@@ -382,9 +486,12 @@ void ringbuffer_get_tail_bytes (struct ringbuffer_t *self, int *pos1, int *lengt
 		*pos1 <<= self->cache_sample_shift;
 	}
 
-	if ((*length2 <<= self->cache_sample_shift))
+	if (length2)
 	{
-		*pos2 <<= self->cache_sample_shift;
+		if ((*length2 <<= self->cache_sample_shift))
+		{
+			*pos2 <<= self->cache_sample_shift;
+		}
 	}
 }
 
@@ -396,9 +503,12 @@ void ringbuffer_get_processing_bytes (struct ringbuffer_t *self, int *pos1, int
 		*pos1 <<= self->cache_sample_shift;
 	}
 
-	if ((*length2 <<= self->cache_sample_shift))
+	if (length2)
 	{
-		*pos2 <<= self->cache_sample_shift;
+		if ((*length2 <<= self->cache_sample_shift))
+		{
+			*pos2 <<= self->cache_sample_shift;
+		}
 	}
 }
 
@@ -410,9 +520,12 @@ void ringbuffer_get_head_bytes (struct ringbuffer_t *self, int *pos1, int *lengt
 		*pos1 <<= self->cache_sample_shift;
 	}
 
-	if ((*length2 <<= self->cache_sample_shift))
+	if (length2)
 	{
-		*pos2 <<= self->cache_sample_shift;
+		if ((*length2 <<= self->cache_sample_shift))
+		{
+			*pos2 <<= self->cache_sample_shift;
+		}
 	}
 }
 
@@ -438,10 +551,10 @@ int ringbuffer_get_head_available_bytes (struct ringbuffer_t *self)
 void ringbuffer_add_tail_callback_samples (struct ringbuffer_t *self, int samples, void (*callback)(void *arg, int samples_ago), const void *arg)
 {
 	int insertat, i;
-	if (samples < 0)
+/*	if (samples < 0)
 	{
 		samples = 0;
-	} else if (samples > (self->cache_read_available + self->cache_processing_available))
+	} else*/ if (samples > (self->cache_read_available + self->cache_processing_available))
 	{
 		samples = self->cache_read_available + self->cache_processing_available;
 	}
@@ -478,10 +591,10 @@ void ringbuffer_add_processing_callback_samples (struct ringbuffer_t *self, int
 		return;
 	}
 
-	if (samples < 0)
+/*	if (samples < 0)
 	{
 		samples = 0;
-	} else if (samples > (self->cache_read_available))
+	} else*/ if (samples > (self->cache_read_available))
 	{
 		samples = self->cache_read_available;
 	}
@@ -508,6 +621,43 @@ void ringbuffer_add_processing_callback_samples (struct ringbuffer_t *self, int
 	self->processing_callbacks_fill++;
 }
 
+void ringbuffer_get_stats (struct ringbuffer_t *self, uint64_t *total_tail)
+{
+	*total_tail = self->total_tail;
+}
+
+const struct ringbufferAPI_t ringbufferAPI =
+{
+	ringbuffer_reset,
+	ringbuffer_tail_consume_bytes,
+	ringbuffer_processing_consume_bytes,
+	ringbuffer_head_add_bytes,
+	ringbuffer_head_add_pause_bytes,
+	ringbuffer_tail_consume_samples,
+	ringbuffer_processing_consume_samples,
+	ringbuffer_head_add_samples,
+	ringbuffer_head_add_pause_samples,
+	ringbuffer_get_tail_bytes,
+	ringbuffer_get_processing_bytes,
+	ringbuffer_get_head_bytes,
+	ringbuffer_get_tail_samples,
+	ringbuffer_get_processing_samples,
+	ringbuffer_get_tailandprocessing_samples,
+	ringbuffer_get_head_samples,
+	ringbuffer_get_tail_available_bytes,
+	ringbuffer_get_processing_available_bytes,
+	ringbuffer_get_head_available_bytes,
+	ringbuffer_get_tail_available_samples,
+	ringbuffer_get_processing_available_samples,
+	ringbuffer_get_head_available_samples,
+	ringbuffer_new_samples,
+	ringbuffer_free,
+	ringbuffer_add_tail_callback_samples,
+	ringbuffer_add_processing_callback_samples,
+	ringbuffer_get_stats
+};
+
+
 
 /* UNIT TEST */
 #ifdef UNIT_TEST
@@ -954,49 +1104,6 @@ int main(int argc, char *argv[])
 	ringbuffer_free (instance);
 	retval += testval;
 
-	printf ("NO PROCESSING, SAMPLES (set)\n");
-	testval = 0;
-	instance = ringbuffer_new_samples(RINGBUFFER_FLAGS_FLOAT | RINGBUFFER_FLAGS_STEREO, 16);
-	testval += ringbuffer_result_nonproc_samples (instance, -1,  0, -1,  0,  /* tail */
-	                                                         0, 15, -1,  0); /* head */
-	/* add 1 sample, into buffer */
-	printf ("Adding 1 sample\n");
-	ringbuffer_head_set_samples(instance, 1);
-	testval += ringbuffer_result_nonproc_samples (instance,  0,  1, -1,  0,  /* tail */
-	                                                         1, 14, -1,  0); /* head */
-	/* add 14 sample, into buffer */
-	printf ("Adding 14 samples\n");
-	ringbuffer_head_set_samples(instance, 15);
-	testval += ringbuffer_result_nonproc_samples (instance,  0, 15, -1,  0,  /* tail */
-	                                                        -1,  0, -1,  0); /* head */
-	/* Remove 10 samples */
-	printf ("Consuming 10 samples\n");
-	ringbuffer_tail_set_samples(instance, 10);
-	testval += ringbuffer_result_nonproc_samples (instance, 10,  5, -1,  0,  /* tail */
-	                                                        15,  1,  0,  9); /* head */
-	/* add 10 sample, into buffer */
-	printf ("Adding 10 samples (head mid-buffer wraps)\n");
-	ringbuffer_head_set_samples(instance, 9);
-	testval += ringbuffer_result_nonproc_samples (instance, 10,  6,  0,  9,  /* tail */
-	                                                        -1,  0, -1,  0); /* head */
-	/* Remove 15 samples */
-	printf ("Consuming 15 samples (remove all, with mid-wrap)\n");
-	ringbuffer_tail_set_samples(instance, 9);
-	testval += ringbuffer_result_nonproc_samples (instance, -1,  0, -1,  0,  /* tail */
-	                                                         9,  7,  0,  8); /* head */
-	/* add 7 samples, into buffer */
-	printf ("Adding 7 samples (head lands on just after wrap)\n");
-	ringbuffer_head_set_samples(instance, 0);
-	testval += ringbuffer_result_nonproc_samples (instance,  9,  7, -1,  0,  /* tail */
-	                                                         0,  8, -1,  0); /* head */
-	/* Remove 7 samples */
-	printf ("Consuming 7 samples (remove all, tail lands just after wrap)\n");
-	ringbuffer_tail_set_samples(instance, 0);
-	testval += ringbuffer_result_nonproc_samples (instance, -1,  0, -1,  0,  /* tail */
-	                                                         0, 15, -1,  0); /* head */
-	ringbuffer_free (instance);
-	retval += testval;
-
 	printf ("PROCESSING, SAMPLES (progressive)\n");
 	testval = 0;
 	instance = ringbuffer_new_samples(RINGBUFFER_FLAGS_FLOAT | RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_PROCESS, 16);
@@ -1066,75 +1173,6 @@ int main(int argc, char *argv[])
 	ringbuffer_free (instance);
 	retval += testval;
 
-	printf ("PROCESSING, SAMPLES (set)\n");
-	testval = 0;
-	instance = ringbuffer_new_samples(RINGBUFFER_FLAGS_FLOAT | RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_PROCESS, 16);
-	testval += ringbuffer_result_proc_samples (instance, -1,  0, -1,  0,  /* tail */
-	                                                     -1,  0, -1,  0,  /* proc */
-	                                                      0, 15, -1,  0); /* head */
-	/* add 1 sample, into buffer */
-	printf ("Adding 1 sample\n");
-	ringbuffer_head_set_samples(instance, 1);
-	testval += ringbuffer_result_proc_samples (instance, -1,  0, -1,  0,  /* tail */
-                                                              0,  1, -1,  0,  /* proc */
-	                                                      1, 14, -1,  0); /* head */
-	/* add 14 sample, into buffer */
-	printf ("Adding 14 samples\n");
-	ringbuffer_head_set_samples(instance, 15);
-	testval += ringbuffer_result_proc_samples (instance, -1,  0, -1,  0,  /* tail */
-                                                              0, 15, -1,  0,  /* proc */
-	                                                     -1,  0, -1,  0); /* head */
-	/* Process 14 samples */
-	printf ("Process 14 samples\n");
-	ringbuffer_processing_set_samples(instance, 14);
-	testval += ringbuffer_result_proc_samples (instance,  0, 14, -1,  0,  /* tail */
-	                                                     14,  1, -1,  0,  /* proc */
-	                                                     -1,  0, -1,  0); /* head */
-	/* Remove 10 samples */
-	printf ("Consuming 10 samples\n");
-	ringbuffer_tail_set_samples(instance, 10);
-	testval += ringbuffer_result_proc_samples (instance, 10,  4, -1,  0,  /* tail */
-	                                                     14,  1, -1,  0,  /* proc */
-	                                                     15,  1,  0,  9); /* head */
-	/* add 10 sample, into buffer */
-	printf ("Adding 10 samples (head mid-buffer wraps)\n");
-	ringbuffer_head_set_samples(instance, 9);
-	testval += ringbuffer_result_proc_samples (instance, 10,  4, -1,  0,  /* tail */
-	                                                     14,  2,  0,  9,  /* proc */
-	                                                     -1,  0, -1,  0); /* head */
-	/* Process 11 samples */
-	printf ("Process 11 samples (head mid-buffer wraps)\n");
-	ringbuffer_processing_set_samples(instance, 9);
-	testval += ringbuffer_result_proc_samples (instance, 10,  6,  0,  9,  /* tail */
-	                                                     -1,  0, -1,  0,  /* proc */
-	                                                     -1,  0, -1,  0); /* head */
-	/* Remove 15 samples */
-	printf ("Consuming 15 samples (remove all, with mid-wrap)\n");
-	ringbuffer_tail_set_samples(instance, 9);
-	testval += ringbuffer_result_proc_samples (instance, -1,  0, -1,  0,  /* tail */
-	                                                     -1,  0, -1,  0,  /* proc */
-	                                                      9,  7,  0,  8); /* head */
-	/* add 7 samples, into buffer */
-	printf ("Adding 7 samples (head lands on just after wrap)\n");
-	ringbuffer_head_set_samples(instance, 0);
-	testval += ringbuffer_result_proc_samples (instance, -1,  0, -1,  0, /* tail */
-	                                                      9,  7, -1,  0,  /* tail */
-	                                                      0,  8, -1,  0); /* head */
-	/* Process 7 samples */
-	printf ("Process 7 samples (head lands on just after wrap)\n");
-	ringbuffer_processing_set_samples(instance, 0);
-	testval += ringbuffer_result_proc_samples (instance,  9,  7, -1,  0,  /* tail */
-	                                                     -1,  0, -1,  0,  /* proc */
-	                                                      0,  8, -1,  0); /* head */
-	/* Remove 7 samples */
-	printf ("Consuming 7 samples (remove all, tail lands just after wrap)\n");
-	ringbuffer_tail_set_samples(instance, 0);
-	testval += ringbuffer_result_proc_samples (instance, -1,  0, -1,  0,  /* tail */
-	                                                     -1,  0, -1,  0,  /* proc */
-	                                                      0, 15, -1,  0); /* head */
-	ringbuffer_free (instance);
-	retval += testval;
-
 	printf ("Going to test callbacks\n");
 	instance = ringbuffer_new_samples(RINGBUFFER_FLAGS_FLOAT | RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_PROCESS, 16);
 	ringbuffer_head_add_samples (instance, 12);
