@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Sebastian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Sebastian Skjelstad <stian.skjelstad@gmail.com>
  *
  * UNIX cdrom filebrowser
  *
@@ -121,7 +121,7 @@ static void try(const char *dev, const char *vdev)
 #ifdef CDROM_VERBOSE
 	fprintf(stderr, "Testing %s\n", dev);
 #endif
-	bzero (temppath, sizeof (temppath));
+	memset (temppath, 0, sizeof (temppath));
 	if (readlink (dev, temppath, sizeof (temppath) - 1) > 0)
 	{
 		if (strncmp (temppath, "/dev/sr", 7)) return;
@@ -282,34 +282,13 @@ static void *cdrom_thread (void *_self)
 	}
 }
 
-static const char *CDA_description[] =
+static int cdint (const struct configAPI_t *configAPI)
 {
-	//                                                                          |
-	"CDA - CDROM Digital Audio, is a virtual file that links a cdrom audio",
-	"track or complete disc to an actual device. The device can be both physical",
-	"or virtual CDROM drive. Marking a random file in the filesystem with this",
-	"filetype will not make it playable.",
-	NULL
-};
-
-static const struct interfaceparameters CDA_p =
-{
-	"playcda", "cdaPlayer",
-	0, 0
-};
-
-static int cdint(void)
-{
-	struct moduletype mt;
 	char dev[32], vdev[12];
 	char a;
 
 	int i;
 
-	fsRegisterExt ("CDA");
-	mt.integer.i = MODULETYPE("CDA");
-	fsTypeRegister (mt, CDA_description, "plOpenCP", &CDA_p);
-
 	ocpdir_t_fill (
 		&cdrom_root,
 		cdrom_root_ref,
@@ -610,7 +589,7 @@ static ocpdirhandle_pt cdrom_drive_readdir_start (struct ocpdir_t *_self, void(*
 	dh->owner = self;
 	dh->initlba = -1;
 
-	bzero (&self->cdrom->lasttoc, sizeof (self->cdrom->lasttoc));
+	memset (&self->cdrom->lasttoc, 0, sizeof (self->cdrom->lasttoc));
 	free (dh->owner->cdrom->lastdiscid);
 	free (dh->owner->cdrom->lasttocstr);
 	dh->owner->cdrom->lastdiscid = 0;
@@ -705,7 +684,7 @@ leadout:
 			goto failout;
 		}
 
-		bzero (offsets, sizeof (offsets));
+		memset (offsets, 0, sizeof (offsets));
 		for (i=first; i <= last; i++)
 		{
 			offsets[i] = dh->owner->cdrom->lasttoc.track[i].lba_addr + 150;
@@ -907,7 +886,7 @@ static int cdrom_drive_readdir_iterate (ocpdirhandle_pt _dh)
 				{
 					snprintf (mi.album, sizeof (mi.album), "%s", dh->owner->cdrom->musicbrainzdata->album);
 				}
-				if (dh->owner->cdrom->musicbrainzdata && dh->owner->cdrom->musicbrainzdata->title[dh->i])
+				if (dh->owner->cdrom->musicbrainzdata && dh->owner->cdrom->musicbrainzdata->title[dh->i][0])
 				{
 					snprintf (mi.title, sizeof (mi.title), "%s", dh->owner->cdrom->musicbrainzdata->title[dh->i]);
 				} else if (!mi.title[0])
@@ -1093,6 +1072,7 @@ static struct ocpfilehandle_t *cdrom_track_open (struct ocpfile_t *_self)
 	ocpfilehandle_t_fill (&retval->head,
 	                      ocpfilehandle_cdrom_track_ref,
 	                      ocpfilehandle_cdrom_track_unref,
+	                      &self->head,
 	                      ocpfilehandle_cdrom_track_seek,
 	                      ocpfilehandle_cdrom_track_seek,
 	                      ocpfilehandle_cdrom_track_seek,
@@ -1131,8 +1111,4 @@ static int cdrom_track_filesize_ready (struct ocpfile_t *_self)
 	return 1;
 }
 
-#ifndef SUPPORT_STATIC_PLUGINS
-char *dllinfo = "";
-#endif
-
-DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {.name = "cdrom", .desc = "OpenCP UNIX audio-cdrom filebrowser (c) 2004-'22 Stian Skjelstad", .ver = DLLVERSION, .size = 0, .Init = cdint, .Close = cdclose};
+DLLEXTINFO_CORE_PREFIX struct linkinfostruct dllextinfo = {.name = "cdrom", .desc = "OpenCP UNIX audio-cdrom filebrowser (c) 2004-'23 Stian Skjelstad", .ver = DLLVERSION, .Init = cdint, .Close = cdclose, .sortindex = 30};
