@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2020-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2020-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Code to cache filehandles I/O
  *
@@ -34,9 +34,13 @@
 
 #define CACHE_LINES 4
 
+#if defined(FILEHANDLE_CACHE_DEBUG)
+static int do_filehandle_debug_print=1;
+#endif
+
 #ifdef FILEHANDLE_CACHE_DEBUG
-#define DEBUG_PRINT(...) do { if (do_debug_print) { fprintf(stderr, __VA_ARGS__); } } while (0)
-#define DUMP_SELF(S) do { if (do_debug_print) { dump_self(s); } } while (0)
+#define DEBUG_PRINT(...) do { if (do_filehandle_debug_print) { fprintf(stderr, __VA_ARGS__); } } while (0)
+#define DUMP_SELF(S) do { if (do_filehandle_debug_print) { dump_self(s); } } while (0)
 #else
 #define DEBUG_PRINT(...) do {} while (0)
 #define DUMP_SELF(S) do {} while (0)
@@ -103,6 +107,7 @@ struct ocpfilehandle_t *cache_filehandle_open_pre (struct ocpfile_t *owner, char
 		&retval->head,
 		cache_filehandle_ref,
 		cache_filehandle_unref,
+	        owner,
 		cache_filehandle_seek_set,
 		cache_filehandle_seek_cur,
 		cache_filehandle_seek_end,
@@ -128,6 +133,8 @@ struct ocpfilehandle_t *cache_filehandle_open_pre (struct ocpfile_t *owner, char
 	retval->cache_line[2].data = tailptr;
 	retval->cache_line[2].fill = retval->cache_line[2].size = taillen;
 
+	retval->head.origin->ref (retval->head.origin);
+
 	return &retval->head;
 }
 
@@ -140,6 +147,7 @@ struct ocpfilehandle_t *cache_filehandle_open (struct ocpfilehandle_t *parent)
 		&retval->head,
 		cache_filehandle_ref,
 		cache_filehandle_unref,
+	        parent->origin,
 		cache_filehandle_seek_set,
 		cache_filehandle_seek_cur,
 		cache_filehandle_seek_end,
@@ -165,6 +173,8 @@ struct ocpfilehandle_t *cache_filehandle_open (struct ocpfilehandle_t *parent)
 		retval->filesize = 0;//UINT64_C(0xffffffffffffffff);
 	}
 
+	retval->head.origin->ref (retval->head.origin);
+
 	retval->head.refcount = 1;
 
 	return &retval->head;
@@ -206,6 +216,12 @@ static void cache_filehandle_unref (struct ocpfilehandle_t *_s)
 		s->parent = 0;
 	}
 
+	if (s->head.origin)
+	{
+		s->head.origin->unref (s->head.origin);
+		s->head.origin = 0;
+	}
+
 	free (s);
 }
 
