@@ -35,14 +35,14 @@ static ssize_t mdb_test_read (int fd, void *buf, size_t size);
 static ssize_t mdb_test_write (int fd, void *buf, size_t size);
 static int mdb_test_open(const char *pathname, int flags, ...);
 static off_t mdb_test_lseek (int fd, off_t offset, int whence);
-static int mdb_test_close (int fd); 
+static int mdb_test_close (int fd);
 static int mdb_test_flock (int fd, int operation);
 
+#define CFDATAHOMEDIR_OVERRIDE "/foo/home/ocp/.ocp/"
 #include "mdb.c"
 #include "../stuff/compat.c"
 
 int fsWriteModInfo = 1;
-char *cfConfigDir = "/foo/home/ocp/.ocp/";
 
 static ssize_t (*mdb_test_read_hook) (int fd, void *buf, size_t size) = 0;
 static ssize_t (*mdb_test_write_hook) (int fd, void *buf, size_t size) = 0;
@@ -59,6 +59,23 @@ static int (*mdb_test_flock_hook) (int fd, int operation) = 0;
 #define ANSI_COLOR_CYAN    "\x1b[36m"
 #define ANSI_COLOR_RESET   "\x1b[0m"
 
+struct ocpfilehandle_t *ancient_filehandle (char *compressionmethod, int compressionmethod_len, struct ocpfilehandle_t *s)
+{
+	return 0;
+}
+
+void latin1_f_to_utf8_z (const char *src, size_t srclen, char *dst, size_t dstlen)
+{
+	snprintf (dst, dstlen, "%*s", (int)(MIN(dstlen - 1, srclen-1)), src);
+}
+
+void cp437_f_to_utf8_z(const char *src, size_t srclen, char *dst, size_t dstlen)
+{
+	snprintf (dst, dstlen, "%*s", (int)(MIN(dstlen - 1, srclen-1)), src);
+}
+
+const struct dirdbAPI_t dirdbAPI;
+
 static ssize_t mdb_test_read (int fd, void *buf, size_t size)
 {
 	if (mdb_test_read_hook) return mdb_test_read_hook (fd, buf, size);
@@ -401,7 +418,7 @@ int mdb_heap1_mdbNew (int test)
 		(mdbDataSize     !=  24) ? ANSI_COLOR_GREEN "OK" ANSI_COLOR_RESET : ANSI_COLOR_RED "Failed, heap stuck" ANSI_COLOR_RESET,
 		(int)mdbDirtyMapSize,
 		(test==1)?
-		(mdbDirtyMapSize != 256) ? ANSI_COLOR_RED "Failed, heap resized" ANSI_COLOR_RESET : ANSI_COLOR_GREEN "OK" ANSI_COLOR_RESET : 
+		(mdbDirtyMapSize != 256) ? ANSI_COLOR_RED "Failed, heap resized" ANSI_COLOR_RESET : ANSI_COLOR_GREEN "OK" ANSI_COLOR_RESET :
 		(mdbDirtyMapSize != 24) ? ANSI_COLOR_GREEN "OK" ANSI_COLOR_RESET : ANSI_COLOR_RED "Failed, heap stuck" ANSI_COLOR_RESET );
 	mdb_heap1_mdbNew_finalize ();
 
@@ -425,7 +442,7 @@ int mdb_heap1_mdbNew (int test)
 		(mdbDataSize     ==  24) ? ANSI_COLOR_RED "Failed, heap resized" ANSI_COLOR_RESET : ANSI_COLOR_GREEN "OK" ANSI_COLOR_RESET,
 		(int)mdbDirtyMapSize,
 		(test==1)?
-		(mdbDirtyMapSize != 256) ? ANSI_COLOR_RED "Failed, heap resized" ANSI_COLOR_RESET : ANSI_COLOR_GREEN "OK" ANSI_COLOR_RESET : 
+		(mdbDirtyMapSize != 256) ? ANSI_COLOR_RED "Failed, heap resized" ANSI_COLOR_RESET : ANSI_COLOR_GREEN "OK" ANSI_COLOR_RESET :
 		(mdbDirtyMapSize != 24) ? ANSI_COLOR_GREEN "OK" ANSI_COLOR_RESET : ANSI_COLOR_RED "Failed, heap stuck" ANSI_COLOR_RESET );
 	mdb_heap1_mdbNew_finalize ();
 
@@ -462,7 +479,7 @@ int mdb_basic_mdbFree (void)
 
 	mdb_basic_mdbFree_prepare ();
 	r = mdbNew (1);
-	bzero (mdbDirtyMap, mdbDirtyMapSize / 8);
+	memset (mdbDirtyMap, 0, mdbDirtyMapSize / 8);
 	mdbFree (r, 1);
 	e = 0,
 	fprintf (stderr, "mdbFree(1):");
@@ -481,7 +498,7 @@ int mdb_basic_mdbFree (void)
 
 	mdb_basic_mdbFree_prepare ();
 	r = mdbNew (2);
-	bzero (mdbDirtyMap, mdbDirtyMapSize / 8);
+	memset (mdbDirtyMap, 0, mdbDirtyMapSize / 8);
 	mdbFree (r, 2);
 	e = 0,
 	fprintf (stderr, "mdbFree(2):");
@@ -1012,8 +1029,8 @@ int mdb_basic_mdbWriteModuleInfo_mdbGetModuleInfo (void)
 	struct moduleinfostruct src;
 	struct moduleinfostruct dst;
 
-	bzero (&src, sizeof (src));
-	bzero (&dst, sizeof (dst));
+	memset (&src, 0, sizeof (src));
+	memset (&dst, 0, sizeof (dst));
 
 	fprintf (stderr, ANSI_COLOR_CYAN "MDB mdbWriteModuleInfo mdbGetModuleInfo\n" ANSI_COLOR_RESET);
 
@@ -1035,12 +1052,12 @@ int mdb_basic_mdbWriteModuleInfo_mdbGetModuleInfo (void)
 	src.channels = 2;
 	src.playtime = 0x0450;
 	src.date = 0x12345678;
-	bzero (src.title, sizeof (src.title));
-	bzero (src.composer, sizeof (src.composer));
-	bzero (src.artist, sizeof (src.artist));
-	bzero (src.style, sizeof (src.style));
-	bzero (src.comment, sizeof (src.comment));
-	bzero (src.album, sizeof (src.album));
+	memset (src.title, 0, sizeof (src.title));
+	memset (src.composer, 0, sizeof (src.composer));
+	memset (src.artist, 0, sizeof (src.artist));
+	memset (src.style, 0, sizeof (src.style));
+	memset (src.comment, 0, sizeof (src.comment));
+	memset (src.album, 0, sizeof (src.album));
 
 	e=0;
 	fprintf (stderr, "NULL-strings mdbWriteModuleInfo() mdbGetModuleInfo() sequence:");
@@ -1133,12 +1150,12 @@ int mdb_basic_mdbWriteModuleInfo_mdbGetModuleInfo (void)
 	fprintf (stderr, "%s\n" ANSI_COLOR_RESET, e ? "" : ANSI_COLOR_GREEN " OK");
 
 	/* remove garbage, after zero-terminations */
-	bzero (src.title, sizeof (src.title));
-	bzero (src.composer, sizeof (src.composer));
-	bzero (src.artist, sizeof (src.artist));
-	bzero (src.style, sizeof (src.style));
-	bzero (src.comment, sizeof (src.comment));
-	bzero (src.album, sizeof (src.album));
+	memset (src.title, 0, sizeof (src.title));
+	memset (src.composer, 0, sizeof (src.composer));
+	memset (src.artist, 0, sizeof (src.artist));
+	memset (src.style, 0, sizeof (src.style));
+	memset (src.comment, 0, sizeof (src.comment));
+	memset (src.album, 0, sizeof (src.album));
 	snprintf (src.title, sizeof(src.title), "%s", "a short standard title");
 	snprintf (src.composer, sizeof (src.composer), "%s", "a short composer name");
 	snprintf (src.artist, sizeof (src.artist), "%s", "a short artist name");
@@ -1370,7 +1387,7 @@ int mdb_basic_mdbInit (void)
 
 	mdb_basic_mdbInit_prepare ();
 
-	fprintf (stderr, "mdbInit(): %s\n" ANSI_COLOR_RESET, mdbInit() ? ANSI_COLOR_GREEN "OK" : ANSI_COLOR_RED "Failed");
+	fprintf (stderr, "mdbInit(): %s\n" ANSI_COLOR_RESET, mdbInit (0) ? ANSI_COLOR_GREEN "OK" : ANSI_COLOR_RED "Failed");
 
 	fprintf (stderr, "mdbDataSize: ");
 	if (mdbDataSize < 25) /* we do allow possible future pre-allocation of free units */
@@ -1657,7 +1674,7 @@ int mdb_basic_mdbUpdate (void)
 
 	mdb_basic_mdbUpdate_prepare ();
 
-	if (!mdbInit())
+	if (!mdbInit (0))
 	{
 		fprintf (stderr, ANSI_COLOR_RED "mdbInit() failed...\n" ANSI_COLOR_RESET);
 		retval |= 1;
