@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Module information DataBase (and some other related stuff=
  *
@@ -39,10 +39,13 @@
 #include "boot/psetting.h"
 #include "dirdb.h"
 #include "filesystem.h"
+#include "filesystem-ancient.h"
 #include "mdb.h"
 #include "pfilesel.h"
+#include "stuff/cp437.h"
 #include "stuff/compat.h"
 #include "stuff/imsrtns.h"
+#include "stuff/latin1.h"
 
 #ifdef MDB_DEBUG
 #define DEBUG_PRINT(...) do { fprintf(stderr, __VA_ARGS__); } while (0)
@@ -50,6 +53,12 @@
 #define DEBUG_PRINT(...) {}
 #endif
 
+#ifdef CFDATAHOMEDIR_OVERRIDE
+# define CFDATAHOMEDIR CFDATAHOMEDIR_OVERRIDE
+#else
+# define CFDATAHOMEDIR configAPI->DataHomeDir
+#endif
+
 #define MDB_USED 1
 #define MDB_STRING_MORE 6        /* STRING, and more data follows in the next node */
 #define MDB_STRING_TERMINATION 2 /* STRING, last (or only) node */
@@ -154,32 +163,35 @@ void mdbRegisterReadInfo (struct mdbreadinforegstruct *r)
 
 	r->next=mdbReadInfos;
 	mdbReadInfos=r;
-	if (r->Event)
-	{
-		r->Event(mdbEvInit);
-	}
 }
 
 void mdbUnregisterReadInfo (struct mdbreadinforegstruct *r)
 {
-	struct mdbreadinforegstruct **prev = &mdbReadInfos;
+	struct mdbreadinforegstruct **prev = &mdbReadInfos, *iter = mdbReadInfos;
 
-	while (*prev)
+	while (iter)
 	{
-		if (*prev == r)
+		if (iter == r)
 		{
 			DEBUG_PRINT ("mdbUnregisterReadInfo(%s)\n", r->name);
 
-			*prev = (*prev)->next;
+			*prev = iter->next;
 			return;
 		}
-		prev = &(*prev)->next;
+		prev = &iter->next;
+		iter = iter->next;
 	}
 
 	DEBUG_PRINT ("mdbUnregisterReadInfo(%s) # WARNING, unable to find entry\n", r->name);
 }
 
 /* detect file infomation using 'plugins' */
+static const struct mdbReadInfoAPI_t mdbReadInfoAPI =
+{
+	cp437_f_to_utf8_z,
+	latin1_f_to_utf8_z,
+	&dirdbAPI
+};
 int mdbReadInfo (struct moduleinfostruct *m, struct ocpfilehandle_t *f)
 {
 	char mdbScanBuf[1084];
@@ -194,6 +206,7 @@ int mdbReadInfo (struct moduleinfostruct *m, struct ocpfilehandle_t *f)
 	}
 	memset (mdbScanBuf, 0, sizeof (mdbScanBuf));
 	maxl = f->read (f, mdbScanBuf, sizeof (mdbScanBuf));
+	f->seek_set (f, 0);
 
 	{
 		const char *path;
@@ -203,9 +216,42 @@ int mdbReadInfo (struct moduleinfostruct *m, struct ocpfilehandle_t *f)
 
 	/* slow version that also allows more I/O */
 	for (rinfos=mdbReadInfos; rinfos; rinfos=rinfos->next)
+	{
 		if (rinfos->ReadInfo)
-			if (rinfos->ReadInfo(m, f, mdbScanBuf, maxl))
+		{
+			if (rinfos->ReadInfo(m, f, mdbScanBuf, maxl, &mdbReadInfoAPI))
+			{
 				return 1;
+			}
+		}
+	}
+
+	{
+		struct ocpfilehandle_t *ancient;
+		char compressionmethod[256];
+
+		if ((ancient = ancient_filehandle (compressionmethod, sizeof (compressionmethod), f)))
+		{
+			snprintf (m->comment, sizeof (m->comment), "Compressed with: %.*s", (int)(sizeof (m->comment) - 17 - 1), compressionmethod);
+
+			maxl = ancient->read (ancient, mdbScanBuf, sizeof (mdbScanBuf));
+			ancient->seek_set (ancient, 0);
+
+			for (rinfos=mdbReadInfos; rinfos; rinfos=rinfos->next)
+			{
+				if (rinfos->ReadInfo)
+				{
+					if (rinfos->ReadInfo(m, ancient, mdbScanBuf, maxl, &mdbReadInfoAPI))
+					{
+						ancient->unref (ancient);
+						return 1;
+					}
+				}
+			}
+
+			ancient->unref (ancient);
+		}
+	}
 
 	return m->modtype.integer.i != 0;
 }
@@ -252,7 +298,7 @@ static uint32_t mdbNew (int size)
 				return UINT32_MAX;
 			}
 			mdbDirtyMap = (uint8_t *)t;
-			bzero (mdbDirtyMap + mdbDirtyMapSize / 8, (M - mdbDirtyMapSize) / 8);
+			memset (mdbDirtyMap + mdbDirtyMapSize / 8, 0, (M - mdbDirtyMapSize) / 8);
 			mdbDirtyMapSize = M;
 		}
 
@@ -264,7 +310,7 @@ static uint32_t mdbNew (int size)
 			return UINT32_MAX;
 		}
 		mdbData=(struct modinfoentry *)t;
-		bzero(mdbData + mdbDataSize, (N - mdbDataSize) * sizeof(mdbData[0]));
+		memset (mdbData + mdbDataSize, 0, (N - mdbDataSize) * sizeof(mdbData[0]));
 		mdbDataSize = N;
 		for (j=i; j<mdbDataSize; j++) /* all appended entries are dirty */
 		{
@@ -300,7 +346,7 @@ static void mdbFree (uint32_t ref, int size)
 
 	for (j = 0; j < size; j++)
 	{
-		bzero (mdbData + ref + j, sizeof (mdbData[0]));
+		memset (mdbData + ref + j, 0, sizeof (mdbData[0]));
 		mdbDirty=1;
 		mdbDirtyMap[(ref + j)>>3] |= 1 << ((ref + j) & 0x07);
 	}
@@ -467,7 +513,7 @@ static int miecmp (const void *a, const void *b)
 }
 
 /* Unit test is available */
-int mdbInit (void)
+int mdbInit (const struct configAPI_t *configAPI)
 {
 	char *path;
 	struct mdbheader header;
@@ -488,7 +534,7 @@ int mdbInit (void)
 	mdbSearchIndexCount = 0;
 	mdbSearchIndexSize = 0;
 
-	makepath_malloc (&path, 0, cfConfigDir, "CPMODNFO.DAT", 0);
+	makepath_malloc (&path, 0, CFDATAHOMEDIR, "CPMODNFO.DAT", 0);
 	fprintf(stderr, "Loading %s .. ", path);
 
 	if (mdbFd >= 0)
@@ -598,7 +644,7 @@ int mdbInit (void)
 
 		qsort(mdbSearchIndexData, mdbSearchIndexCount, sizeof(*mdbSearchIndexData), miecmp);
 
-		for (i=0; i<mdbSearchIndexSize; i++)
+		for (i=0; i<mdbSearchIndexCount; i++)
 		{
 			DEBUG_PRINT("%5d => 0x%08"PRIx32" %"PRIu64" 0x%02x%02x%02x%02x%02x%02x%02x%02x\n",
 				i, mdbSearchIndexData[i], mdbData[mdbSearchIndexData[i]].mie.general.size,
@@ -839,7 +885,7 @@ static uint32_t mdbGetModuleReference (const char *name, uint64_t size)
 	m->mie.general.style_ref = UINT32_MAX;
 	m->mie.general.comment_ref = UINT32_MAX;
 	m->mie.general.album_ref = UINT32_MAX;
-	bzero (m->mie.general.reserved, sizeof (m->mie.general.reserved));
+	memset (m->mie.general.reserved, 0, sizeof (m->mie.general.reserved));
 	DEBUG_PRINT("mdbGetModuleReference(\"%s\" %"PRIu64") => new => 0x%08"PRIx32"\n", name, size, i);
 	return i;
 }
@@ -864,7 +910,7 @@ static void mdbGetString (char *dst, int dstlen, uint32_t mdb_ref)
 	while (1)
 	{
 		int l;
-		*dst = 0; /* parent is bzero() already, but does not hurt to ensure */
+		*dst = 0; /* parent is memset() already, but does not hurt to ensure */
 		if ((mdb_ref == 0) || (mdb_ref >= mdbDataSize) || (!dstlen))
 		{
 			return;
@@ -893,7 +939,7 @@ static void mdbGetString (char *dst, int dstlen, uint32_t mdb_ref)
 /* Partially unit test in mdbInit */
 int mdbGetModuleInfo (struct moduleinfostruct *m, uint32_t mdb_ref)
 {
-	bzero(m, sizeof(struct moduleinfostruct));
+	memset (m, 0, sizeof(*m));
 	assert (mdb_ref > 0);
 	assert (mdb_ref < mdbDataSize);
 	assert (mdbData[mdb_ref].mie.general.record_flags == MDB_USED);
