@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2022 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2022-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Glue logic for fetching and caching data from MusicBrainz
  * online music database (CDROM).
@@ -41,12 +41,11 @@
 #include "dirdb.h"
 #include "filesystem.h"
 #include "filesystem-drive.h"
-#include "filesystem-file-mem.h"
+#include "filesystem-file-dev.h"
 #include "filesystem-setup.h"
 #include "musicbrainz.h"
 #include "pfilesel.h"
 #include "stuff/compat.h"
-#include "stuff/framelock.h"
 #include "stuff/poutput.h"
 #include "stuff/utf-8.h"
 
@@ -151,10 +150,16 @@ static int musicbrainz_spawn (struct musicbrainz_queue_t *t)
 	open ("/dev/null", O_RDONLY);
 
 	close (1); /* stdout */
-	dup (fdout[1]);
+	if (dup (fdout[1]) != 1)
+	{
+		perror ("dup() failed");
+	}
 
 	close (2); /* stderr */
-	dup (fderr[1]);
+	if (dup (fderr[1]) != 2)
+	{
+		perror ("dup() failed");
+	}
 
 	close (fdout[0]);
 	close (fdout[1]);
@@ -171,7 +176,7 @@ static int musicbrainz_spawn (struct musicbrainz_queue_t *t)
 		"-L", temp,
 		NULL);
 
-	perror ("execve(curl");
+	perror ("execve(curl)");
 
 	_exit (1);
 }
@@ -789,7 +794,7 @@ void musicbrainz_lookup_discid_cancel (void *token)
 	}
 }
 
-int musicbrainz_init (void)
+int musicbrainz_init (const struct configAPI_t *configAPI)
 {
 	char *path;
 	char header[64];
@@ -800,7 +805,7 @@ int musicbrainz_init (void)
 	}
 	musicbrainz_setup_init ();
 
-	makepath_malloc (&path, 0, cfConfigDir, "CPMUSBRN.DAT", 0);
+	makepath_malloc (&path, 0, configAPI->DataHomeDir, "CPMUSBRN.DAT", 0);
 	fprintf (stderr, "Loading %s .. ", path);
 	musicbrainz.fddb = open (path, O_RDWR | O_CREAT, S_IREAD|S_IWRITE);
 	if (musicbrainz.fddb < 0)
@@ -950,7 +955,10 @@ void musicbrainz_done (void)
 		pos += 28 + 8 + 4;
 		pos += (musicbrainz.cache[i].size & SIZE_MASK);
 	}
-	ftruncate (musicbrainz.fddb, pos);
+	if (ftruncate (musicbrainz.fddb, pos))
+	{
+		perror ("ftruncate() failed");
+	}
 done:
 	for (i=0; i < musicbrainz.cachecount; i++)
 	{
@@ -975,46 +983,40 @@ void musicbrainz_database_h_free (struct musicbrainz_database_h *e)
 
 static struct ocpfile_t      *musicbrainzsetup; // needs to overlay an dialog above filebrowser, and after that the file is "finished"   Special case of DEVv
 
-static int                    musicbrainzSetupInit (struct moduleinfostruct *info, struct ocpfilehandle_t *f, const struct interfaceparameters *ip);
-static interfaceReturnEnum    musicbrainzSetupRun  (void);
-static struct interfacestruct musicbrainzSetupIntr = {musicbrainzSetupInit, musicbrainzSetupRun, 0, "musicbrainzSetup" INTERFACESTRUCT_TAIL};
+static void musicbrainzSetupRun  (void **token, const struct DevInterfaceAPI_t *API);
 
 static void musicbrainz_setup_init (void)
 {
-	struct moduleinfostruct m;
-	uint32_t mdbref;
+	musicbrainzsetup = dev_file_create (
+		dmSetup->basedir,
+		"musicbrainz.dev",
+		"MusicBrainz Cache DataBase",
+		"",
+		0, /* token */
+		0, /* Init */
+		musicbrainzSetupRun,
+		0, /* Close */
+		0  /* Destructor */
+	);
 
-	musicbrainzsetup = mem_file_open (dmSetup->basedir, dirdbFindAndRef (dmSetup->basedir->dirdb_ref, "musicbrainz.dev", dirdb_use_file), strdup (musicbrainzSetupIntr.name), strlen (musicbrainzSetupIntr.name));
-	dirdbUnref (musicbrainzsetup->dirdb_ref, dirdb_use_file);
-	mdbref = mdbGetModuleReference2 (musicbrainzsetup->dirdb_ref, strlen (musicbrainzSetupIntr.name));
-	mdbGetModuleInfo (&m, mdbref);
-	m.modtype.integer.i = MODULETYPE("DEVv");
-	strcpy (m.title, "MusicBrain Cache DataBase");
-	mdbWriteModuleInfo (mdbref, &m);
 	filesystem_setup_register_file (musicbrainzsetup);
-	plRegisterInterface (&musicbrainzSetupIntr);
 }
 
 static void musicbrainz_setup_done (void)
 {
-	plUnregisterInterface (&musicbrainzSetupIntr);
 	if (musicbrainzsetup)
 	{
 		filesystem_setup_unregister_file (musicbrainzsetup);
+		musicbrainzsetup->unref (musicbrainzsetup);
 		musicbrainzsetup = 0;
 	}
 }
 
-static int musicbrainzSetupInit (struct moduleinfostruct *info, struct ocpfilehandle_t *f, const struct interfaceparameters *ip)
-{
-	return 1;
-}
-
 static void musicbrainzSetupDialogDraw (struct musicbrainz_cacheline_sort_t *entry, int epos)
 {
 	int mlWidth = 55;
 	int mlHeight = 7;
-	int mlTop = (plScrHeight - mlHeight) / 2 ;
+	int mlTop = (plScrHeight - mlHeight) / 2;
 	int mlLeft = (plScrWidth - mlWidth) / 2;
 
 	display_nprintf (mlTop + 0, mlLeft, 0x17, mlWidth, "\xda%*C\xc4\xbf", mlWidth - 2); /* +---------+ */
@@ -1149,12 +1151,15 @@ static void musicbrainzSetupDraw (const char *title, int dsel, struct musicbrain
 			}
 		} else {
 			char timebuffer[28];
-			struct tm *thetime;
+			struct tm thetime;
 			time_t inputtime;
 
 			inputtime = musicbrainz.cache[sorted[index].pointsat].lastscan;
-			thetime = localtime (&inputtime);
-			strftime(timebuffer, sizeof (timebuffer), "%d.%m.%Y %H:%M %z(%Z)", thetime);
+			if (!localtime_r (&inputtime, &thetime))
+			{
+				memset (&thetime, 0, sizeof (thetime));
+			}
+			strftime(timebuffer, sizeof (timebuffer), "%d.%m.%Y %H:%M %z(%Z)", &thetime);
 
 			displaychr (mlTop + i, mlLeft + 1, (dsel==index)?0x87:0x07, ' ', 1);
 			displaystr (mlTop + i, mlLeft + 2, (dsel==index)?0x87:0x07, timebuffer, 29); /* includes two exrta space */
@@ -1287,24 +1292,24 @@ static struct musicbrainz_cacheline_sort_t *musicbrainz_create_sort(void)
 	return sorted;
 }
 
-static interfaceReturnEnum musicbrainzSetupRun (void)
+static void musicbrainzSetupRun (void **token, const struct DevInterfaceAPI_t *API)
 {
 	int dsel = 0, dialog = 0, epos = 0;
 	struct musicbrainz_cacheline_sort_t *sorted = musicbrainz_create_sort ();
 	if (!sorted && musicbrainz.cachecount)
 	{ /* malloc failure... */
-		return interfaceReturnNextAuto;
+		return;
 	}
 	while (1)
 	{
-		fsDraw();
+		API->fsDraw();
 		musicbrainzSetupDraw("MusicBrain Cache DataBase", dsel, sorted);
 		if (dialog)
 		{
-	                musicbrainzSetupDialogDraw (sorted + dsel, epos);
-			while (ekbhit())
+			musicbrainzSetupDialogDraw (sorted + dsel, epos);
+			while (API->console->KeyboardHit())
 			{
-				int key = egetch();
+				int key = API->console->KeyboardGetChar();
 				switch (key)
 				{
 					case KEY_DOWN:
@@ -1343,6 +1348,7 @@ static interfaceReturnEnum musicbrainzSetupRun (void)
 							epos = 3;
 						}
 						break;
+					case KEY_EXIT:
 					case KEY_ESC:
 						if (musicbrainz.cachecount)
 						{
@@ -1392,7 +1398,7 @@ static interfaceReturnEnum musicbrainzSetupRun (void)
 									int mlTop = (plScrHeight - mlHeight) / 2 ;
 									int mlLeft = (plScrWidth - mlWidth) / 2;
 
-									fsDraw();
+									API->fsDraw();
 
 									display_nprintf (mlTop + 0, mlLeft, 0x1e, mlWidth, "\xda%*C\xc4\xbf", mlWidth - 2);
 									display_nprintf (mlTop + 1, mlLeft, 0x1e, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
@@ -1402,13 +1408,14 @@ static interfaceReturnEnum musicbrainzSetupRun (void)
 									display_nprintf (mlTop + 5, mlLeft, 0x1e, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 									display_nprintf (mlTop + 6, mlLeft, 0x1e, mlWidth, "\xc0%*C\xc4\xd9", mlWidth - 2);
 
-									framelock ();
+									API->console->FrameLock ();
 
-									while (ekbhit())
+									while (API->console->KeyboardHit())
 									{
-										int key = egetch();
+										int key = API->console->KeyboardGetChar();
 										switch (key)
 										{
+											case KEY_EXIT:
 											case KEY_ESC:
 												musicbrainz_lookup_discid_cancel (handle);
 												superbreak = 1;
@@ -1425,7 +1432,7 @@ static interfaceReturnEnum musicbrainzSetupRun (void)
 							sorted = musicbrainz_create_sort ();
 							if (!sorted && musicbrainz.cachecount)
 							{ /* malloc failure... */
-								return interfaceReturnNextAuto;
+								return;
 							}
 							for (dsel = 0; dsel < musicbrainz.cachecount; dsel++)
 							{
@@ -1457,7 +1464,7 @@ static interfaceReturnEnum musicbrainzSetupRun (void)
 							sorted = musicbrainz_create_sort ();
 							if (!sorted && musicbrainz.cachecount)
 							{ /* malloc failure... */
-								return interfaceReturnNextAuto;
+								return;
 							}
 						} else if (epos == 3)
 						{ /* submit */
@@ -1510,9 +1517,9 @@ static interfaceReturnEnum musicbrainzSetupRun (void)
 				}
 			}
 		} else {
-			while (ekbhit())
+			while (API->console->KeyboardHit())
 			{
-				int key = egetch();
+				int key = API->console->KeyboardGetChar();
 				switch (key)
 				{
 					case KEY_HOME:
@@ -1540,15 +1547,16 @@ static interfaceReturnEnum musicbrainzSetupRun (void)
 							goto superexit;
 						}
 						break;
+					case KEY_EXIT:
 					case KEY_ESC:
 						free (sorted);
-						return interfaceReturnNextAuto;
+						return;
 					default:
 						break;
 				}
 			}
 		}
 superexit:
-		framelock();
+		API->console->FrameLock();
 	}
 }
