@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * CP hypertext help viewer
  *
@@ -339,7 +339,7 @@ static int doReadHelpFile(FILE *file)
 	};
 }
 
-static char plReadHelpExternal(void)
+static char plReadHelpExternal (const struct configAPI_t *configAPI)
 {
 	char   *helpname;
 	FILE   *bf;
@@ -347,7 +347,7 @@ static char plReadHelpExternal(void)
 	if (Page && (HelpfileErr==hlpErrOk))
 		return 1;
 
-	makepath_malloc (&helpname, 0, cfDataDir, "ocp.hlp", 0);
+	makepath_malloc (&helpname, 0, configAPI->DataDir, "ocp.hlp", 0);
 	if ((bf=fopen(helpname, "r")))
 	{
 		free (helpname);
@@ -597,7 +597,6 @@ void brDisplayHelp(void)
 	unsigned int curlinky;
 	char destbuffer[60];
 	char strbuffer[256];
-	char numbuffer[4];
 	int descxp;
 	unsigned int y;
 
@@ -614,18 +613,7 @@ void brDisplayHelp(void)
 
 	displaystr(plWinFirstLine-1, 0, 0x09, "   OpenCP help ][   ", 20);
 
-
-	if (HelpfileErr==hlpErrOk)
-		strcpy(strbuffer, curpage->desc);
-	else
-		strcpy(strbuffer, "Error!");
-
-	_convnum(100*plHelpScroll/MAX(plHelpHeight-plWinHeight, 1), numbuffer, 10, 3);
-
-	strcat(strbuffer, "-");
-	strcat(strbuffer, numbuffer);
-	strcat(strbuffer, "%");
-
+	snprintf (strbuffer, sizeof (strbuffer), "%s-%3d%%", (HelpfileErr==hlpErrOk)?curpage->desc:"Error!", 100*plHelpScroll/MAX(plHelpHeight-plWinHeight, 1));
 	memset(destbuffer, 0x20, 60);
 	descxp=MAX(0, 59-(signed)strlen(strbuffer));
 
@@ -664,46 +652,42 @@ void brDisplayHelp(void)
 		int addx = (plScrWidth - 80) / 2;
 		for (y=0; y<plWinHeight; y++)
 		{
+			int yp=(y+plHelpScroll)*80;
 			if ((y+plHelpScroll)>=plHelpHeight)
 			{
 				displayvoid(y+plWinFirstLine, 0, plScrWidth);
 				continue;
 			}
+
+			displayvoid (y+plWinFirstLine, 0, addx);
 			if (y!=curlinky)
-			{
-				displayvoid(y+plWinFirstLine, 0, addx);
-				displaystrattr(y+plWinFirstLine, 0+addx, &curpage->rendered[(y+plHelpScroll)*80], 80);
-				displayvoid(y+plWinFirstLine, 80+addx, plScrWidth-80-addx);
-			} else {
-				int yp=(y+plHelpScroll)*80;
+			{ /* display line as is, no links needs to be highlighted */
+				displaystrattr(y+plWinFirstLine, 0+addx, &curpage->rendered[yp], 80);
+			} else { /* there is a link that should be highlighted on the current line */
 				int xp;
 			        char dummystr[82];
 				int i, off;
 
-				displayvoid(y+plWinFirstLine, 0, addx);
 				if (curlink->posx!=0)
-			        {
+			        { /* print data before the link */
 					displaystrattr(y+plWinFirstLine, addx, &curpage->rendered[yp], curlink->posx);
 			        };
 
-			        xp=curlink->posx+curlink->len;
+				/* highlight the link */
+				for (i=0, off=yp+curlink->posx; curpage->rendered[off] & 0xff; i++, off++)
+					dummystr[i]=curpage->rendered[off] & 0xff;
+				dummystr[i]=0;
+				displaystr(y+plWinFirstLine, curlink->posx+addx, 4, dummystr, curlink->len);
 
+				/* print data after the link */
+			        xp=curlink->posx+curlink->len;
 			        displaystrattr(y+plWinFirstLine, xp+addx,
 		                       &curpage->rendered[yp+xp],
 		                       79-xp);
 
-
-				for (i=0, off=yp+curlink->posx; curpage->rendered[off] & 0xff; i++, off++)
-					dummystr[i]=curpage->rendered[off] & 0xff;
-
-				dummystr[i]=0;
-
-				displaystr(y+plWinFirstLine, curlink->posx+addx, 4, dummystr, curlink->len);
-
-				displayvoid(y+plWinFirstLine, 80+addx, plScrWidth-80-addx);
-
 			        /* and all this just to prevent flickering. ARG! */
 			}
+			displayvoid (y+plWinFirstLine, 80+addx, plScrWidth-80-addx);
 		}
 	}
 }
@@ -912,13 +896,13 @@ int brHelpKey(uint16_t key)
 	return 1;
 }
 
-static int hlpGlobalInit(void)
+static int hlpGlobalInit (const struct configAPI_t *configAPI)
 {
 	helppage *pg;
 
 	plHelpHeight=plHelpScroll=0;
 
-	if (!plReadHelpExternal())
+	if (!plReadHelpExternal (configAPI))
 	{
 		fprintf(stderr, "Warning. Failed to read help files\n");
 		return errOk; /* this error is not fatal to rest of the player */
@@ -975,7 +959,4 @@ static void hlpGlobalClose(void)
 	hlpFreePages();
 }
 
-#ifndef SUPPORT_STATIC_PLUGINS
-char *dllinfo = "";
-#endif
-DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {.name = "cphelper", .desc = "OpenCP help browser (c) 1998-'22 Fabian Giesen", .ver = DLLVERSION, .size = 0, .Init = hlpGlobalInit, .Close = hlpGlobalClose};
+DLLEXTINFO_CORE_PREFIX struct linkinfostruct dllextinfo = {.name = "cphelper", .desc = "OpenCP help browser (c) 1998-'23 Fabian Giesen", .ver = DLLVERSION, .sortindex = 20, .Init = hlpGlobalInit, .Close = hlpGlobalClose};
