@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2008-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2008-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * FLACPlay - Player for FLAC
  *
@@ -19,32 +19,24 @@
  */
 
 #include "config.h"
-#include <errno.h>
-#include <fcntl.h>
 #include <FLAC/all.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include "types.h"
-#include "cpiface/gif.h"
-#include "cpiface/jpeg.h"
-#include "cpiface/png.h"
-#include "dev/deviplay.h"
+#include "cpiface/cpiface.h"
 #include "dev/mcp.h"
 #include "dev/player.h"
-#include "dev/plrasm.h"
 #include "dev/ringbuffer.h"
 #include "filesel/filesystem.h"
 #include "flacplay.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
-#include "stuff/poll.h"
-#include "stuff/timer.h"
-
-static volatile int clipbusy=0;
+#include "stuff/poutput.h"
 
 /* options */
-static int inpause;
+static int flac_inpause;
 
 static uint32_t voll,volr;
 static int vol;
@@ -76,32 +68,21 @@ static unsigned int flacrate; /* this is the source rate */
 static int flacstereo; /* this we can ignore I think */
 static unsigned int flac_max_blocksize;
 static int flacbits=0;
+static uint32_t flacRate; /* this is the target rate (devp) */
 
 static uint64_t  flaclastpos;
 
-/* devp pre-buffer zone */
-static uint16_t *buf16; /* here we dump data, with correct rate, ready for converting, sample size, layout, etc */
-
-/* devp buffer zone */
-static uint32_t bufpos; /* devp write head location */
-static uint32_t buflen; /* devp buffer-size in samples */
-static void *plrbuf; /* the devp buffer */
-static int stereo; /* boolean */
-static int bit16; /* boolean */
-static int signedout; /* boolean */
-static int reversestereo; /* boolean */
 static int donotloop=1;
 
-static int (*_GET)(int ch, int opt);
-static void (*_SET)(int ch, int opt, int val);
+static volatile int clipbusy=0;
 
 static int flacPendingSeek = 0;
 static uint64_t flacPendingSeekPos;
 
-struct flac_comment_t __attribute__ ((visibility ("internal"))) **flac_comments;
-int                   __attribute__ ((visibility ("internal")))   flac_comments_count;
-struct flac_picture_t __attribute__ ((visibility ("internal")))  *flac_pictures;
-int                   __attribute__ ((visibility ("internal")))   flac_pictures_count;
+OCP_INTERNAL struct flac_comment_t **flac_comments;
+OCP_INTERNAL int                     flac_comments_count;
+OCP_INTERNAL struct flac_picture_t  *flac_pictures;
+OCP_INTERNAL int                     flac_pictures_count;
 
 static void add_comment2(const char *title, const char *value, const uint32_t valuelen)
 {
@@ -215,6 +196,10 @@ do { \
 	} \
 	rs = _rs * volr / 256.0; \
 	ls = _ls * voll / 256.0; \
+	if (srnd) \
+	{ \
+		ls ^= 0xffff; \
+	} \
 } while(0)
 
 /* FLAC decoder needs more data */
@@ -274,8 +259,9 @@ static FLAC__StreamDecoderWriteStatus write_callback (
 	size of buffer =  frame->header.channels * frame->header.blocksize * frame->header.bits_per_sample / 8
 	buffer is a 2D array with [channel][sample] as index
 */
+	struct cpifaceSessionAPI_t *cpifaceSession = client_data;
+
 	unsigned int i;
-	unsigned short xormask=0x0000;
 
 	int pos1, length1, pos2, length2;
 
@@ -284,10 +270,7 @@ static FLAC__StreamDecoderWriteStatus write_callback (
 	else
 		flaclastpos=frame->header.number.sample_number;
 
-	if (srnd)
-		xormask=0xffff;
-
-	ringbuffer_get_head_samples (flacbufpos, &pos1, &length1, &pos2, &length2);
+	cpifaceSession->ringbufferAPI->get_head_samples (flacbufpos, &pos1, &length1, &pos2, &length2);
 
 	if (frame->header.blocksize > (length1+length2))
 	{
@@ -297,14 +280,9 @@ static FLAC__StreamDecoderWriteStatus write_callback (
 
 	for (i=0;i<frame->header.blocksize;i++)
 	{
-		int ls, rs;
-
-		ls  = make_16bit(buffer[0][i], frame->header.bits_per_sample);
-		rs = make_16bit(buffer[1][i], frame->header.bits_per_sample);
 		/* append to buffer */
-		PANPROC;
-		flacbuf[pos1*2]=(int16_t)ls^xormask;
-		flacbuf[pos1*2+1]=rs;
+		flacbuf[pos1*2+0] = make_16bit(buffer[0][i], frame->header.bits_per_sample);
+		flacbuf[pos1*2+1] = make_16bit(buffer[1][i], frame->header.bits_per_sample);
 		pos1++;
 		if (!--length1)
 		{
@@ -315,7 +293,7 @@ static FLAC__StreamDecoderWriteStatus write_callback (
 		}
 	}
 
-	ringbuffer_head_add_samples (flacbufpos, frame->header.blocksize);
+	cpifaceSession->ringbufferAPI->head_add_samples (flacbufpos, frame->header.blocksize);
 
 	samples_for_bitrate += frame->header.blocksize;
 	samplerate_for_bitrate = frame->header.sample_rate;
@@ -335,6 +313,8 @@ static void metadata_callback(
 	void *client_data)
 #endif
 {
+	struct cpifaceSessionAPI_t *cpifaceSession = client_data;
+
 	switch (metadata->type)
 	{
 		case FLAC__METADATA_TYPE_STREAMINFO:
@@ -448,7 +428,7 @@ static void metadata_callback(
 			{
 				uint16_t actual_height, actual_width;
 				uint8_t *data_bgra;
-				if (!GIF87_try_open_bgra (&actual_width, &actual_height, &data_bgra, metadata->data.picture.data, metadata->data.picture.data_length))
+				if (!cpifaceSession->console->try_open_gif (&actual_width, &actual_height, &data_bgra, metadata->data.picture.data, metadata->data.picture.data_length))
 				{
 					add_picture (actual_width, actual_height, data_bgra, (const char *)metadata->data.picture.description, metadata->data.picture.type);
 				}
@@ -460,7 +440,7 @@ static void metadata_callback(
 			{
 				uint16_t actual_height, actual_width;
 				uint8_t *data_bgra;
-				if (!try_open_png (&actual_width, &actual_height, &data_bgra, metadata->data.picture.data, metadata->data.picture.data_length))
+				if (!cpifaceSession->console->try_open_png (&actual_width, &actual_height, &data_bgra, metadata->data.picture.data, metadata->data.picture.data_length))
 				{
 					add_picture (actual_width, actual_height, data_bgra, (const char *)metadata->data.picture.description, metadata->data.picture.type);
 				}
@@ -470,7 +450,7 @@ static void metadata_callback(
 			{
 				uint16_t actual_height, actual_width;
 				uint8_t *data_bgra;
-				if (!try_open_jpeg (&actual_width, &actual_height, &data_bgra, metadata->data.picture.data, metadata->data.picture.data_length))
+				if (!cpifaceSession->console->try_open_jpeg (&actual_width, &actual_height, &data_bgra, metadata->data.picture.data, metadata->data.picture.data_length))
 				{
 					add_picture (actual_width, actual_height, data_bgra, (const char *)metadata->data.picture.description, metadata->data.picture.type);
 				}
@@ -602,9 +582,9 @@ static void error_callback(
 	fprintf(stderr, "playflac: ERROR libflac: %s\n", FLAC__StreamDecoderErrorStatusString[status]);
 }
 
-static void flacIdler(void)
+static void flacIdler (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	while (ringbuffer_get_head_available_samples (flacbufpos) >= flac_max_blocksize)
+	while (cpifaceSession->ringbufferAPI->get_head_available_samples (flacbufpos) >= flac_max_blocksize)
 	{
 		uint64_t prePOS;
 		uint64_t postPOS;
@@ -649,499 +629,223 @@ static void flacIdler(void)
 			}
 		}
 		postPOS = flacfile->getpos (flacfile);
-		bitrate = (postPOS - prePOS) * 8 * samplerate_for_bitrate / samples_for_bitrate;
+		/* Due to logic above extra check is necessary on samples_for_bitrate */
+		bitrate = samples_for_bitrate != 0 ? (postPOS - prePOS) * 8 * samplerate_for_bitrate / samples_for_bitrate : 0;
 	}
 }
 
-void __attribute__ ((visibility ("internal"))) flacMetaDataLock(void)
+OCP_INTERNAL void flacMetaDataLock (void)
 {
 	clipbusy++;
 }
 
-void __attribute__ ((visibility ("internal"))) flacMetaDataUnlock(void)
+OCP_INTERNAL void flacMetaDataUnlock (void)
 {
 	clipbusy--;
 }
 
-void __attribute__ ((visibility ("internal"))) flacIdle(void)
+OCP_INTERNAL void flacIdle (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	uint32_t bufplayed;
-	uint32_t bufdelta;
-	uint32_t pass2;
-
 	if (clipbusy++)
 	{
 		clipbusy--;
 		return;
 	}
 
-	/* Where is our devp reading head? */
-	bufplayed=plrGetBufPos()>>(stereo+bit16);
-	bufdelta=(buflen+bufplayed-bufpos)%buflen; /* all these are in samples */
-
-	/* No delta on the devp? */
-	if (!bufdelta)
+	if (flac_inpause || (eof_buffer && eof_flacfile))
 	{
-		clipbusy--;
-		if (plrIdle)
-			plrIdle();
-		return;
-	}
-
-	/* fill up our buffers */
-	flacIdler();
-
-	if (inpause)
-	{
-		if ((bufpos+bufdelta)>buflen)
-			pass2=bufpos+bufdelta-buflen;
-		else
-			pass2=0;
-		if (bit16)
-		{
-			plrClearBuf((uint16_t *)plrbuf+(bufpos<<stereo), (bufdelta-pass2)<<stereo, !signedout);
-			if (pass2)
-				plrClearBuf((uint16_t *)plrbuf, pass2<<stereo, !signedout);
-		} else {
-			plrClearBuf(buf16, bufdelta<<stereo, !signedout);
-			plr16to8((uint8_t *)plrbuf+(bufpos<<stereo), buf16, (bufdelta-pass2)<<stereo);
-			if (pass2)
-				plr16to8((uint8_t *)plrbuf, buf16+((bufdelta-pass2)<<stereo), pass2<<stereo);
-		}
-		bufpos+=bufdelta;
-		if (bufpos>=buflen)
-			bufpos-=buflen;
+		cpifaceSession->plrDevAPI->Pause (1);
 	} else {
-		int pos1, length1, pos2, length2;
-		int i;
-		unsigned int buf16_filled=0;
+		void *targetbuf;
+		unsigned int targetlength; /* in samples */
 
-		/* how much data is available.. we are using a ringbuffer, so we might receive two fragments */
-		ringbuffer_get_tail_samples (flacbufpos, &pos1, &length1, &pos2, &length2);
+		cpifaceSession->plrDevAPI->Pause (0);
 
-		if (flacbufrate==0x10000) /* 1.0... just copy into buf16 direct until we run out of target buffer or source buffer */
+		cpifaceSession->plrDevAPI->GetBuffer (&targetbuf, &targetlength);
+
+		if (targetlength)
 		{
-			if (bufdelta>(length1+length2))
+			int16_t *t = targetbuf;
+			unsigned int accumulated_target = 0;
+			unsigned int accumulated_source = 0;
+			int pos1, length1, pos2, length2;
+
+			/* fill up our buffers */
+			flacIdler (cpifaceSession);
+
+			/* how much data is available.. we are using a ringbuffer, so we might receive two fragments */
+			cpifaceSession->ringbufferAPI->get_tail_samples (flacbufpos, &pos1, &length1, &pos2, &length2);
+
+			if (flacbufrate==0x10000)
 			{
-				bufdelta=(length1+length2);
-				eof_buffer=1;
+				if (targetlength>(length1+length2))
+				{
+					targetlength=(length1+length2);
+					eof_buffer=1;
+				} else {
+					eof_buffer=0;
+				}
+
+				// limit source to not overrun target buffer
+				if (length1 > targetlength)
+				{
+					length1 = targetlength;
+					length2 = 0;
+				} else if ((length1 + length2) > targetlength)
+				{
+					length2 = targetlength - length1;
+				}
+
+				accumulated_source = accumulated_target = length1 + length2;
+
+				while (length1)
+				{
+					while (length1)
+					{
+						int16_t rs, ls;
+
+						rs = flacbuf[(pos1<<1) + 0];
+						ls = flacbuf[(pos1<<1) + 1];
+
+						PANPROC;
+
+						*(t++) = rs;
+						*(t++) = ls;
+
+						pos1++;
+						length1--;
+
+						//accumulated_target++;
+					}
+					length1 = length2;
+					length2 = 0;
+					pos1 = pos2;
+					pos2 = 0;
+				}
+				//accumulated_source = accumulated_target;
 			} else {
+				/* We are going to perform cubic interpolation of rate conversion... this bit is tricky */
 				eof_buffer=0;
-			}
 
-			while (buf16_filled<bufdelta)
-			{
-				uint32_t w=bufdelta-buf16_filled;
-
-				if (!length1)
+				while (targetlength && length1)
 				{
-					pos1 = pos2;
+					while (targetlength && length1)
+					{
+						uint32_t wpm1, wp0, wp1, wp2;
+						int32_t rc0, rc1, rc2, rc3, rvm1,rv1,rv2;
+						int32_t lc0, lc1, lc2, lc3, lvm1,lv1,lv2;
+						unsigned int progress;
+						int16_t rs, ls;
+
+						if ((length1+length2) <= 3)
+						{
+							eof_buffer=1;
+							break;
+						}
+						/* will we overflow the flacbuf if we advance? */
+						if ((length1+length2) < ((flacbufrate+flacbuffpos)>>16))
+						{
+							eof_buffer=1;
+							break;
+						}
+
+						switch (length1) /* if we are close to the wrap between buffer segment 1 and 2, len1 will grow down to a small number */
+						{
+							case 1:  wpm1 = pos1; wp0 = pos2;     wp1 = pos2 + 1; wp2 = pos2 + 2; break;
+							case 2:  wpm1 = pos1; wp0 = pos1 + 1; wp1 = pos2;     wp2 = pos2 + 1; break;
+							case 3:  wpm1 = pos1; wp0 = pos1 + 1; wp1 = pos1 + 2; wp2 = pos2;     break;
+							default: wpm1 = pos1; wp0 = pos1 + 1; wp1 = pos1 + 2; wp2 = pos1 + 3; break;
+						}
+
+						rvm1 = (uint16_t)flacbuf[(wpm1<<1)+0]^0x8000; /* we temporary need data to be unsigned - hence the ^0x8000 */
+						lvm1 = (uint16_t)flacbuf[(wpm1<<1)+1]^0x8000;
+						 rc0 = (uint16_t)flacbuf[(wp0 <<1)+0]^0x8000;
+						 lc0 = (uint16_t)flacbuf[(wp0 <<1)+1]^0x8000;
+						 rv1 = (uint16_t)flacbuf[(wp1 <<1)+0]^0x8000;
+						 lv1 = (uint16_t)flacbuf[(wp1 <<1)+1]^0x8000;
+						 rv2 = (uint16_t)flacbuf[(wp2 <<1)+0]^0x8000;
+						 lv2 = (uint16_t)flacbuf[(wp2 <<1)+1]^0x8000;
+
+						rc1 = rv1-rvm1;
+						rc2 = 2*rvm1-2*rc0+rv1-rv2;
+						rc3 = rc0-rvm1-rv1+rv2;
+						rc3 =  imulshr16(rc3,flacbuffpos);
+						rc3 += rc2;
+						rc3 =  imulshr16(rc3,flacbuffpos);
+						rc3 += rc1;
+						rc3 =  imulshr16(rc3,flacbuffpos);
+						rc3 += rc0;
+						if (rc3<0)
+							rc3=0;
+						if (rc3>65535)
+							rc3=65535;
+
+						lc1 = lv1-lvm1;
+						lc2 = 2*lvm1-2*lc0+lv1-lv2;
+						lc3 = lc0-lvm1-lv1+lv2;
+						lc3 =  imulshr16(lc3,flacbuffpos);
+						lc3 += lc2;
+						lc3 =  imulshr16(lc3,flacbuffpos);
+						lc3 += lc1;
+						lc3 =  imulshr16(lc3,flacbuffpos);
+						lc3 += lc0;
+						if (lc3<0)
+							lc3=0;
+						if (lc3>65535)
+							lc3=65535;
+
+						rs = rc3 ^ 0x8000;
+						ls = lc3 ^ 0x8000;
+
+						PANPROC;
+
+						*(t++) = rs;
+						*(t++) = ls;
+
+						flacbuffpos+=flacbufrate;
+						progress = flacbuffpos>>16;
+						flacbuffpos&=0xFFFF;
+						accumulated_source+=progress;
+						pos1+=progress;
+						length1-=progress;
+						targetlength--;
+
+						accumulated_target++;
+					} /* while (targetlength && length1) */
 					length1 = length2;
-					pos2 = 0;
 					length2 = 0;
-				}
-
-				if (!length1)
-				{
-					fprintf (stderr, "playflac: ERROR, length1 == 0, in flacIdle\n");
-					_exit(1);
-				}
-
-				if (w>length1)
-				{
-					w=length1;
-				}
-				memcpy(buf16+2/*stereo*/*buf16_filled, flacbuf+pos1*2/*stereo*/, w*sizeof(uint16_t)*2/*stereo*/);
-				buf16_filled+=w;
-
-				length1-=w;
-				pos1+=w;
-
-				ringbuffer_tail_consume_samples (flacbufpos, w);
-			}
-		} else { /* re-sample intil we don't have more target-buffer or source-buffer */
-			unsigned int accumulated_progress = 0;
-
-			eof_buffer=0;
-
-			for (buf16_filled=0; buf16_filled<bufdelta; buf16_filled++)
-			{
-				int32_t c0, c1, c2, c3, ls, rs, vm1, v1, v2;
-				uint32_t wpm1, wp0, wp1, wp2;
-				unsigned int progress;
-
-				/* will the interpolation overflow? */
-				if ((length1+length2) <= 3)
-				{
-					eof_buffer=1;
-					break;
-				}
-				/* will we overflow the flacbuf if we advance? */
-				if ((length1+length2) < ((flacbufrate+flacbuffpos)>>16))
-				{
-					eof_buffer=1;
-					break;
-				}
-
-				switch (length1)
-				{
-					case 1:
-						wpm1 = pos1;
-						wp0  = pos2;
-						wp1  = pos2+1;
-						wp2  = pos2+2;
-						break;
-					case 2:
-						wpm1 = pos1;
-						wp0  = pos1+1;
-						wp1  = pos2;
-						wp2  = pos2+1;
-						break;
-					case 3:
-						wpm1 = pos1;
-						wp0  = pos1+1;
-						wp1  = pos1+2;
-						wp2  = pos2;
-						break;
-					default:
-						wpm1 = pos1;
-						wp0  = pos1+1;
-						wp1  = pos1+2;
-						wp2  = pos1+3;
-						break;
-				}
-
-				vm1= (unsigned)(flacbuf[wpm1*2])^0x8000;
-				c0 = (unsigned)(flacbuf[wp0*2])^0x8000;
-				v1 = (unsigned)(flacbuf[wp1*2])^0x8000;
-				v2 = (unsigned)(flacbuf[wp2*2])^0x8000;
-				c1 = v1-vm1;
-				c2 = 2*vm1-2*c0+v1-v2;
-				c3 = c0-vm1-v1+v2;
-				c3 =  imulshr16(c3,flacbuffpos);
-				c3 += c2;
-				c3 =  imulshr16(c3,flacbuffpos);
-				c3 += c1;
-				c3 =  imulshr16(c3,flacbuffpos);
-				ls = c3+c0;
-				if (ls>65535)
-					ls=65535;
-				else if (ls<0)
-					ls=0;
-
-				vm1= (unsigned)(flacbuf[wpm1*2+1])^0x8000;
-				c0 = (unsigned)(flacbuf[wp0*2+1])^0x8000;
-				v1 = (unsigned)(flacbuf[wp1*2+1])^0x8000;
-				v2 = (unsigned)(flacbuf[wp2*2+1])^0x8000;
-				c1 = v1-vm1;
-				c2 = 2*vm1-2*c0+v1-v2;
-				c3 = c0-vm1-v1+v2;
-				c3 =  imulshr16(c3,flacbuffpos);
-				c3 += c2;
-				c3 =  imulshr16(c3,flacbuffpos);
-				c3 += c1;
-				c3 =  imulshr16(c3,flacbuffpos);
-				rs = c3+c0;
-				if (rs>65535)
-					rs=65535;
-				else if(rs<0)
-					rs=0;
-				buf16[buf16_filled*2+0]=(uint16_t)ls^0x8000;
-				buf16[buf16_filled*2+1]=(uint16_t)rs^0x8000;
-
-				flacbuffpos+=flacbufrate;
-				progress = flacbuffpos>>16;
-				flacbuffpos&=0xFFFF;
-
-				accumulated_progress += progress;
-
-				/* did we wrap? if so, progress up to the wrapping point */
-				if (progress >= length1)
-				{
-					progress -= length1;
 					pos1 = pos2;
-					length1 = length2;
 					pos2 = 0;
-					length2 = 0;
-				}
-				if (progress)
-				{
-					pos1 += progress;
-					length1 -= progress;
-				}
-			}
-			ringbuffer_tail_consume_samples (flacbufpos, accumulated_progress);
-		}
-
-		bufdelta=buf16_filled;
-
-		/* when we copy out of buf16, pass the buffer-len that wraps around end-of-buffer till pass2 */
-		if ((bufpos+bufdelta)>buflen)
-			pass2=bufpos+bufdelta-buflen;
-		else
-			pass2=0;
-		bufdelta-=pass2;
-
-		if (bit16)
-		{
-			if (stereo)
-			{
-				if (reversestereo)
-				{
-					int16_t *p=(int16_t *)plrbuf+2*bufpos;
-					int16_t *b=(int16_t *)buf16;
-					if (signedout)
-					{
-						for (i=0; i<bufdelta; i++)
-						{
-							p[0]=b[1];
-							p[1]=b[0];
-							p+=2;
-							b+=2;
-						}
-						p=(int16_t *)plrbuf;
-						for (i=0; i<pass2; i++)
-						{
-							p[0]=b[1];
-							p[1]=b[0];
-							p+=2;
-							b+=2;
-						}
-					} else {
-						for (i=0; i<bufdelta; i++)
-						{
-							p[0]=b[1]^0x8000;
-							p[1]=b[0]^0x8000;
-							p+=2;
-							b+=2;
-						}
-						p=(int16_t *)plrbuf;
-						for (i=0; i<pass2; i++)
-						{
-							p[0]=b[1]^0x8000;
-							p[1]=b[0]^0x8000;
-							p+=2;
-							b+=2;
-						}
-					}
-				} else {
-					int16_t *p=(int16_t *)plrbuf+2*bufpos;
-					int16_t *b=(int16_t *)buf16;
-					if (signedout)
-					{
-						for (i=0; i<bufdelta; i++)
-						{
-							p[0]=b[0];
-							p[1]=b[1];
-							p+=2;
-							b+=2;
-						}
-						p=(int16_t *)plrbuf;
-						for (i=0; i<pass2; i++)
-						{
-							p[0]=b[0];
-							p[1]=b[1];
-							p+=2;
-							b+=2;
-						}
-					} else {
-						for (i=0; i<bufdelta; i++)
-						{
-							p[0]=b[0]^0x8000;
-							p[1]=b[1]^0x8000;
-							p+=2;
-							b+=2;
-						}
-						p=(int16_t *)plrbuf;
-						for (i=0; i<pass2; i++)
-						{
-							p[0]=b[0]^0x8000;
-							p[1]=b[1]^0x8000;
-							p+=2;
-							b+=2;
-						}
-					}
-				}
-			} else {
-				int16_t *p=(int16_t *)plrbuf+bufpos;
-				int16_t *b=(int16_t *)buf16;
-				if (signedout)
-				{
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[0];
-						p++;
-						b++;
-					}
-					p=(int16_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[0];
-						p++;
-						b++;
-					}
-				} else {
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[0]^0x8000;
-						p++;
-						b++;
-					}
-					p=(int16_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[0]^0x8000;
-						p++;
-						b++;
-					}
-				}
-			}
-		} else {
-			if (stereo)
-			{
-				if (reversestereo)
-				{
-					uint8_t *p=(uint8_t *)plrbuf+2*bufpos;
-					uint8_t *b=(uint8_t *)buf16;
-					if (signedout)
-					{
-						for (i=0; i<bufdelta; i++)
-						{
-							p[0]=b[3];
-							p[1]=b[1];
-							p+=2;
-							b+=4;
-						}
-						p=(uint8_t *)plrbuf;
-						for (i=0; i<pass2; i++)
-						{
-							p[0]=b[3];
-							p[1]=b[1];
-							p+=2;
-							b+=4;
-						}
-					} else {
-						for (i=0; i<bufdelta; i++)
-						{
-							p[0]=b[3]^0x80;
-							p[1]=b[1]^0x80;
-							p+=2;
-							b+=4;
-						}
-						p=(uint8_t *)plrbuf;
-						for (i=0; i<pass2; i++)
-						{
-							p[0]=b[3]^0x80;
-							p[1]=b[1]^0x80;
-							p+=2;
-							b+=4;
-						}
-					}
-				} else {
-					uint8_t *p=(uint8_t *)plrbuf+2*bufpos;
-					uint8_t *b=(uint8_t *)buf16;
-					if (signedout)
-					{
-						for (i=0; i<bufdelta; i++)
-						{
-							p[0]=b[1];
-							p[1]=b[3];
-							p+=2;
-							b+=4;
-						}
-						p=(uint8_t *)plrbuf;
-						for (i=0; i<pass2; i++)
-						{
-							p[0]=b[1];
-							p[1]=b[3];
-							p+=2;
-							b+=4;
-						}
-					} else {
-						for (i=0; i<bufdelta; i++)
-						{
-							p[0]=b[1]^0x80;
-							p[1]=b[3]^0x80;
-							p+=2;
-							b+=4;
-						}
-						p=(uint8_t *)plrbuf;
-						for (i=0; i<pass2; i++)
-						{
-							p[0]=b[1]^0x80;
-							p[1]=b[3]^0x80;
-							p+=2;
-							b+=4;
-						}
-					}
-				}
-			} else {
-				uint8_t *p=(uint8_t *)plrbuf+bufpos;
-				uint8_t *b=(uint8_t *)buf16;
-				if (signedout)
-				{
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[1];
-						p++;
-						b+=2;
-					}
-					p=(uint8_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[1];
-						p++;
-						b+=2;
-					}
-				} else {
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[1]^0x80;
-						p++;
-						b+=2;
-					}
-					p=(uint8_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[1]^0x80;
-						p++;
-						b+=2;
-					}
-				}
-			}
-		}
-
-		bufpos+=buf16_filled;
-		if (bufpos>=buflen)
-			bufpos-=buflen;
+				} /* while (targetlength && length1) */
+			} /* if (flacbufrate==0x10000) */
+			cpifaceSession->ringbufferAPI->tail_consume_samples (flacbufpos, accumulated_source);
+			cpifaceSession->plrDevAPI->CommitBuffer (accumulated_target);
+		} /* if (targetlength) */
 	}
 
-	plrAdvanceTo(bufpos<<(stereo+bit16));
-
-	if (plrIdle)
-		plrIdle();
+	cpifaceSession->plrDevAPI->Idle();
 
 	clipbusy--;
 }
 
-void __attribute__ ((visibility ("internal"))) flacSetLoop(uint8_t s)
+OCP_INTERNAL void flacSetLoop (uint8_t s)
 {
 	donotloop=!s;
 }
-int __attribute__ ((visibility ("internal"))) flacIsLooped(void)
+OCP_INTERNAL int flacIsLooped (void)
 {
 	return eof_buffer&&eof_flacfile;
 }
-void __attribute__ ((visibility ("internal"))) flacPause(int p)
+OCP_INTERNAL void flacPause (int p)
 {
-	inpause=p;
+	flac_inpause=p;
 }
 
 static void flacSetSpeed(uint16_t sp)
 {
-	if (sp<32)
-		sp=32;
-	flacbufrate=imuldiv(256*sp, flacrate, plrRate);
+	if (sp < 4)
+		sp = 4;
+	flacbufrate=imuldiv(256*sp, flacrate, flacRate);
 }
 
 static void flacSetVolume(void)
@@ -1153,7 +857,7 @@ static void flacSetVolume(void)
 		voll=(voll*(64-bal))>>6;
 }
 
-static void SET(int ch, int opt, int val)
+static void flacSet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt, int val)
 {
 	switch (opt)
 	{
@@ -1167,10 +871,6 @@ static void SET(int ch, int opt, int val)
 			break;
 		case mcpMasterPanning:
 			pan=val;
-			if (reversestereo)
-			{
-				pan = -pan;
-			}
 			flacSetVolume();
 			break;
 		case mcpMasterVolume:
@@ -1183,12 +883,13 @@ static void SET(int ch, int opt, int val)
 			break;
 	}
 }
-static int GET(int ch, int opt)
+
+static int flacGet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt)
 {
 	return 0;
 }
 
-void __attribute__ ((visibility ("internal"))) flacGetInfo(struct flacinfo *info)
+OCP_INTERNAL void flacGetInfo (struct flacinfo *info)
 {
 	info->pos=flaclastpos;
 	info->len=samples;
@@ -1200,11 +901,11 @@ void __attribute__ ((visibility ("internal"))) flacGetInfo(struct flacinfo *info
 	snprintf (info->opt50, sizeof (info->opt50), "%s - %s", FLAC__VERSION_STRING, FLAC__VENDOR_STRING);
 	info->bitrate=bitrate;
 }
-uint64_t __attribute__ ((visibility ("internal"))) flacGetPos(void)
+OCP_INTERNAL uint64_t flacGetPos (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	return (flaclastpos + samples - ringbuffer_get_tail_available_samples (flacbufpos)) % samples;
+	return (flaclastpos + samples - cpifaceSession->ringbufferAPI->get_tail_available_samples (flacbufpos)) % samples;
 }
-void __attribute__ ((visibility ("internal"))) flacSetPos(uint64_t pos)
+OCP_INTERNAL void flacSetPos (uint64_t pos)
 {
 	if (pos>=samples)
 	{
@@ -1219,199 +920,10 @@ void __attribute__ ((visibility ("internal"))) flacSetPos(uint64_t pos)
 	flacPendingSeekPos = pos;
 }
 
-int __attribute__ ((visibility ("internal"))) flacOpenPlayer(struct ocpfilehandle_t *file)
-{
-	int temp;
-	uint32_t flacbuflen;
-
-	if (flacfile)
-	{
-		flacfile->unref (flacfile);
-		flacfile = 0;
-	}
-	flacfile = file;
-	flacfile->ref (flacfile);
-
-	inpause=0;
-	voll=256;
-	volr=256;
-	bal=0;
-	vol=64;
-	pan=64;
-	srnd=0;
-	eof_flacfile=0;
-	eof_buffer=0;
-
-	buf16=0;
-	flacbuf=0;
-	flacbufpos = 0;
-#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
-	decoder = FLAC__seekable_stream_decoder_new();
-#else
-	decoder = FLAC__stream_decoder_new();
-#endif
-	if (!decoder)
-	{
-		fprintf(stderr, "playflac: FLAC__seekable_stream_decoder_new() failed, out of memory?\n");
-		return 0;
-	}
-
-	FLAC__stream_decoder_set_metadata_respond_all (decoder);
-
-	flac_max_blocksize=0;
-	flacrate=0;
-	flacstereo=1;
-
-#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
-	FLAC__seekable_stream_decoder_set_md5_checking(decoder, 0);
-
-	FLAC__seekable_stream_decoder_set_read_callback(decoder, read_callback);
-	FLAC__seekable_stream_decoder_set_write_callback(decoder, write_callback);
-	FLAC__seekable_stream_decoder_set_metadata_callback(decoder, metadata_callback);
-	FLAC__seekable_stream_decoder_set_seek_callback(decoder, seek_callback);
-	FLAC__seekable_stream_decoder_set_tell_callback(decoder, tell_callback);
-	FLAC__seekable_stream_decoder_set_length_callback(decoder, length_callback);
-	FLAC__seekable_stream_decoder_set_eof_callback(decoder, eof_callback);
-	FLAC__seekable_stream_decoder_set_client_data(decoder, 0);
-	FLAC__seekable_stream_decoder_set_error_callback(decoder, error_callback);
-	if ((temp=FLAC__seekable_stream_decoder_init(decoder))!=FLAC__SEEKABLE_STREAM_DECODER_OK)
-	{
-		fprintf(stderr, "playflac: FLAC__seekable_stream_decoder_init() failed, %s\n", FLAC__SeekableStreamDecoderStateString[temp]);
-		FLAC__seekable_stream_decoder_delete(decoder);
-		decoder = NULL;
-		goto error_out;
-	}
-	if (!FLAC__seekable_stream_decoder_process_until_end_of_metadata(decoder))
-	{
-		fprintf(stderr, "playflac: FLAC__seekable_stream_decoder_process_until_end_of_metadata() failed\n");
-		goto error_out;
-	}
-#else
-	FLAC__stream_decoder_set_md5_checking(decoder, true);
-	if((temp=FLAC__stream_decoder_init_stream(
-	   decoder,
-	   read_callback,
-	   seek_callback,
-	   tell_callback,
-	   length_callback,
-	   eof_callback,
-	   write_callback,
-	   metadata_callback,
-	   error_callback,
-	   0 /*my_client_data*/
-	)) != FLAC__STREAM_DECODER_INIT_STATUS_OK)
-	{
-		fprintf(stderr, "playflac: FLAC__stream_decoder_init_stream() failed, %s\n", FLAC__StreamDecoderStateString[temp]);
-		FLAC__stream_decoder_delete(decoder);
-		decoder = NULL;
-		goto error_out;
-	}
-	if (!FLAC__stream_decoder_process_until_end_of_metadata(decoder))
-	{
-		fprintf(stderr, "playflac: FLAC__seekable_stream_decoder_process_until_end_of_metadata() failed\n");
-		goto error_out;
-	}
-#endif
-
-	if (flac_max_blocksize<=0)
-	{
-		fprintf(stderr, "playflac: max blocksize not set\n");
-		goto error_out;
-	}
-
-	plrSetOptions(flacrate, (PLR_SIGNEDOUT|PLR_16BIT)|PLR_STEREO);
-
-	stereo=!!(plrOpt&PLR_STEREO);
-	bit16=!!(plrOpt&PLR_16BIT);
-	signedout=!!(plrOpt&PLR_SIGNEDOUT);
-	reversestereo=!!(plrOpt&PLR_REVERSESTEREO);
-
-	flacbufrate=imuldiv(65536, flacrate, plrRate);
-
-	flacbuflen = flac_max_blocksize * 2 /* we need to be able to fit two buffers in here */ + 64 /* slack */;
-	if (flacbuflen<8192)
-		flacbuflen=8192;
-	if (!(flacbuf=malloc(flacbuflen*sizeof(uint16_t)*2/*stereo*/)))
-	{
-		fprintf(stderr, "playflac: malloc() failed\n");
-		goto error_out;
-	}
-
-	flacbufpos = ringbuffer_new_samples (RINGBUFFER_FLAGS_16BIT | RINGBUFFER_FLAGS_STEREO, flacbuflen);
-	flacbuffpos=0;
-
-	if (!plrOpenPlayer(&plrbuf, &buflen, plrBufSize * plrRate / 1000, file))
-	{
-		fprintf(stderr, "playflac: plrOpenPlayer() failed\n");
-		goto error_out;
-	}
-
-	if (!(buf16=malloc(sizeof(uint16_t) * buflen * 2 /* stereo */)))
-	{
-		fprintf(stderr, "playflac: malloc() failed\n");
-		goto error_out;
-	}
-	bufpos=0;
-
-	if (!pollInit(flacIdle))
-	{
-		fprintf(stderr, "playflac: pollInit failed\n");
-		goto error_out;
-	}
-
-	_SET=mcpSet;
-	_GET=mcpGet;
-	mcpSet=SET;
-	mcpGet=GET;
-
-	mcpNormalize (mcpNormalizeDefaultPlayP);
-
-	return 1;
-
-error_out:
-	plrClosePlayer();
-	return 0;
-}
-
-void __attribute__ ((visibility ("internal"))) flacClosePlayer(void)
+static void flacFreeComments (void)
 {
 	int i, j;
 
-	pollClose();
-	plrClosePlayer();
-
-	if (flacbuf)
-	{
-		free(flacbuf);
-		flacbuf=0;
-	}
-	if (flacbufpos)
-	{
-		ringbuffer_free (flacbufpos);
-		flacbufpos = 0;
-	}
-	if (buf16)
-	{
-		free(buf16);
-		buf16=0;
-	}
-
-	if (flacfile)
-	{
-		flacfile->unref (flacfile);
-		flacfile = 0;
-	}
-	if (!decoder)
-		return;
-#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
-	FLAC__seekable_stream_decoder_finish(decoder);
-	FLAC__seekable_stream_decoder_delete(decoder);
-#else
-	FLAC__stream_decoder_finish(decoder);
-	FLAC__stream_decoder_delete(decoder);
-#endif
-	decoder = NULL;
-
 	for (i=0; i < flac_comments_count; i++)
 	{
 		for (j=0; j < flac_comments[i]->value_count; j++)
@@ -1434,15 +946,205 @@ void __attribute__ ((visibility ("internal"))) flacClosePlayer(void)
 	free (flac_pictures);
 	flac_pictures = 0;
 	flac_pictures_count = 0;
+}
 
-	if (_SET)
+OCP_INTERNAL int flacOpenPlayer (struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
+{
+	enum plrRequestFormat format;
+	int temp;
+	uint32_t flacbuflen;
+	int retval;
+
+	if (!cpifaceSession->plrDevAPI)
+	{
+		return errPlay;
+	}
+
+	flacfile = file;
+	flacfile->ref (flacfile);
+
+	flac_inpause=0;
+	voll=256;
+	volr=256;
+	bal=0;
+	vol=64;
+	pan=64;
+	srnd=0;
+	eof_flacfile=0;
+	eof_buffer=0;
+
+	flacbuf=0;
+	flacbufpos = 0;
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
+	decoder = FLAC__seekable_stream_decoder_new();
+#else
+	decoder = FLAC__stream_decoder_new();
+#endif
+	if (!decoder)
+	{
+		cpifaceSession->cpiDebug (cpifaceSession, "[FLAC] FLAC__seekable_stream_decoder_new() failed, out of memory?\n");
+		retval = errFileRead;
+		goto error_out_flacfile;
+	}
+
+	FLAC__stream_decoder_set_metadata_respond_all (decoder);
+
+	flac_max_blocksize=0;
+	flacrate=0;
+	flacstereo=1;
+
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
+	FLAC__seekable_stream_decoder_set_md5_checking(decoder, 0);
+
+	FLAC__seekable_stream_decoder_set_read_callback(decoder, read_callback);
+	FLAC__seekable_stream_decoder_set_write_callback(decoder, write_callback);
+	FLAC__seekable_stream_decoder_set_metadata_callback(decoder, metadata_callback);
+	FLAC__seekable_stream_decoder_set_seek_callback(decoder, seek_callback);
+	FLAC__seekable_stream_decoder_set_tell_callback(decoder, tell_callback);
+	FLAC__seekable_stream_decoder_set_length_callback(decoder, length_callback);
+	FLAC__seekable_stream_decoder_set_eof_callback(decoder, eof_callback);
+	FLAC__seekable_stream_decoder_set_client_data(decoder, cpifaceSession);
+	FLAC__seekable_stream_decoder_set_error_callback(decoder, error_callback);
+	if ((temp=FLAC__seekable_stream_decoder_init(decoder))!=FLAC__SEEKABLE_STREAM_DECODER_OK)
+	{
+		cpifaceSession->cpiDebug (cpifaceSession, "[FLAC] FLAC__seekable_stream_decoder_init() failed, %s\n", FLAC__SeekableStreamDecoderStateString[temp]);
+		retval = errFormStruc;
+		goto error_out_decoder;
+	}
+	if (!FLAC__seekable_stream_decoder_process_until_end_of_metadata(decoder))
+	{
+		cpifaceSession->cpiDebug (cpifaceSession, "[FLAC] FLAC__seekable_stream_decoder_process_until_end_of_metadata() failed\n");
+		retval = errFormStruc;
+		goto error_out_decoder;
+	}
+#else
+	FLAC__stream_decoder_set_md5_checking(decoder, true);
+	if((temp=FLAC__stream_decoder_init_stream(
+	   decoder,
+	   read_callback,
+	   seek_callback,
+	   tell_callback,
+	   length_callback,
+	   eof_callback,
+	   write_callback,
+	   metadata_callback,
+	   error_callback,
+	   cpifaceSession /*my_client_data*/
+	)) != FLAC__STREAM_DECODER_INIT_STATUS_OK)
+	{
+		cpifaceSession->cpiDebug (cpifaceSession, "[FLAC] FLAC__stream_decoder_init_stream() failed, %s\n", FLAC__StreamDecoderStateString[temp]);
+		retval = errFormStruc;
+		goto error_out_decoder;
+	}
+	if (!FLAC__stream_decoder_process_until_end_of_metadata(decoder))
+	{
+		cpifaceSession->cpiDebug (cpifaceSession, "[FLAC] FLAC__seekable_stream_decoder_process_until_end_of_metadata() failed\n");
+		retval = errFormStruc;
+		goto error_out_decoder;
+	}
+#endif
+
+	if (flac_max_blocksize<=0)
+	{
+		cpifaceSession->cpiDebug (cpifaceSession, "[FLAC] max blocksize not set\n");
+		retval = errFormStruc;
+		goto error_out_decoder;
+	}
+
+	flacRate=flacrate;
+	format=PLR_STEREO_16BIT_SIGNED;
+	if (!cpifaceSession->plrDevAPI->Play (&flacRate, &format, file, cpifaceSession))
+	{
+		cpifaceSession->cpiDebug (cpifaceSession, "[FLAC] plrOpenPlayer() failed\n");
+		retval = errPlay;
+		goto error_out_decoder;
+	}
+
+	flacbufrate=imuldiv(65536, flacrate, flacRate);
+	flacbuflen = flac_max_blocksize * 2 /* we need to be able to fit two buffers in here */ + 64 /* slack */;
+	if (flacbuflen<8192)
+		flacbuflen=8192;
+	if (!(flacbuf=malloc(flacbuflen*sizeof(uint16_t)*2/*stereo*/)))
 	{
-		mcpSet = _SET;
-		_SET = 0;
+		cpifaceSession->cpiDebug (cpifaceSession, "[FLAC] malloc() failed\n");
+		retval = errAllocMem;
+		goto error_out_plrDevAPI_Start;
 	}
-	if (_GET)
+
+	flacbufpos = cpifaceSession->ringbufferAPI->new_samples (RINGBUFFER_FLAGS_16BIT | RINGBUFFER_FLAGS_STEREO, flacbuflen);
+	if (!flacbufpos)
 	{
-		mcpGet = _GET;
-		_GET = 0;
+		cpifaceSession->cpiDebug (cpifaceSession, "[FLAC] ringbuffer_new_samples() failed\n");
+		retval = errAllocMem;
+		goto error_out_flacbuf;
 	}
+	flacbuffpos=0;
+
+	cpifaceSession->mcpSet=flacSet;
+	cpifaceSession->mcpGet=flacGet;
+
+	cpifaceSession->Normalize (cpifaceSession, mcpNormalizeDefaultPlayP);
+
+	return errOk;
+
+	//cpifaceSession->ringbufferAPI->free (flacbufpos);
+	//flacbufpos = 0;
+error_out_flacbuf:
+	free (flacbuf);
+	flacbuf = 0;
+error_out_plrDevAPI_Start:
+	cpifaceSession->plrDevAPI->Stop (cpifaceSession);
+error_out_decoder:
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
+	FLAC__seekable_stream_decoder_finish(decoder);
+	FLAC__seekable_stream_decoder_delete(decoder);
+#else
+	FLAC__stream_decoder_finish(decoder);
+	FLAC__stream_decoder_delete(decoder);
+#endif
+	decoder = NULL;
+error_out_flacfile:
+	flacfile->unref (flacfile);
+	flacfile = 0;
+
+	flacFreeComments ();
+
+	return retval;
+}
+
+OCP_INTERNAL void flacClosePlayer (struct cpifaceSessionAPI_t *cpifaceSession)
+{
+	if (cpifaceSession->plrDevAPI)
+	{
+		cpifaceSession->plrDevAPI->Stop (cpifaceSession);
+	}
+
+	if (flacbuf)
+	{
+		free(flacbuf);
+		flacbuf=0;
+	}
+	if (flacbufpos)
+	{
+		cpifaceSession->ringbufferAPI->free (flacbufpos);
+		flacbufpos = 0;
+	}
+
+	if (flacfile)
+	{
+		flacfile->unref (flacfile);
+		flacfile = 0;
+	}
+	if (!decoder)
+		return;
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
+	FLAC__seekable_stream_decoder_finish(decoder);
+	FLAC__seekable_stream_decoder_delete(decoder);
+#else
+	FLAC__stream_decoder_finish(decoder);
+	FLAC__stream_decoder_delete(decoder);
+#endif
+	decoder = NULL;
+
+	flacFreeComments ();
 }
