@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * GMDPlay - generic modular module player
  *
@@ -35,10 +35,10 @@
 #include "cpiface/cpiface.h"
 #include "dev/mcp.h"
 #include "gmdplay.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
 
-#define MAXLCHAN MP_MAXCHANNELS
-#define MAXPCHAN 32
+#define GMD_MAXPCHAN 32
 
 struct trackdata
 {
@@ -111,7 +111,7 @@ struct trackdata
 	int mute;
 };
 
-static int pchan[MAXPCHAN];
+static int pchan[GMD_MAXPCHAN];
 
 
 static uint16_t notetab[16]={32768,30929,29193,27554,26008,24548,23170,21870,20643,19484,18390,17358,16384,15464,14596,13777};
@@ -166,7 +166,7 @@ static int lockpattern;
 static uint16_t patternnum;
 static uint16_t looppat;
 static uint16_t endpat;
-static struct trackdata tdata[MP_MAXCHANNELS];
+static struct trackdata tdata[GMD_MAXLCHAN];
 static struct trackdata *tdataend;
 static struct gmdtrack gtrack;
 static const struct gmdenvelope *envelopes;
@@ -185,13 +185,13 @@ static int16_t brkpat;
 static int16_t brkrow;
 static uint8_t newtickmode;
 static uint8_t processtick;
-static uint8_t patlooprow[MAXLCHAN];
-static uint8_t patloopcount[MAXLCHAN];
+static uint8_t patlooprow[GMD_MAXLCHAN];
+static uint8_t patloopcount[GMD_MAXLCHAN];
 static uint8_t globchan;
 static uint8_t patdelay;
 static uint8_t globalvol;
-static uint8_t globalvolslide[MAXLCHAN];
-static int8_t globalvolslval[MAXLCHAN];
+static uint8_t globalvolslide[GMD_MAXLCHAN];
+static int8_t globalvolslval[GMD_MAXLCHAN];
 static uint8_t looped;
 static uint8_t exponential;
 static uint8_t samiextrawurscht;
@@ -212,10 +212,10 @@ static int quelen;
 #define EPITCHMIN -72*256
 #define EPITCHMAX 96*256
 
-static void readque(void)
+static void readque (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int type,val1/*,val2*/;
-	int time=mcpGet(-1, mcpGTimer);
+	int time = cpifaceSession->mcpGet (cpifaceSession, -1, mcpGTimer);
 	while (1)
 	{
 		if (querpos==quewpos)
@@ -292,7 +292,7 @@ static inline signed long checkpitch(int32_t pitch)
 		return checkpitchh(pitch);
 }
 
-static uint8_t PlayNote(struct trackdata *t, const uint8_t *dat)
+static uint8_t PlayNote (struct cpifaceSessionAPI_t *cpifaceSession, struct trackdata *t, const uint8_t *dat)
 {
 	const uint8_t *od=dat;
 	int16_t ins=-1;
@@ -362,7 +362,7 @@ static uint8_t PlayNote(struct trackdata *t, const uint8_t *dat)
 		{
 			if (t->phys!=-1)
 			{
-				mcpSet(t->phys, mcpCReset, 0);
+				cpifaceSession->mcpSet (cpifaceSession, t->phys, mcpCReset, 0);
 				pchan[t->phys]=-1;
 				t->phys=-1;
 			}
@@ -397,7 +397,7 @@ static uint8_t PlayNote(struct trackdata *t, const uint8_t *dat)
 		if (exponential)
 			t->pitchslidepitch=t->finalpitch=t->pitch=60*256-(t->nteval<<8)+t->samp->normnote;
 		else
-			t->pitchslidepitch=t->finalpitch=t->pitch=mcpGetFreq6848(60*256+t->samp->normnote-(t->nteval<<8));
+			t->pitchslidepitch=t->finalpitch=t->pitch=cpifaceSession->mcpAPI->GetFreq6848(60*256+t->samp->normnote-(t->nteval<<8));
 		if (samiextrawurscht&&t->insofs)
 			pos=(t->samp->opt&MP_OFFSETDIV2)?(t->insofs>>1):t->insofs;
 		t->newpos=pos;
@@ -412,7 +412,7 @@ static uint8_t PlayNote(struct trackdata *t, const uint8_t *dat)
 			if (exponential)
 				t->pitchslidepitch=60*256-(t->nteval<<8)+t->samp->normnote;
 			else
-				t->pitchslidepitch=mcpGetFreq6848(60*256+t->samp->normnote-(t->nteval<<8));
+				t->pitchslidepitch=cpifaceSession->mcpAPI->GetFreq6848(60*256+t->samp->normnote-(t->nteval<<8));
 		}
 	}
 
@@ -422,7 +422,7 @@ static uint8_t PlayNote(struct trackdata *t, const uint8_t *dat)
 	return dat-od;
 }
 
-static void PlayGCommand(const uint8_t *cmd, uint8_t len)
+static void PlayGCommand (struct cpifaceSessionAPI_t *cpifaceSession, const uint8_t *cmd, uint8_t len)
 {
 	const uint8_t *cend=cmd+len;
 	while (cmd<cend)
@@ -434,10 +434,10 @@ static void PlayGCommand(const uint8_t *cmd, uint8_t len)
 				break;
 			case cmdSpeed:
 				speed=*cmd;
-				mcpSet(-1, mcpGSpeed, 256*2*speed/5);
+				cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, 256*2*speed/5);
 				break;
 			case cmdFineSpeed:
-				mcpSet(-1, mcpGSpeed, 256*2*(10*speed+*cmd)/50);
+				cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, 256*2*(10*speed+*cmd)/50);
 				break;
 			case cmdBreak:
 				if (brkpat==-1)
@@ -496,14 +496,14 @@ static void PlayGCommand(const uint8_t *cmd, uint8_t len)
 	}
 }
 
-static void PlayCommand(struct trackdata *t, const uint8_t *cmd, uint8_t len)
+static void PlayCommand (struct cpifaceSessionAPI_t *cpifaceSession, struct trackdata *t, const uint8_t *cmd, uint8_t len)
 {
 	const uint8_t *cend=cmd+len;
 	while (cmd<cend)
 	{
 		if (*cmd&cmdPlayNote)
 		{
-			cmd+=PlayNote(t, cmd);
+			cmd+=PlayNote (cpifaceSession, t, cmd);
 			continue;
 		}
 		switch (*cmd++)
@@ -897,16 +897,16 @@ static void PlayCommand(struct trackdata *t, const uint8_t *cmd, uint8_t len)
 static void DoGCommand(void)
 {
 	int i;
-	for (i=0; i<MAXLCHAN; i++)
+	for (i=0; i<GMD_MAXLCHAN; i++)
 		if (globalvolslide[i])
 			if (processtick)
 				globalvol=checkvol(globalvol+globalvolslval[i]);
 }
 
-static void DoCommand(struct trackdata *t)
+static void DoCommand (struct cpifaceSessionAPI_t *cpifaceSession, struct trackdata *t)
 {
 	if (t->delay==currenttick)
-		PlayNote(t, t->delaycmd);
+		PlayNote (cpifaceSession, t, t->delaycmd);
 
 	switch (t->volslide)
 	{
@@ -963,7 +963,7 @@ static void DoCommand(struct trackdata *t)
 				if (exponential)
 					t->finalpitch=((t->pitch+0x80-t->samp->normnote)&~0xFF)+t->samp->normnote;
 				else
-					t->finalpitch=mcpGetFreq6848(((mcpGetNote6848(t->pitch)+0x80-t->samp->normnote)&~0xFF)+t->samp->normnote);
+					t->finalpitch=cpifaceSession->mcpAPI->GetFreq6848(((cpifaceSession->mcpAPI->GetNote6848(t->pitch)+0x80-t->samp->normnote)&~0xFF)+t->samp->normnote);
 			else
 				t->finalpitch=t->pitch;
 			break;
@@ -1123,21 +1123,27 @@ static void putque(int time, int type, int val1, int val2)
 	quewpos=(quewpos+1)%quelen;
 }
 
-static void PlayTick(void)
+static void PlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	struct trackdata *td;
 	int i;
 
 	int cmdtime;
 
+	if (donotloopmodule && looped)
+	{
+		cpifaceSession->mcpSet (cpifaceSession, -1, mcpMasterPause, 1);
+		return;
+	}
+
 	if (!physchan)
 		return;
 
 	for (i=0; i<physchan; i++)
-		if (!mcpGet(i, mcpCStatus))
+		if (!cpifaceSession->mcpGet (cpifaceSession, i, mcpCStatus))
 			if (pchan[i]!=-1)
 			{
-				mcpSet(i, mcpCReset, 0);
+				cpifaceSession->mcpSet (cpifaceSession, i, mcpCReset, 0);
 				tdata[pchan[i]].phys=-1;
 				pchan[i]=-1;
 			}
@@ -1230,13 +1236,13 @@ static void PlayTick(void)
 				}
 				for (i=0; i<physchan; i++)
 				{
-					mcpSet(i, mcpCReset, 0);
+					cpifaceSession->mcpSet (cpifaceSession, i, mcpCReset, 0);
 					pchan[i]=-1;
 				}
 				tempo=6;
 				speed=125;
 				globalvol=0xFF;
-				mcpSet(-1, mcpGSpeed, 12800);
+				cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, 12800);
 			}
 			LoadPattern(currentpattern, currentrow);
 		}
@@ -1264,7 +1270,7 @@ static void PlayTick(void)
 					break;
 				if (t->ptr[0]!=currentrow)
 					break;
-				PlayCommand(td, t->ptr+2, t->ptr[1]);
+				PlayCommand (cpifaceSession, td, t->ptr+2, t->ptr[1]);
 				t->ptr+=t->ptr[1]+2;
 			}
 		}
@@ -1275,7 +1281,7 @@ static void PlayTick(void)
 				break;
 			if (gtrack.ptr[0]!=currentrow)
 				break;
-			PlayGCommand(gtrack.ptr+2, gtrack.ptr[1]);
+			PlayGCommand (cpifaceSession, gtrack.ptr+2, gtrack.ptr[1]);
 			gtrack.ptr+=gtrack.ptr[1]+2;
 		}
 
@@ -1285,7 +1291,7 @@ static void PlayTick(void)
 
 	DoGCommand();
 	for (td=tdata; td<tdataend; td++)
-		DoCommand(td);
+		DoCommand (cpifaceSession, td);
 
 	for (td=tdata; td<tdataend; td++)
 	{
@@ -1380,7 +1386,7 @@ static void PlayTick(void)
 				int16_t dep=((env->env[td->pchenvpos]-128)<<fs->pchint)>>1;
 
 				if (expopitchenv&&!exponential)
-					td->finalpitch=checkpitch(umuldiv(td->finalpitch, mcpGetFreq8363(dep), 8363));
+					td->finalpitch=checkpitch(umuldiv(td->finalpitch, cpifaceSession->mcpAPI->GetFreq8363(dep), 8363));
 				else
 					td->finalpitch=checkpitch(td->finalpitch-dep);
 
@@ -1433,7 +1439,7 @@ static void PlayTick(void)
 			dep=(dep*td->vibsweeppos)>>16;
 
 			if (expopitchenv&&!exponential)
-				td->finalpitch=checkpitch(umuldiv(td->finalpitch, mcpGetFreq8363(dep), 8363));
+				td->finalpitch=checkpitch(umuldiv(td->finalpitch, cpifaceSession->mcpAPI->GetFreq8363(dep), 8363));
 			else
 				td->finalpitch=checkpitch(td->finalpitch-dep);
 
@@ -1451,7 +1457,7 @@ static void PlayTick(void)
         dep=dep*td->vibsweeppos/fs.vibswp;
 
 	if (expopitchenv&&!exponential)
-        td->finalpitch=checkpitch(umuldiv(td->finalpitch, mcpGetFreq8363(dep), 8363));
+        td->finalpitch=checkpitch(umuldiv(td->finalpitch, cpifaceSession->mcpAPI->GetFreq8363(dep), 8363));
 	else
         td->finalpitch=checkpitch(td->finalpitch-dep);
 
@@ -1490,7 +1496,7 @@ static void PlayTick(void)
 
 		if (td->newinst!=-1)
 			if (td->phys!=-1)
-				mcpSet(td->phys, mcpCInstrument, td->newinst);
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCInstrument, td->newinst);
 		if (td->newpos!=(uint_fast32_t)-1)
 		{
 			uint_fast32_t l;
@@ -1506,8 +1512,8 @@ static void PlayTick(void)
 					tdata[pchan[i]].phys=-1;
 				pchan[i]=td->num;
 				td->phys=i;
-				mcpSet(td->phys, mcpCReset, 0);
-				mcpSet(td->phys, mcpCInstrument, td->samp->handle);
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCReset, 0);
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCInstrument, td->samp->handle);
 			}
 			sm=&sampleinfos[td->samp->handle];
 			l=sm->length;
@@ -1520,46 +1526,55 @@ static void PlayTick(void)
 			if (td->newpos>=l)
 				td->newpos=l-16;
 
-			mcpSet(td->phys, mcpCPosition, td->newpos);
-			mcpSet(td->phys, mcpCLoop, 1);
-			mcpSet(td->phys, mcpCDirect, 0);
-			mcpSet(td->phys, mcpCStatus, 1);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPosition, td->newpos);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCLoop, 1);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCDirect, 0);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCStatus, 1);
 		}
 		if (td->newdir!=-1)
-			mcpSet(td->phys, mcpCDirect, td->newdir);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCDirect, td->newdir);
 		if (td->newloop!=-1)
-			mcpSet(td->phys, mcpCLoop, td->newloop);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCLoop, td->newloop);
 		if (td->phys!=-1)
 		{
 			if (td->stopchan)
-				mcpSet(td->phys, mcpCStatus, 0);
-			mcpSet(td->phys, mcpCVolume, (donotloopmodule&&looped)?0:vol);
-			mcpSet(td->phys, mcpCPanning, pan);
-			mcpSet(td->phys, mcpCPanY, td->pany);
-			mcpSet(td->phys, mcpCPanZ, td->panz);
-			mcpSet(td->phys, mcpCSurround, td->pansrnd);
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCStatus, 0);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCVolume, (donotloopmodule&&looped)?0:vol);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPanning, pan);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPanY, td->pany);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPanZ, td->panz);
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCSurround, td->pansrnd);
 			if (exponential)
-				mcpSet(td->phys, mcpCPitch, -checkpitche(td->finalpitch));
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPitch, -checkpitche(td->finalpitch));
 			else
-				mcpSet(td->phys, mcpCPitch6848, checkpitchh(td->finalpitch));
-			mcpSet(td->phys, mcpCMute, td->mute);
+				cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCPitch6848, checkpitchh(td->finalpitch));
+			cpifaceSession->mcpSet (cpifaceSession, td->phys, mcpCMute, td->mute);
 		}
 	}
-	readque();
-	cmdtime=mcpGet(-1, mcpGCmdTimer);
+	readque (cpifaceSession);
+	cmdtime = cpifaceSession->mcpGet (cpifaceSession, -1, mcpGCmdTimer);
 	putque(cmdtime, -1, (currentrow<<8)|(currentpattern<<16), 0);
 }
 
-char __attribute__ ((visibility ("internal"))) mpPlayModule(const struct gmdmodule *m, struct ocpfilehandle_t *file)
+OCP_INTERNAL int mpPlayModule (const struct gmdmodule *m, struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int i;
+
+	if (!cpifaceSession->mcpDevAPI)
+	{
+		return errPlay;
+	}
+
+
 	for (i=65; i<=128; i++)
 		sintab[i]=sintab[128-i];
 	for (i=129; i<256; i++)
 		sintab[i]=-sintab[256-i];
 
 	if (m->orders[0]==0xFFFF)
-		return 0;
+	{
+		return errFormStruc;
+	}
 
 	sampleinfos=m->samples;
 	modsampnum=m->modsampnum;
@@ -1612,30 +1627,36 @@ char __attribute__ ((visibility ("internal"))) mpPlayModule(const struct gmdmodu
 	quelen=100;
 	que=malloc(sizeof(int)*quelen*4);
 	if (!que)
-		return 0;
+	{
+		return errAllocMem;
+	}
 	querpos=0;
 	quewpos=0;
 
-	if (!mcpOpenPlayer(channels, PlayTick, file))
-		return 0;
+	if (!cpifaceSession->mcpDevAPI->OpenPlayer (channels, PlayTick, file, cpifaceSession))
+	{
+		return errPlay;
+	}
 
-	mcpNormalize (mcpNormalizeDefaultPlayW);
+	cpifaceSession->Normalize (cpifaceSession, mcpNormalizeDefaultPlayW);
 
-	physchan=mcpNChan;
+	physchan = cpifaceSession->PhysicalChannelCount;
 
-	return 1;
+	return errOk;
 }
 
-void __attribute__ ((visibility ("internal"))) mpStopModule(void)
+OCP_INTERNAL void mpStopModule (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int i;
 	for (i=0; i<physchan; i++)
-		mcpSet(i, mcpCReset, 0);
-	mcpClosePlayer();
+	{
+		cpifaceSession->mcpSet (cpifaceSession, i, mcpCReset, 0);
+	}
+	cpifaceSession->mcpDevAPI->ClosePlayer (cpifaceSession);
 	free(que);
 }
 
-void __attribute__ ((visibility ("internal"))) mpGetChanInfo(uint8_t ch, struct chaninfo *ci)
+OCP_INTERNAL void mpGetChanInfo (uint8_t ch, struct chaninfo *ci)
 {
 	const struct trackdata *t=&tdata[ch];
 	ci->ins=0xFF;
@@ -1663,17 +1684,17 @@ void __attribute__ ((visibility ("internal"))) mpGetChanInfo(uint8_t ch, struct
 
 
 
-uint16_t __attribute__ ((visibility ("internal"))) mpGetRealNote(uint8_t ch)
+OCP_INTERNAL uint16_t mpGetRealNote (struct cpifaceSessionAPI_t *cpifaceSession, uint8_t ch)
 {
 	struct trackdata *td=&tdata[ch];
 	if (exponential)
 		return 60*256+td->samp->normnote-checkpitche(td->finalpitch);
 	else
-		return 60*256+td->samp->normnote+mcpGetNote8363(6848*8363/checkpitchh(td->finalpitch));
+		return 60*256+td->samp->normnote+cpifaceSession->mcpAPI->GetNote8363(6848*8363/checkpitchh(td->finalpitch));
   /*    return td.nteval<<8; */
 }
 
-void __attribute__ ((visibility ("internal"))) mpGetGlobInfo(struct globinfo *gi)
+OCP_INTERNAL void mpGetGlobInfo (struct globinfo *gi)
 {
 	int i;
 
@@ -1686,24 +1707,24 @@ void __attribute__ ((visibility ("internal"))) mpGetGlobInfo(struct globinfo *gi
 	gi->patnum=patternnum;
 	gi->globvol=globalvol;
 	gi->globvolslide=0;
-	for (i=0; i<MAXLCHAN; i++)
+	for (i=0; i<GMD_MAXLCHAN; i++)
 		if (globalvolslide[i])
 			gi->globvolslide=globalvolslide[i];
 }
 
-void __attribute__ ((visibility ("internal"))) mpGetPosition(uint16_t *pat, uint8_t *row)
+OCP_INTERNAL void mpGetPosition (uint16_t *pat, uint8_t *row)
 {
 	*pat=currentpattern;
 	*row=currentrow;
 }
 
-int __attribute__ ((visibility ("internal"))) mpGetRealPos(void)
+OCP_INTERNAL int mpGetRealPos (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	readque();
+	readque (cpifaceSession);
 	return realpos;
 }
 
-void __attribute__ ((visibility ("internal"))) mpSetPosition(int16_t pat, int16_t row)
+OCP_INTERNAL void mpSetPosition (struct cpifaceSessionAPI_t *cpifaceSession, int16_t pat, int16_t row)
 {
 	unsigned int i;
 	if (row<0)
@@ -1746,7 +1767,7 @@ void __attribute__ ((visibility ("internal"))) mpSetPosition(int16_t pat, int16_
 			lockpattern=pat;
 		for (i=0; i<physchan; i++)
 		{
-			mcpSet(i, mcpCReset, 0);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCReset, 0);
 			pchan[i]=-1;
 		}
 		for (i=0; i<channels; i++)
@@ -1761,17 +1782,17 @@ void __attribute__ ((visibility ("internal"))) mpSetPosition(int16_t pat, int16_
 	currenttick=tempo;
 }
 
-char __attribute__ ((visibility ("internal"))) mpLooped(void)
+OCP_INTERNAL char mpLooped (void)
 {
 	return looped;
 }
 
-void __attribute__ ((visibility ("internal"))) mpSetLoop(uint8_t s)
+OCP_INTERNAL void mpSetLoop (uint8_t s)
 {
 	donotloopmodule=!s;
 }
 
-void __attribute__ ((visibility ("internal"))) mpLockPat(int st)
+OCP_INTERNAL void mpLockPat (int st)
 {
 	if (st)
 		lockpattern=currentpattern;
@@ -1779,46 +1800,42 @@ void __attribute__ ((visibility ("internal"))) mpLockPat(int st)
 		lockpattern=-1;
 }
 
-int __attribute__ ((visibility ("internal"))) mpGetChanSample(unsigned int ch, int16_t *buf, unsigned int len, uint32_t rate, int opt)
+OCP_INTERNAL int mpGetChanSample (struct cpifaceSessionAPI_t *cpifaceSession, unsigned int ch, int16_t *buf, unsigned int len, uint32_t rate, int opt)
 {
 	if (tdata[ch].phys==-1)
 	{
 		memset(buf, 0, len*2);
 		return 1;
 	}
-	return mcpGetChanSample(tdata[ch].phys, buf, len, rate, opt);
+	return cpifaceSession->mcpGetChanSample (cpifaceSession, tdata[ch].phys, buf, len, rate, opt);
 }
 
-void __attribute__ ((visibility ("internal"))) mpMute(int ch, int mute)
+OCP_INTERNAL void mpMute (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int mute)
 {
+	cpifaceSession->MuteChannel[ch] = mute;
 	tdata[ch].mute=mute;
 	if (tdata[ch].phys!=-1)
-		mcpSet(tdata[ch].phys, mcpCMute, mute);
+		cpifaceSession->mcpSet (cpifaceSession, tdata[ch].phys, mcpCMute, mute);
 }
 
-int __attribute__ ((visibility ("internal"))) mpGetMute(int ch)
-{
-	return tdata[ch].mute;
-}
-
-int __attribute__ ((visibility ("internal"))) mpGetChanStatus(int ch)
+OCP_INTERNAL int mpGetChanStatus (struct cpifaceSessionAPI_t *cpifaceSession, int ch)
 {
 	if (tdata[ch].phys==-1)
 		return 0;
-	return mcpGet(tdata[ch].phys, mcpCStatus);
+	return cpifaceSession->mcpGet (cpifaceSession, tdata[ch].phys, mcpCStatus);
 }
 
-void __attribute__ ((visibility ("internal"))) mpGetRealVolume(int ch, int *l, int *r)
+OCP_INTERNAL void mpGetRealVolume (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int *l, int *r)
 {
 	if (tdata[ch].phys==-1)
 	{
 		*l=*r=0;
 		return;
 	}
-	mcpGetRealVolume(tdata[ch].phys, l, r);
+	cpifaceSession->mcpGetRealVolume (tdata[ch].phys, l, r);
 }
 
-int __attribute__ ((visibility ("internal"))) mpLoadSamples(struct gmdmodule *m)
+OCP_INTERNAL int mpLoadSamples (struct cpifaceSessionAPI_t *cpifaceSession, struct gmdmodule *m)
 {
-	return mcpLoadSamples(m->samples, m->sampnum);
+	return cpifaceSession->mcpDevAPI->LoadSamples (cpifaceSession, m->samples, m->sampnum);
 }
