@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2019-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2019-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * HVLPlay glue logic between interface and actual renderer
  *
@@ -21,7 +21,6 @@
 
 #include "config.h"
 #include <assert.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -31,15 +30,13 @@
 #include "types.h"
 #include "hvlplay.h"
 #include "cpiface/cpiface.h" /* merge in from hvlpinst.c, to compensate for buffer-delay */
-#include "dev/deviplay.h"
 #include "dev/mcp.h"
 #include "dev/player.h"
-#include "dev/plrasm.h"
 #include "dev/ringbuffer.h"
 #include "loader.h"
 #include "player.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
-#include "stuff/poll.h"
 
 #define MAXIMUM_SLOW_DOWN 32
 #define ROW_BUFFERS 25 /* half a second */
@@ -67,16 +64,19 @@ static int16_t  last_ht_PosNr;           /* These are delayed, so should be corr
 static int16_t  last_ht_Tempo;           /* These are delayed, so should be correct */
 static uint8_t  last_ht_SpeedMultiplier; /* These are delayed, so should be correct */
 
-static struct hvl_statbuffer_t hvl_statbuffer[ROW_BUFFERS] = {0}; // half a second */
+static struct hvl_statbuffer_t hvl_statbuffer[ROW_BUFFERS] = {{0}}; // half a second */
 static int hvl_statbuffers_available = 0;
 
-struct hvl_chaninfo __attribute__ ((visibility ("internal"))) ChanInfo[MAX_CHANNELS];
+OCP_INTERNAL struct hvl_chaninfo ChanInfo[MAX_CHANNELS];
 
 struct hvl_tune *ht = 0;
+#warning current_cpifaceSession is temporary
+static struct cpifaceSessionAPI_t *current_cpifaceSession = 0;
 static int hvl_samples_per_row;
 
 static int16_t *hvl_buf_stereo;
 static int16_t *hvl_buf_16chan;
+static uint32_t hvlRate;
 
 static struct ringbuffer_t *hvl_buf_pos;
 /*             tail              processing        head
@@ -85,34 +85,19 @@ static struct ringbuffer_t *hvl_buf_pos;
  *          As the tail catches up, we know data has been played, and we update our stats on the screen
  */
 
-/* devp pre-buffer zone */
-static int16_t *buf16 = 0; /* here we dump out data before it goes live */
-
-/* devp buffer zone */
-static volatile uint32_t bufpos; /* devp write head location */
-static uint32_t buflen; /* devp buffer-size in samples */
-static volatile uint32_t kernpos; /* devp read/tail location - used to track when to show buf8_states */
-static void *plrbuf; /* the devp buffer */
-static int stereo; /* boolean */
-static int bit16; /* boolean */
-static int signedout; /* boolean */
-static int reversestereo; /* boolean */
-static int active=0;
-static volatile int PauseSamples;
-
 static uint32_t hvlbuffpos;
 static int hvl_doloop;
 static int hvl_looped;
 static int hvl_inpause;
 
+static uint64_t samples_committed;
+static uint64_t samples_lastui;
+
 static int bal, vol;
 static unsigned long voll,volr;
 static int pan;
 static int srnd;
 
-static int (*_GET)(int ch, int opt);
-static void (*_SET)(int ch, int opt, int val);
-
 static uint8_t hvl_muted[MAX_CHANNELS];
 
 #define PANPROC \
@@ -144,13 +129,13 @@ do { \
 	} \
 } while(0)
 
-void __attribute__ ((visibility ("internal"))) hvlGetChanInfo (int chan, struct hvl_chaninfo *ci)
+OCP_INTERNAL void hvlGetChanInfo (int chan, struct hvl_chaninfo *ci)
 {
 	memcpy (ci, ChanInfo + chan, sizeof (*ci));
 	ci->muted = hvl_muted[chan];
 }
 
-void __attribute__ ((visibility ("internal"))) hvlGetChanVolume (int chan, int *l, int *r)
+OCP_INTERNAL void hvlGetChanVolume (struct cpifaceSessionAPI_t *cpifaceSession, int chan, int *l, int *r)
 {
 	int16_t *src;
 	int pos1, pos2;
@@ -160,7 +145,7 @@ void __attribute__ ((visibility ("internal"))) hvlGetChanVolume (int chan, int *
 	*l = 0;
 	*r = 0;
 
-	ringbuffer_get_tail_samples (hvl_buf_pos, &pos1, &length1, &pos2, &length2);
+	cpifaceSession->ringbufferAPI->get_tail_samples (hvl_buf_pos, &pos1, &length1, &pos2, &length2);
 
 	src = hvl_buf_16chan + MAX_CHANNELS * 2 * pos1;
 
@@ -208,7 +193,7 @@ static void hvl_statbuffer_callback_from_hvlbuf (void *arg, int samples_ago)
 	{
 		if ((state->ChanInfo[i].ins >= 0) && (state->ChanInfo[i].ins <= 255))
 		{
-			if (plSelCh == i)
+			if (current_cpifaceSession->SelectedChannel == i)
 			{
 				plInstUsed[state->ChanInfo[i].ins] = 3;
 			} else {
@@ -227,7 +212,7 @@ static void hvl_statbuffer_callback_from_hvlbuf (void *arg, int samples_ago)
 	hvl_statbuffers_available++;
 }
 
-void __attribute__ ((visibility ("internal"))) hvlIdler (void)
+OCP_INTERNAL void hvlIdler (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	while (hvl_statbuffers_available) /* we only prepare more data if hvl_statbuffers_available is non-zero. This gives about 0.5 seconds worth of sample-data */
 	{
@@ -238,6 +223,14 @@ void __attribute__ ((visibility ("internal"))) hvlIdler (void)
 		int16_t *src;
 		int16_t *dst;
 
+		/* limit the internal render buffer to 100ms */
+		cpifaceSession->ringbufferAPI->get_tailandprocessing_samples (hvl_buf_pos, &pos1, &length1, &pos2, &length2);
+		length1 += length2;
+		if (length1 >= hvlRate / 10)
+		{
+			break;
+		}
+
 		for (i=0; i < ROW_BUFFERS; i++)
 		{
 			if (hvl_statbuffer[i].in_use)
@@ -296,7 +289,7 @@ void __attribute__ ((visibility ("internal"))) hvlIdler (void)
 			hvl_statbuffer[i].ChanInfo[j].fxBparam   = Step->stp_FXbParam;
 		}
 
-		ringbuffer_get_head_samples (hvl_buf_pos, &pos1, &length1, &pos2, &length2);
+		cpifaceSession->ringbufferAPI->get_head_samples (hvl_buf_pos, &pos1, &length1, &pos2, &length2);
 
 		/* We can fit length1+length2 samples into out devp-mirrored buffer */
 
@@ -348,47 +341,18 @@ void __attribute__ ((visibility ("internal"))) hvlIdler (void)
 		}
 
 		hvl_statbuffer[i].in_use = 1;
-		ringbuffer_add_tail_callback_samples (hvl_buf_pos, 0, hvl_statbuffer_callback_from_hvlbuf, hvl_statbuffer + i);
+		cpifaceSession->ringbufferAPI->add_tail_callback_samples (hvl_buf_pos, 0, hvl_statbuffer_callback_from_hvlbuf, hvl_statbuffer + i);
 
 		/* Adding hvl_samples_per_row to our devp-mirrored buffer */
 
-		ringbuffer_head_add_samples (hvl_buf_pos, hvl_samples_per_row);
+		cpifaceSession->ringbufferAPI->head_add_samples (hvl_buf_pos, hvl_samples_per_row);
 
 		hvl_statbuffers_available--;
 	}
 }
 
-static void hvlUpdateKernPos (void)
+OCP_INTERNAL void hvlIdle (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	uint32_t delta, newpos;
-	newpos = plrGetPlayPos() >> (stereo+bit16);
-	delta = (buflen + newpos - kernpos) % buflen;
-
-	if (PauseSamples)
-	{
-		if (delta >= PauseSamples)
-		{
-			delta -= PauseSamples;
-			PauseSamples = 0;
-		} else {
-			PauseSamples -= delta;
-			delta = 0;
-		}
-	}
-
-	if (delta)
-	{
-		/* devp-buffer used completed playing DELTA amounts of samples */
-		ringbuffer_tail_consume_samples (hvl_buf_pos, delta);
-	}
-
-	kernpos = newpos;
-}
-
-void __attribute__ ((visibility ("internal"))) hvlIdle (void)
-{
-	uint32_t bufdelta;
-	uint32_t pass2;
 	static volatile int clipbusy=0;
 
 	if (clipbusy++)
@@ -397,315 +361,136 @@ void __attribute__ ((visibility ("internal"))) hvlIdle (void)
 		return;
 	}
 
-	/* "fill" up our buffers */
-	hvlIdler();
-
-	hvlUpdateKernPos ();
+	cpifaceSession->plrDevAPI->Idle();
 
+	if (hvl_inpause || (hvl_looped == 3))
 	{
-		uint32_t buf_read_pos; /* bufpos is last given buf_write_pos */
-
-		buf_read_pos = plrGetBufPos() >> (stereo+bit16);
-
-		bufdelta = ( buflen + buf_read_pos - bufpos )%buflen;
-	}
-
-	if (!bufdelta)
-	{
-		clipbusy--;
-		if (plrIdle)
-			plrIdle();
-		return;
-	}
-
-	if (hvl_inpause)
-	{
-		/* If we are in pause, we fill buffer with the correct type of zeroes */
-		/* But we also make sure that the buffer-fill is never more than 0.1s, making it reponsive */
-		uint32_t min_bufdelta;
-
-		if (buflen > plrRate/10)
-		{
-			min_bufdelta = buflen - (plrRate/10);
-		} else {
-			min_bufdelta = 0;
-		}
-
-		if (bufdelta > min_bufdelta)
-		{
-			bufdelta = bufdelta - min_bufdelta;
-
-			if ((bufpos+bufdelta)>buflen)
-				pass2=bufpos+bufdelta-buflen;
-			else
-				pass2=0;
-
-			if (bit16)
-			{
-				plrClearBuf((uint16_t *)plrbuf+(bufpos<<stereo), (bufdelta-pass2)<<stereo, signedout);
-				if (pass2)
-					plrClearBuf((uint16_t *)plrbuf, pass2<<stereo, signedout);
-			} else {
-				plrClearBuf(buf16, bufdelta<<stereo, signedout);
-				plr16to8((uint8_t *)plrbuf+(bufpos<<stereo), (uint16_t *)buf16, (bufdelta-pass2)<<stereo);
-				if (pass2)
-					plr16to8((uint8_t *)plrbuf, (uint16_t *)buf16+((bufdelta-pass2)<<stereo), pass2<<stereo);
-			}
-			bufpos+=bufdelta;
-			if (bufpos>=buflen)
-				bufpos-=buflen;
-
-			/* Put "data" into PauseSamples, instead of hvl_buf_pos please */
-			PauseSamples += bufdelta;
-			//ringbuffer_processing_consume_samples (hvl_buf_pos, bufdelta); /* add this rate buf16_filled == tail_used */
-		}
+		cpifaceSession->plrDevAPI->Pause (1);
 	} else {
-		int pos1, length1, pos2, length2;
-		int i;
-		int buf16_filled = 0;
-		int16_t *t = buf16;
+		void *targetbuf;
+		unsigned int targetlength; /* in samples */
 
+		cpifaceSession->plrDevAPI->Pause (0);
 
-		/* how much data is available to transfer into devp.. we are using a ringbuffer, so we might receive two fragments */
-		ringbuffer_get_processing_samples (hvl_buf_pos, &pos1, &length1, &pos2, &length2);
+		cpifaceSession->plrDevAPI->GetBuffer (&targetbuf, &targetlength);
 
-		if (bufdelta > (length1+length2))
+		if (targetlength)
 		{
-			bufdelta=(length1+length2);
-			hvl_looped |= 2;
+			int16_t *t = targetbuf;
+			unsigned int accumulated_target = 0;
+			unsigned int accumulated_source = 0;
+			int pos1, length1, pos2, length2;
 
-		} else {
-			hvl_looped &= ~2;
-		}
+			hvlIdler (cpifaceSession);
 
-		for (buf16_filled=0; buf16_filled<bufdelta; buf16_filled++)
-		{
-			int16_t rs, ls;
+			/* how much data is available.. we are using a ringbuffer, so we might receive two fragments */
+			// warning this deviates, it uses get_processing_samples instead of tail, we need to keep a copy of the sound-buffers per channel, so makes more sense to track it here
+			cpifaceSession->ringbufferAPI->get_processing_samples (hvl_buf_pos, &pos1, &length1, &pos2, &length2);
 
-			if (!length1)
+			/* bufrate is always correct, since we get the correct speed always from the renderer */
+
+			//if (hvlbufrate==0x10000)
 			{
-				pos1 = pos2;
-				length1 = length2;
-				pos2 = 0;
-				length2 = 0;
-			}
-
-			assert (length1);
-
-			rs = hvl_buf_stereo[(pos1<<1)    ];
-			ls = hvl_buf_stereo[(pos1<<1) + 1];
-
-			PANPROC;
-
-			*(t++) = rs;
-			*(t++) = ls;
-
-			pos1++;
-			length1--;
-		}
-
-		ringbuffer_processing_consume_samples (hvl_buf_pos, buf16_filled); /* add this rate buf16_filled == tail_used */
-
-		bufdelta=buf16_filled;
-
-		if ((bufpos+bufdelta)>buflen)
-			pass2=bufpos+bufdelta-buflen;
-		else
-			pass2=0;
-		bufdelta-=pass2;
-
-		if (bit16)
-		{
-			if (stereo)
-			{
-				int16_t *p=(int16_t *)plrbuf+2*bufpos;
-				int16_t *b=buf16;
-				if (signedout)
+				if (targetlength>(length1+length2))
 				{
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[0];
-						p[1]=b[1];
-						p+=2;
-						b+=2;
-					}
-					p=(int16_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[0];
-						p[1]=b[1];
-						p+=2;
-						b+=2;
-					}
+					targetlength=(length1+length2);
+					hvl_looped |= 2;
 				} else {
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[0]^0x8000;
-						p[1]=b[1]^0x8000;
-						p+=2;
-						b+=2;
-					}
-					p=(int16_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[0]^0x8000;
-						p[1]=b[1]^0x8000;
-						p+=2;
-						b+=2;
-					}
+					hvl_looped &= ~2;
 				}
-			} else {
-				int16_t *p=(int16_t *)plrbuf+bufpos;
-				int16_t *b=buf16;
-				if (signedout)
-				{
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[0];
-						p++;
-						b++;
-					}
-					p=(int16_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[0];
-						p++;
-						b++;
-					}
-				} else {
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[0]^0x8000;
-						p++;
-						b++;
-					}
-					p=(int16_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[0]^0x8000;
-						p++;
-						b++;
-					}
-				}
-			}
-		} else {
-			if (stereo)
-			{
-				uint8_t *p=(uint8_t *)plrbuf+2*bufpos;
-				uint8_t *b=(uint8_t *)buf16;
-				if (signedout)
-				{
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[1];
-						p[1]=b[3];
-						p+=2;
-						b+=4;
-					}
-					p=(uint8_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[1];
-						p[1]=b[3];
-						p+=2;
-						b+=4;
-					}
-				} else {
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[1]^0x80;
-						p[1]=b[3]^0x80;
-						p+=2;
-						b+=4;
-					}
-					p=(uint8_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[1]^0x80;
-						p[1]=b[3]^0x80;
-						p+=2;
-						b+=4;
-					}
-				}
-			} else {
-				uint8_t *p=(uint8_t *)plrbuf+bufpos;
-				int16_t *b=buf16;
-				if (signedout)
-				{
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=(b[0]+b[1])>>9;
-						p++;
-						b+=2;
-					}
 
-					p=(uint8_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=(b[0]+b[1])>>9;
-						p++;
-						b+=2;
-					}
-				} else {
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=((b[0]+b[1])>>9)^0x80;
-						p++;
-						b+=2;
-					}
-					p=(uint8_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=((b[0]+b[1])>>9)^0x80;
-						p++;
-						b+=2;
-					}
+				// limit source to not overrun target buffer
+				if (length1 > targetlength)
+				{
+					length1 = targetlength;
+					length2 = 0;
+				} else if ((length1 + length2) > targetlength)
+				{
+					length2 = targetlength - length1;
 				}
-			}
-		}
-		bufpos+=buf16_filled;
-		if (bufpos>=buflen)
-			bufpos-=buflen;
+
+				accumulated_source = accumulated_target = length1 + length2;
+
+				while (length1)
+				{
+					while (length1)
+					{
+						int16_t rs, ls;
+
+						rs = hvl_buf_stereo[(pos1<<1) + 0];
+						ls = hvl_buf_stereo[(pos1<<1) + 1];
+
+						PANPROC;
+
+						*(t++) = rs;
+						*(t++) = ls;
+
+						pos1++;
+						length1--;
+
+						//accumulated_target++;
+					}
+					length1 = length2;
+					length2 = 0;
+					pos1 = pos2;
+					pos2 = 0;
+				}
+				//accumulated_source = accumulated_target;
+			} /* } else { } //if (hvlbufrate==0x10000) */
+			// warning this deviates, it uses processing_consume_samples instead of tail...
+			cpifaceSession->ringbufferAPI->processing_consume_samples (hvl_buf_pos, accumulated_source);
+			cpifaceSession->plrDevAPI->CommitBuffer (accumulated_target);
+			samples_committed += accumulated_target;
+		} /* if (targetlength) */
 	}
 
-	plrAdvanceTo(bufpos<<(stereo+bit16));
-
-	if (plrIdle)
-		plrIdle();
+	{
+		uint64_t delay = cpifaceSession->plrDevAPI->Idle();
+		uint64_t new_ui = samples_committed - delay;
+		if (new_ui > samples_lastui)
+		{
+			cpifaceSession->ringbufferAPI->tail_consume_samples (hvl_buf_pos, new_ui - samples_lastui);
+			samples_lastui = new_ui;
+		}
+	}
 
 	clipbusy--;
 }
 
-void __attribute__ ((visibility ("internal"))) hvlSetLoop (uint8_t s)
+OCP_INTERNAL void hvlSetLoop (uint8_t s)
 {
 	hvl_doloop = s;
 }
 
-char __attribute__ ((visibility ("internal"))) hvlLooped (void)
+OCP_INTERNAL char hvlLooped (void)
 {
 	return hvl_looped == 3;
 }
 
-void __attribute__ ((visibility ("internal"))) hvlPause (uint8_t p)
+OCP_INTERNAL void hvlPause (uint8_t p)
 {
 	hvl_inpause = p;
 }
 
 static void hvlSetSpeed (uint16_t sp)
 {
-	hvl_samples_per_row = plrRate * 256 / (50 * sp);
+	if (sp < 4)
+		sp = 4;
+	hvl_samples_per_row = hvlRate * 256 / (50 * sp);
 
 	/* pause can cause slower than MAXIMUM_SLOW_DOWN, so we floor the value at that */
-	if (hvl_samples_per_row > (plrRate * MAXIMUM_SLOW_DOWN / 50))
+	if (hvl_samples_per_row > (hvlRate * MAXIMUM_SLOW_DOWN / 50))
 	{
-		hvl_samples_per_row = plrRate * MAXIMUM_SLOW_DOWN / 50;
+		hvl_samples_per_row = hvlRate * MAXIMUM_SLOW_DOWN / 50;
 	}
 }
 
 static void hvlSetPitch (uint16_t sp)
 {
-	ht->ht_Frequency = plrRate * 256 / sp;
-	ht->ht_FreqF = (double)plrRate * 256 / sp;
+	if (sp < 4)
+		sp = 4;
+	ht->ht_Frequency = hvlRate * 256 / sp;
+	ht->ht_FreqF = (double)hvlRate * 256 / sp;
 }
 
 static void hvlSetVolume (void)
@@ -717,7 +502,7 @@ static void hvlSetVolume (void)
 		volr=(volr*(64-bal))>>6;
 }
 
-static void SET(int ch, int opt, int val)
+static void hvlSet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt, int val)
 {
 	switch (opt)
 	{
@@ -732,10 +517,6 @@ static void SET(int ch, int opt, int val)
 			break;
 		case mcpMasterPanning:
 			pan=val;
-			if (reversestereo)
-			{
-				pan = -pan;
-			}
 			hvlSetVolume();
 			break;
 		case mcpMasterVolume:
@@ -748,17 +529,18 @@ static void SET(int ch, int opt, int val)
 			break;
 	}
 }
-static int GET(int ch, int opt)
+
+static int hvlGet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt)
 {
 	return 0;
 }
 
-void __attribute__ ((visibility ("internal"))) hvlRestartSong ()
+OCP_INTERNAL void hvlRestartSong ()
 {
 	hvl_InitSubsong (ht, ht->ht_SongNum);
 }
 
-void __attribute__ ((visibility ("internal"))) hvlPrevSubSong ()
+OCP_INTERNAL void hvlPrevSubSong ()
 {
 	if (ht->ht_SongNum)
 	{
@@ -767,7 +549,7 @@ void __attribute__ ((visibility ("internal"))) hvlPrevSubSong ()
 	hvl_InitSubsong (ht, ht->ht_SongNum);
 }
 
-void __attribute__ ((visibility ("internal"))) hvlNextSubSong ()
+OCP_INTERNAL void hvlNextSubSong ()
 {
 	if (ht->ht_SongNum+1 <= ht->ht_SubsongNr)
 	{
@@ -776,21 +558,22 @@ void __attribute__ ((visibility ("internal"))) hvlNextSubSong ()
 	hvl_InitSubsong (ht, ht->ht_SongNum);
 }
 
-void __attribute__ ((visibility ("internal")))  hvlMute (int ch, int m)
+OCP_INTERNAL void hvlMute (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int m)
 {
+	cpifaceSession->MuteChannel[ch] = m;
 	hvl_muted[ch] = m;
 }
 
-int __attribute__ ((visibility ("internal"))) hvlGetChanSample(unsigned int ch, int16_t *s, unsigned int len, uint32_t rate, int opt)
+OCP_INTERNAL int hvlGetChanSample (struct cpifaceSessionAPI_t *cpifaceSession, unsigned int ch, int16_t *s, unsigned int len, uint32_t rate, int opt)
 {
-	int stereo = (opt&cpiGetSampleStereo)?1:0;
-	uint32_t step = imuldiv(0x00010000, plrRate, (signed)rate);
+	int stereo = (opt&mcpGetSampleStereo)?1:0;
+	uint32_t step = imuldiv(0x00010000, hvlRate, (signed)rate);
 	int16_t *src;
 	int pos1, pos2;
 	int length1, length2;
 	uint32_t posf = 0;
 
-	ringbuffer_get_tail_samples (hvl_buf_pos, &pos1, &length1, &pos2, &length2);
+	cpifaceSession->ringbufferAPI->get_tail_samples (hvl_buf_pos, &pos1, &length1, &pos2, &length2);
 
 	src = hvl_buf_16chan + MAX_CHANNELS * 2 * pos1;
 
@@ -822,7 +605,7 @@ int __attribute__ ((visibility ("internal"))) hvlGetChanSample(unsigned int ch,
 			}
 			if (!length1)
 			{
-				memsetd(s, 0, len<<stereo);
+				memset (s, 0, (len<<stereo)<<2);
 				return !!hvl_muted[ch];
 			}
 		}
@@ -830,7 +613,7 @@ int __attribute__ ((visibility ("internal"))) hvlGetChanSample(unsigned int ch,
 	return !!hvl_muted[ch];
 }
 
-void __attribute__ ((visibility ("internal"))) hvlGetStats (int *row, int *rows, int *order, int *orders, int *subsong, int *subsongs, int *tempo, int *speedmult)
+OCP_INTERNAL void hvlGetStats (int *row, int *rows, int *order, int *orders, int *subsong, int *subsongs, int *tempo, int *speedmult)
 {
 	*row       = last_ht_NoteNr;
 	*order     = last_ht_PosNr;
@@ -842,28 +625,38 @@ void __attribute__ ((visibility ("internal"))) hvlGetStats (int *row, int *rows,
 	*speedmult = last_ht_SpeedMultiplier;
 }
 
-struct hvl_tune __attribute__ ((visibility ("internal"))) *hvlOpenPlayer (const uint8_t *mem, size_t memlen, struct ocpfilehandle_t *file)
+OCP_INTERNAL int hvlOpenPlayer (const uint8_t *mem, size_t memlen, struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	int BufSize;
+	enum plrRequestFormat format;
+	int retval;
+
+	if (!cpifaceSession->plrDevAPI)
+	{
+		return errPlay;
+	}
 
 	hvl_InitReplayer ();
 
-	plrSetOptions(44100, (PLR_SIGNEDOUT|PLR_16BIT)|PLR_STEREO);
+	hvlRate=0;
+	format=PLR_STEREO_16BIT_SIGNED;
+	if (!cpifaceSession->plrDevAPI->Play (&hvlRate, &format, file, cpifaceSession))
+	{
+		return errPlay;
+	}
 
-	stereo=!!(plrOpt&PLR_STEREO);
-	bit16=!!(plrOpt&PLR_16BIT);
-	signedout=!!(plrOpt&PLR_SIGNEDOUT);
-	reversestereo=!!(plrOpt&PLR_REVERSESTEREO);
+	current_cpifaceSession = cpifaceSession;
 
-	ht = hvl_LoadTune_memory (mem, memlen, 4, plrRate);
+	ht = hvl_LoadTune_memory (cpifaceSession, mem, memlen, 4, hvlRate);
 	if (!ht)
 	{
-		goto error_out;
+		retval = errFormStruc;
+		goto error_out_plrDevAPI_Play;
 	}
 
 	if( !hvl_InitSubsong( ht, 0 ) )
 	{
-		goto error_out;
+		retval = errFormStruc;
+		goto error_out_tune;
 	}
 
 	last_ht_SongNum = 0;
@@ -872,104 +665,87 @@ struct hvl_tune __attribute__ ((visibility ("internal"))) *hvlOpenPlayer (const
 	last_ht_Tempo = 1;
 	last_ht_SpeedMultiplier = 1;
 
-	BufSize=plrBufSize;
-	if (BufSize > 40)
-	{
-		BufSize = 40;
-	}
-	if (!plrOpenPlayer(&plrbuf, &buflen, BufSize * plrRate / 1000, file))
-	{
-		goto error_out;
-	}
-	bufpos=0;
-	kernpos=0;
 	hvlbuffpos = 0x00000000;
-	PauseSamples = 0;
 	hvl_inpause = 0;
-	active = 1;
 	hvl_doloop = 0;
+	hvl_looped = 0;
+	samples_committed=0;
+	samples_lastui=0;
 
-	hvl_samples_per_row = plrRate / 50;
+	hvl_samples_per_row = hvlRate / 50;
 
-	buf16 = malloc (sizeof (int16_t) * buflen * 2);
 	hvl_buf_stereo = malloc (sizeof (int16_t) * (ROW_BUFFERS + 2) * MAXIMUM_SLOW_DOWN * 2 * hvl_samples_per_row); /* The + 2 is on purpose, so we do not have to wrap when calling hvl_DecodeFrame(), and another to have enough space when buffer utilization is close to maximum */
 	hvl_buf_16chan = malloc (sizeof (int16_t) * (ROW_BUFFERS + 2) * MAXIMUM_SLOW_DOWN * 2 * MAX_CHANNELS * hvl_samples_per_row);
 
-	if ((!buf16) && (!hvl_buf_stereo) && (!hvl_buf_16chan))
+	if ((!hvl_buf_stereo) && (!hvl_buf_16chan))
 	{
-		goto error_out;
+		retval = errAllocMem;
+		goto error_out_mem;
 	}
 
-	hvl_buf_pos = ringbuffer_new_samples (RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_16BIT | RINGBUFFER_FLAGS_SIGNED | RINGBUFFER_FLAGS_PROCESS, (ROW_BUFFERS + 1) * MAXIMUM_SLOW_DOWN * hvl_samples_per_row);
+	hvl_buf_pos = cpifaceSession->ringbufferAPI->new_samples (RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_16BIT | RINGBUFFER_FLAGS_SIGNED | RINGBUFFER_FLAGS_PROCESS, (ROW_BUFFERS + 1) * MAXIMUM_SLOW_DOWN * hvl_samples_per_row);
 	if (!hvl_buf_pos)
 	{
-		goto error_out;
+		retval = errAllocMem;
+		goto error_out_mem;
 	}
 
-	bzero (hvl_muted, sizeof (hvl_muted));
+	memset (hvl_muted, 0, sizeof (hvl_muted));
 
-	bzero (hvl_statbuffer, sizeof (hvl_statbuffer));
+	memset (hvl_statbuffer, 0, sizeof (hvl_statbuffer));
 	hvl_statbuffers_available = ROW_BUFFERS;
 
-	bzero (plInstUsed, sizeof (plInstUsed));
+	memset (plInstUsed, 0, sizeof (plInstUsed));
 
-	if (!pollInit(hvlIdle))
+	cpifaceSession->mcpSet = hvlSet;
+	cpifaceSession->mcpGet = hvlGet;
+
+	cpifaceSession->Normalize (cpifaceSession, mcpNormalizeDefaultPlayP | mcpNormalizeCanSpeedPitchUnlock);
+
+	return errOk;
+
+	//if (hvl_buf_pos)
+	//{
+	//	cpifaceSession->ringbufferAPI->free (hvl_buf_pos);
+	//	hvl_buf_pos = 0;
+	//}
+error_out_mem:
+	free (hvl_buf_stereo);
+	hvl_buf_stereo = 0;
+	free (hvl_buf_16chan);
+	hvl_buf_16chan = 0;
+error_out_tune:
+	if (ht)
 	{
-		goto error_out;
+		hvl_FreeTune (ht);
+		ht = 0;
 	}
-	active = 3;
+error_out_plrDevAPI_Play:
+	cpifaceSession->plrDevAPI->Stop (cpifaceSession);
 
-	_SET=mcpSet;
-	_GET=mcpGet;
-	mcpSet=SET;
-	mcpGet=GET;
+	current_cpifaceSession = 0;
 
-	mcpNormalize (mcpNormalizeDefaultPlayP);
-
-	return ht;
-
-error_out:
-	hvlClosePlayer();
-
-	return 0;
+	return retval;
 }
 
-void __attribute__ ((visibility ("internal"))) hvlClosePlayer (void)
+OCP_INTERNAL void hvlClosePlayer (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	if (active & 2)
+	if (cpifaceSession->plrDevAPI)
 	{
-		pollClose();
+		cpifaceSession->plrDevAPI->Stop (cpifaceSession);
 	}
-	if (active & 1)
-	{
-		plrClosePlayer();
-		plrbuf = 0;
-	}
-	active = 0;
 
 	if (hvl_buf_pos)
 	{
-		ringbuffer_free (hvl_buf_pos);
+		cpifaceSession->ringbufferAPI->free (hvl_buf_pos);
 		hvl_buf_pos = 0;
 	}
 
-	if (buf16)
-	{
-		free (buf16);
-		buf16 = 0;
-	}
+	free (hvl_buf_stereo);
+	hvl_buf_stereo = 0;
 
-	if (hvl_buf_stereo)
-	{
-		free (hvl_buf_stereo);
-		hvl_buf_stereo = 0;
-	}
-
-	if (hvl_buf_16chan)
-	{
-		free (hvl_buf_16chan);
-		hvl_buf_16chan = 0;
-	}
+	free (hvl_buf_16chan);
+	hvl_buf_16chan = 0;
 
 	if (ht)
 	{
@@ -977,14 +753,5 @@ void __attribute__ ((visibility ("internal"))) hvlClosePlayer (void)
 		ht = 0;
 	}
 
-	if (_SET)
-	{
-		mcpSet = _SET;
-		_SET = 0;
-	}
-	if (_GET)
-	{
-		mcpGet = _GET;
-		_GET = 0;
-	}
+	current_cpifaceSession = 0;
 }
