@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * OGGPlay - Player for Ogg Vorbis files
  *
@@ -35,27 +35,20 @@
 #include <vorbis/codec.h>
 #include <vorbis/vorbisfile.h>
 #include "types.h"
-#include "cpiface/gif.h"
-#include "cpiface/jpeg.h"
-#include "cpiface/png.h"
-#include "dev/deviplay.h"
+#include "cpiface/cpiface.h"
 #include "dev/mcp.h"
 #include "dev/player.h"
 #include "dev/plrasm.h"
 #include "dev/ringbuffer.h"
 #include "filesel/filesystem.h"
 #include "oggplay.h"
+#include "stuff/err.h"
 #include "stuff/imsrtns.h"
-#include "stuff/timer.h"
-#include "stuff/poll.h"
+#include "stuff/poutput.h"
 
 static int current_section;
 
-static int stereo; /* 32 bit booleans are fast */
-static int bit16;
-static int signedout;
-static uint32_t samprate;
-static uint8_t reversestereo;
+static uint32_t oggRate; /* devp rate */
 
 static uint32_t voll,volr;
 static int vol;
@@ -67,11 +60,6 @@ static int opt25_50;
 static char opt25[26];
 static char opt50[51];
 
-static int16_t *buf16=NULL;
-static uint32_t bufpos;
-static uint32_t buflen;
-static void *plrbuf;
-
 static OggVorbis_File ov;
 static int oggstereo;
 static int oggrate;
@@ -84,18 +72,15 @@ static struct ringbuffer_t *oggbufpos = 0;
 static uint_fast32_t oggbuffpos;
 static uint_fast32_t oggbufrate;
 static volatile int active;
-static int looped;
+static int ogg_looped;
 static int donotloop;
 
-static int inpause;
+static int ogg_inpause;
 
 static volatile int clipbusy=0;
 
 static struct ocpfilehandle_t *oggfile;
 
-static int (*_GET)(int ch, int opt);
-static void (*_SET)(int ch, int opt, int val);
-
 #define PANPROC \
 do { \
 	float _rs = rs, _ls = ls; \
@@ -125,7 +110,16 @@ do { \
 	} \
 } while(0)
 
-static void oggIdler(void)
+static void plrMono16ToStereo16(int16_t *buf, int len)
+{ /* convert from end to start, so that we do not overwrite samples as data expands in size */
+	int i;
+	for (i = len; i >= 0; i--)
+	{
+		buf[i<<1] = buf[(i<<1)+1] = buf[i];
+	}
+}
+
+static void oggIdler (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	if (!active)
 		return;
@@ -149,10 +143,10 @@ static void oggIdler(void)
 		}
 		if (ov_pcm_tell(&ov)!=oggpos)
 		{
-			fprintf (stderr, "[playogg]: warning, frame position is broken in file (got=%" PRId64 ", expected= %" PRId64 ")\n", ov_pcm_tell(&ov), oggpos);
+			cpifaceSession->cpiDebug (cpifaceSession, "[OGG] warning, frame position is broken in file (got=%" PRId64 ", expected= %" PRId64 ")\n", ov_pcm_tell(&ov), oggpos);
 		}
 
-		ringbuffer_get_head_samples (oggbufpos, &pos1, &length1, &pos2, &length2);
+		cpifaceSession->ringbufferAPI->get_head_samples (oggbufpos, &pos1, &length1, &pos2, &length2);
 
 		if (!length1)
 		{
@@ -176,8 +170,8 @@ static void oggIdler(void)
 
 			if (result<=0) /* broken data... we can survive */
 			{
-				plrClearBuf (oggbuf+(pos1<<1), read<<1, 0); /* always clear stereo */
-				fprintf (stderr, "[playogg] ov_read failed: %ld\n", result);
+				memset (oggbuf+(pos1<<1), 0, read<<2); /* always clear 16bit, stereo, signed */
+				cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_read failed: %ld\n", result);
 				result=read;
 			} else {
 				result>>=(1+oggstereo);
@@ -186,7 +180,7 @@ static void oggIdler(void)
 					plrMono16ToStereo16 (oggbuf+(pos1<<1), result);
 				}
 			}
-			ringbuffer_head_add_samples (oggbufpos, result);
+			cpifaceSession->ringbufferAPI->head_add_samples (oggbufpos, result);
 		} else {
 			break;
 		}
@@ -195,11 +189,11 @@ static void oggIdler(void)
 		{
 			if (donotloop)
 			{
-				looped |= 1;
+				ogg_looped |= 1;
 				oggpos = ogglen;
 				break;
 			} else {
-				looped &= ~1;
+				ogg_looped &= ~1;
 				oggpos = 0;
 				oggneedseek = 1;
 			}
@@ -209,396 +203,184 @@ static void oggIdler(void)
 	}
 }
 
-void __attribute__ ((visibility ("internal"))) oggIdle(void)
+OCP_INTERNAL void oggIdle (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	uint32_t bufdelta;
-	uint32_t pass2;
-
 	if (clipbusy++)
 	{
 		clipbusy--;
 		return;
 	}
 
+	if (ogg_inpause || (ogg_looped == 3))
 	{
-		uint32_t bufplayed;
-
-		bufplayed=plrGetBufPos()>>(stereo+bit16);
-
-		bufdelta=(buflen+bufplayed-bufpos)%buflen;
-	}
-
-	if (!bufdelta)
-	{
-		clipbusy--;
-		if (plrIdle)
-			plrIdle();
-		return;
-	}
-	oggIdler();
-
-	if (inpause)
-	{ /* If we are in pause, we fill buffer with the correct type of zeroes */
-		if ((bufpos+bufdelta)>buflen)
-			pass2=bufpos+bufdelta-buflen;
-		else
-			pass2=0;
-		if (bit16)
-		{
-			plrClearBuf((uint16_t *)plrbuf+(bufpos<<stereo), (bufdelta-pass2)<<stereo, signedout);
-			if (pass2)
-				plrClearBuf((uint16_t *)plrbuf, pass2<<stereo, signedout);
-		} else {
-			plrClearBuf(buf16, bufdelta<<stereo, signedout);
-			plr16to8((uint8_t *)plrbuf+(bufpos<<stereo), (uint16_t *)buf16, (bufdelta-pass2)<<stereo);
-			if (pass2)
-				plr16to8((uint8_t *)plrbuf, (uint16_t *)buf16+((bufdelta-pass2)<<stereo), pass2<<stereo);
-		}
-		bufpos+=bufdelta;
-		if (bufpos>=buflen)
-			bufpos-=buflen;
+		cpifaceSession->plrDevAPI->Pause (1);
 	} else {
-		int pos1, length1, pos2, length2;
-		int i;
-		int buf16_filled = 0;
+		void *targetbuf;
+		unsigned int targetlength; /* in samples */
 
-		/* how much data is available.. we are using a ringbuffer, so we might receive two fragments */
-		ringbuffer_get_tail_samples (oggbufpos, &pos1, &length1, &pos2, &length2);
+		cpifaceSession->plrDevAPI->Pause (0);
 
-		/* are the speed 1:1, if so filling up buf16 is very easy */
-		if (oggbufrate==0x10000)
+		cpifaceSession->plrDevAPI->GetBuffer (&targetbuf, &targetlength);
+
+		if (targetlength)
 		{
-			int16_t *t = buf16;
+			int16_t *t = targetbuf;
+			unsigned int accumulated_target = 0;
+			unsigned int accumulated_source = 0;
+			int pos1, length1, pos2, length2;
 
-			if (bufdelta>(length1+length2))
+			/* fill up our buffers */
+			oggIdler (cpifaceSession);
+
+			/* how much data is available.. we are using a ringbuffer, so we might receive two fragments */
+			cpifaceSession->ringbufferAPI->get_tail_samples (oggbufpos, &pos1, &length1, &pos2, &length2);
+
+			if (oggbufrate==0x10000)
 			{
-				bufdelta=(length1+length2);
-				looped |= 2;
-			} else {
-				looped &= ~2;
-			}
-
-			for (buf16_filled=0; buf16_filled<bufdelta; buf16_filled++)
-			{
-				int16_t rs, ls;
-
-				if (!length1)
+				if (targetlength>(length1+length2))
 				{
-					pos1 = pos2;
-					length1 = length2;
-					pos2 = 0;
+					targetlength=(length1+length2); // limiting targetlength here, saves us from doing this per sample later
+					ogg_looped |= 2;
+				} else {
+					ogg_looped &= ~2;
+				}
+
+				// limit source to not overrun target buffer
+				if (length1 > targetlength)
+				{
+					length1 = targetlength;
 					length2 = 0;
+				} else if ((length1 + length2) > targetlength)
+				{
+					length2 = targetlength - length1;
 				}
 
-				if (!length1)
+				accumulated_source = accumulated_target = length1 + length2;
+
+				while (length1)
 				{
-					fprintf (stderr, "playogg: ERROR, length1 == 0, in oggIdle\n");
-					_exit(1);
-				}
+					while (length1)
+					{
+						int16_t rs, ls;
 
-				rs = oggbuf[pos1<<1];
-				ls = oggbuf[(pos1<<1) + 1];
+						rs = oggbuf[pos1<<1];
+						ls = oggbuf[(pos1<<1) + 1];
 
-				PANPROC;
+						PANPROC;
 
-				*(t++) = rs;
-				*(t++) = ls;
+						*(t++) = rs;
+						*(t++) = ls;
 
-				pos1++;
-				length1--;
-			}
-
-			ringbuffer_tail_consume_samples (oggbufpos, buf16_filled); /* add this rate buf16_filled == tail_used */
-		} else {
-			/* We are going to perform cubic interpolation of rate conversion... this bit is tricky */
-			unsigned int accumulated_progress = 0;
-
-			looped &= ~2;
-
-			for (buf16_filled=0; buf16_filled<bufdelta; buf16_filled++)
-			{
-				uint32_t wpm1, wp0, wp1, wp2;
-				int32_t rc0, rc1, rc2, rc3, rvm1,rv1,rv2;
-				int32_t lc0, lc1, lc2, lc3, lvm1,lv1,lv2;
-				unsigned int progress;
-				int16_t rs, ls;
-
-				/* will the interpolation overflow? */
-				if ((length1+length2) <= 3)
-				{
-					looped |= 2;
-					break;
-				}
-				/* will we overflow the wavebuf if we advance? */
-				if ((length1+length2) < ((oggbufrate+oggbuffpos)>>16))
-				{
-					looped |= 2;
-					break;
-				}
-
-				switch (length1) /* if we are close to the wrap between buffer segment 1 and 2, len1 will grow down to a small number */
-				{
-					case 1:
-						wpm1 = pos1;
-						wp0  = pos2;
-						wp1  = pos2+1;
-						wp2  = pos2+2;
-						break;
-					case 2:
-						wpm1 = pos1;
-						wp0  = pos1+1;
-						wp1  = pos2;
-						wp2  = pos2+1;
-						break;
-					case 3:
-						wpm1 = pos1;
-						wp0  = pos1+1;
-						wp1  = pos1+2;
-						wp2  = pos2;
-						break;
-					default:
-						wpm1 = pos1;
-						wp0  = pos1+1;
-						wp1  = pos1+2;
-						wp2  = pos1+3;
-						break;
-				}
-
-
-				rvm1 = (uint16_t)oggbuf[(wpm1<<1)+0]^0x8000; /* we temporary need data to be unsigned - hence the ^0x8000 */
-				lvm1 = (uint16_t)oggbuf[(wpm1<<1)+1]^0x8000;
-				 rc0 = (uint16_t)oggbuf[(wp0<<1)+0]^0x8000;
-				 lc0 = (uint16_t)oggbuf[(wp0<<1)+1]^0x8000;
-				 rv1 = (uint16_t)oggbuf[(wp1<<1)+0]^0x8000;
-				 lv1 = (uint16_t)oggbuf[(wp1<<1)+1]^0x8000;
-				 rv2 = (uint16_t)oggbuf[(wp2<<1)+0]^0x8000;
-				 lv2 = (uint16_t)oggbuf[(wp2<<1)+1]^0x8000;
-
-				rc1 = rv1-rvm1;
-				rc2 = 2*rvm1-2*rc0+rv1-rv2;
-				rc3 = rc0-rvm1-rv1+rv2;
-				rc3 =  imulshr16(rc3,oggbuffpos);
-				rc3 += rc2;
-				rc3 =  imulshr16(rc3,oggbuffpos);
-				rc3 += rc1;
-				rc3 =  imulshr16(rc3,oggbuffpos);
-				rc3 += rc0;
-				if (rc3<0)
-					rc3=0;
-				if (rc3>65535)
-					rc3=65535;
-
-				lc1 = lv1-lvm1;
-				lc2 = 2*lvm1-2*lc0+lv1-lv2;
-				lc3 = lc0-lvm1-lv1+lv2;
-				lc3 =  imulshr16(lc3,oggbuffpos);
-				lc3 += lc2;
-				lc3 =  imulshr16(lc3,oggbuffpos);
-				lc3 += lc1;
-				lc3 =  imulshr16(lc3,oggbuffpos);
-				lc3 += lc0;
-				if (lc3<0)
-					lc3=0;
-				if (lc3>65535)
-					lc3=65535;
-
-				rs = rc3 ^ 0x8000;
-				ls = lc3 ^ 0x8000;
-
-				PANPROC;
-
-				buf16[(buf16_filled<<1)+0] = rs;
-				buf16[(buf16_filled<<1)+1] = ls;
-
-				oggbuffpos+=oggbufrate;
-				progress = oggbuffpos>>16;
-				oggbuffpos &= 0xffff;
-
-				accumulated_progress += progress;
-
-				/* did we wrap? if so, progress up to the wrapping point */
-				if (progress >= length1)
-				{
-					progress -= length1;
-					pos1 = pos2;
+						pos1++;
+						length1--;
+						//accumulated_target++;
+					}
 					length1 = length2;
-					pos2 = 0;
 					length2 = 0;
+					pos1 = pos2;
+					pos2 = 0;
 				}
-				if (progress)
-				{
-					pos1 += progress;
-					length1 -= progress;
-				}
-			}
-			ringbuffer_tail_consume_samples (oggbufpos, accumulated_progress);
-		}
-
-		bufdelta=buf16_filled;
-
-		if ((bufpos+bufdelta)>buflen)
-			pass2=bufpos+bufdelta-buflen;
-		else
-			pass2=0;
-		bufdelta-=pass2;
-
-		if (bit16)
-		{
-			if (stereo)
-			{
-				int16_t *p=(int16_t *)plrbuf+2*bufpos;
-				int16_t *b=buf16;
-				if (signedout)
-				{
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[0];
-						p[1]=b[1];
-						p+=2;
-						b+=2;
-					}
-					p=(int16_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[0];
-						p[1]=b[1];
-						p+=2;
-						b+=2;
-					}
-				} else {
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[0]^0x8000;
-						p[1]=b[1]^0x8000;
-						p+=2;
-						b+=2;
-					}
-					p=(int16_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[0]^0x8000;
-						p[1]=b[1]^0x8000;
-						p+=2;
-						b+=2;
-					}
-				}
+				//accumulated_source = accumulated_target;
 			} else {
-				int16_t *p=(int16_t *)plrbuf+bufpos;
-				int16_t *b=buf16;
-				if (signedout)
+				ogg_looped &= ~2;
+
+				while (targetlength && length1)
 				{
-					for (i=0; i<bufdelta; i++)
+					while (targetlength && length1)
 					{
-						p[0]=b[0];
-						p++;
-						b++;
-					}
-					p=(int16_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[0];
-						p++;
-						b++;
-					}
-				} else {
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[0]^0x8000;
-						p++;
-						b++;
-					}
-					p=(int16_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[0]^0x8000;
-						p++;
-						b++;
-					}
-				}
-			}
-		} else {
-			if (stereo)
-			{
-				uint8_t *p=(uint8_t *)plrbuf+2*bufpos;
-				uint8_t *b=(uint8_t *)buf16;
-				if (signedout)
-				{
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[1];
-						p[1]=b[3];
-						p+=2;
-						b+=4;
-					}
-					p=(uint8_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[1];
-						p[1]=b[3];
-						p+=2;
-						b+=4;
-					}
-				} else {
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=b[1]^0x80;
-						p[1]=b[3]^0x80;
-						p+=2;
-						b+=4;
-					}
-					p=(uint8_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=b[1]^0x80;
-						p[1]=b[3]^0x80;
-						p+=2;
-						b+=4;
-					}
-				}
-			} else {
-				uint8_t *p=(uint8_t *)plrbuf+bufpos;
-				int16_t *b=buf16;
-				if (signedout)
-				{
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=(b[0]+b[1])>>9;
-						p++;
-						b+=2;
-					}
-					p=(uint8_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=(b[0]+b[1])>>9;
-						p++;
-						b+=2;
-					}
-				} else {
-					for (i=0; i<bufdelta; i++)
-					{
-						p[0]=((b[0]+b[1])>>9)^0x80;
-						p++;
-						b+=2;
-					}
-					p=(uint8_t *)plrbuf;
-					for (i=0; i<pass2; i++)
-					{
-						p[0]=((b[0]+b[1])>>9)^0x80;
-						p++;
-						b+=2;
-					}
-				}
-			}
-		}
-		bufpos+=buf16_filled;
-		if (bufpos>=buflen)
-			bufpos-=buflen;
+						uint32_t wpm1, wp0, wp1, wp2;
+						int32_t rc0, rc1, rc2, rc3, rvm1,rv1,rv2;
+						int32_t lc0, lc1, lc2, lc3, lvm1,lv1,lv2;
+						unsigned int progress;
+						int16_t rs, ls;
+
+						if ((length1+length2) <= 3)
+						{
+							ogg_looped |= 2;
+							break;
+						}
+						/* will we overflow the oggbuf if we advance? */
+						if ((length1+length2) < ((oggbufrate+oggbuffpos)>>16))
+						{
+							ogg_looped |= 2;
+							break;
+						}
+
+						switch (length1) /* if we are close to the wrap between buffer segment 1 and 2, len1 will grow down to a small number */
+						{
+							case 1:  wpm1 = pos1; wp0 = pos2;     wp1 = pos2 + 1; wp2 = pos2 + 2; break;
+							case 2:  wpm1 = pos1; wp0 = pos1 + 1; wp1 = pos2;     wp2 = pos2 + 1; break;
+							case 3:  wpm1 = pos1; wp0 = pos1 + 1; wp1 = pos1 + 2; wp2 = pos2;     break;
+							default: wpm1 = pos1; wp0 = pos1 + 1; wp1 = pos1 + 2; wp2 = pos1 + 3; break;
+						}
+
+						rvm1 = (uint16_t)oggbuf[(wpm1<<1)+0]^0x8000; /* we temporary need data to be unsigned - hence the ^0x8000 */
+						lvm1 = (uint16_t)oggbuf[(wpm1<<1)+1]^0x8000;
+						 rc0 = (uint16_t)oggbuf[(wp0 <<1)+0]^0x8000;
+						 lc0 = (uint16_t)oggbuf[(wp0 <<1)+1]^0x8000;
+						 rv1 = (uint16_t)oggbuf[(wp1 <<1)+0]^0x8000;
+						 lv1 = (uint16_t)oggbuf[(wp1 <<1)+1]^0x8000;
+						 rv2 = (uint16_t)oggbuf[(wp2 <<1)+0]^0x8000;
+						 lv2 = (uint16_t)oggbuf[(wp2 <<1)+1]^0x8000;
+
+						rc1 = rv1-rvm1;
+						rc2 = 2*rvm1-2*rc0+rv1-rv2;
+						rc3 = rc0-rvm1-rv1+rv2;
+						rc3 =  imulshr16(rc3,oggbuffpos);
+						rc3 += rc2;
+						rc3 =  imulshr16(rc3,oggbuffpos);
+						rc3 += rc1;
+						rc3 =  imulshr16(rc3,oggbuffpos);
+						rc3 += rc0;
+						if (rc3<0)
+							rc3=0;
+						if (rc3>65535)
+							rc3=65535;
+
+						lc1 = lv1-lvm1;
+						lc2 = 2*lvm1-2*lc0+lv1-lv2;
+						lc3 = lc0-lvm1-lv1+lv2;
+						lc3 =  imulshr16(lc3,oggbuffpos);
+						lc3 += lc2;
+						lc3 =  imulshr16(lc3,oggbuffpos);
+						lc3 += lc1;
+						lc3 =  imulshr16(lc3,oggbuffpos);
+						lc3 += lc0;
+						if (lc3<0)
+							lc3=0;
+						if (lc3>65535)
+							lc3=65535;
+
+						rs = rc3 ^ 0x8000;
+						ls = lc3 ^ 0x8000;
+
+						PANPROC;
+
+						*(t++) = rs;
+						*(t++) = ls;
+
+						oggbuffpos+=oggbufrate;
+						progress = oggbuffpos>>16;
+						oggbuffpos &= 0xffff;
+						accumulated_source+=progress;
+						pos1+=progress;
+						length1-=progress;
+						targetlength--;
+
+						accumulated_target++;
+					} /* while (targetlength && length1) */
+					length1 = length2;
+					length2 = 0;
+					pos1 = pos2;
+					pos2 = 0;
+				} /* while (targetlength && length1) */
+			} /* if (oggbufrate==0x10000) */
+			cpifaceSession->ringbufferAPI->tail_consume_samples (oggbufpos, accumulated_source);
+			cpifaceSession->plrDevAPI->CommitBuffer (accumulated_target);
+		} /* if (targetlength) */
 	}
 
-	plrAdvanceTo(bufpos<<(stereo+bit16));
-
-	if (plrIdle)
-		plrIdle();
+	cpifaceSession->plrDevAPI->Idle();
 
 	clipbusy--;
 }
@@ -649,10 +431,10 @@ static long tell_func (void *token)
 }
 
 
-struct ogg_comment_t __attribute__ ((visibility ("internal"))) **ogg_comments;
-int                  __attribute__ ((visibility ("internal")))   ogg_comments_count;
-struct ogg_picture_t __attribute__ ((visibility ("internal")))  *ogg_pictures;
-int                  __attribute__ ((visibility ("internal")))   ogg_pictures_count;
+OCP_INTERNAL struct ogg_comment_t **ogg_comments;
+OCP_INTERNAL int                    ogg_comments_count;
+OCP_INTERNAL struct ogg_picture_t  *ogg_pictures;
+OCP_INTERNAL int                    ogg_pictures_count;
 
 static void add_comment2(const char *title, const char *value)
 {
@@ -712,7 +494,7 @@ static void add_picture(const uint16_t actual_width,
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wunused-but-set-variable"
 #endif
-static void add_picture_binary(const uint8_t *src, unsigned int srclen)
+static void add_picture_binary (struct cpifaceSessionAPI_t *cpifaceSession, const uint8_t *src, unsigned int srclen)
 {
 	uint32_t picture_type;
 	uint32_t mime_length;
@@ -814,7 +596,7 @@ static void add_picture_binary(const uint8_t *src, unsigned int srclen)
 	{
 		uint16_t actual_height, actual_width;
 		uint8_t *data_bgra;
-		if (!GIF87_try_open_bgra (&actual_width, &actual_height, &data_bgra, data, data_length))
+		if (!cpifaceSession->console->try_open_gif (&actual_width, &actual_height, &data_bgra, data, data_length))
 		{
 			add_picture (actual_width, actual_height, data_bgra, (const char *)description, description_length, picture_type);
 		}
@@ -826,7 +608,7 @@ static void add_picture_binary(const uint8_t *src, unsigned int srclen)
 	{
 		uint16_t actual_height, actual_width;
 		uint8_t *data_bgra;
-		if (!try_open_png (&actual_width, &actual_height, &data_bgra, (uint8_t *)data, data_length))
+		if (!cpifaceSession->console->try_open_png (&actual_width, &actual_height, &data_bgra, (uint8_t *)data, data_length))
 		{
 			add_picture (actual_width, actual_height, data_bgra, (const char *)description, description_length, picture_type);
 		}
@@ -837,7 +619,7 @@ static void add_picture_binary(const uint8_t *src, unsigned int srclen)
 	{
 		uint16_t actual_height, actual_width;
 		uint8_t *data_bgra;
-		if (!try_open_jpeg (&actual_width, &actual_height, &data_bgra, data, data_length))
+		if (!cpifaceSession->console->try_open_jpeg (&actual_width, &actual_height, &data_bgra, data, data_length))
 		{
 			add_picture (actual_width, actual_height, data_bgra, (const char *)description, description_length, picture_type);
 		}
@@ -859,7 +641,7 @@ static uint8_t base64_index (const char src)
 	return 65;
 }
 
-static void add_picture_base64(const char *src)
+static void add_picture_base64 (struct cpifaceSessionAPI_t *cpifaceSession, const char *src)
 {
 	int srclen = strlen (src);
 	uint8_t *dst;
@@ -924,17 +706,17 @@ static void add_picture_base64(const char *src)
 
 	dst -= dstlen;
 
-	add_picture_binary(dst, dstlen);
+	add_picture_binary (cpifaceSession, dst, dstlen);
 
 	free (dst);
 }
 
-static void add_comment(const char *src)
+static void add_comment (struct cpifaceSessionAPI_t *cpifaceSession, const char *src)
 {
 	char *equal, *tmp, *tmp2;
 	if (!strncasecmp (src, "METADATA_BLOCK_PICTURE=", 23))
 	{
-		add_picture_base64(src + 23);
+		add_picture_base64 (cpifaceSession, src + 23);
 		return;
 	}
 	equal = strchr (src, '=');
@@ -964,26 +746,26 @@ static void add_comment(const char *src)
 	free (tmp);
 }
 
-char __attribute__ ((visibility ("internal"))) oggLooped(void)
+OCP_INTERNAL char oggLooped (void)
 {
-	return looped == 3;
+	return ogg_looped == 3;
 }
 
-void __attribute__ ((visibility ("internal"))) oggSetLoop(uint8_t s)
+OCP_INTERNAL void oggSetLoop (uint8_t s)
 {
 	donotloop=!s;
 }
 
-void __attribute__ ((visibility ("internal"))) oggPause(uint8_t p)
+OCP_INTERNAL void oggPause (uint8_t p)
 {
-	inpause=p;
+	ogg_inpause=p;
 }
 
 static void oggSetSpeed (uint16_t sp)
 {
-	if (sp<32)
-		sp=32;
-	oggbufrate=imuldiv(256*sp, oggrate, samprate);
+	if (sp < 4)
+		sp = 4;
+	oggbufrate=imuldiv(256*sp, oggrate, oggRate);
 }
 
 static void oggSetVolume (void)
@@ -995,7 +777,7 @@ static void oggSetVolume (void)
 		volr=(volr*(64-bal))>>6;
 }
 
-static void SET(int ch, int opt, int val)
+static void oggSet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt, int val)
 {
 	switch (opt)
 	{
@@ -1009,10 +791,6 @@ static void SET(int ch, int opt, int val)
 			break;
 		case mcpMasterPanning:
 			pan=val;
-			if (reversestereo)
-			{
-				pan = -pan;
-			}
 			oggSetVolume();
 			break;
 		case mcpMasterVolume:
@@ -1025,21 +803,21 @@ static void SET(int ch, int opt, int val)
 			break;
 	}
 }
-static int GET(int ch, int opt)
+
+static int oggGet (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int opt)
 {
 	return 0;
 }
 
-ogg_int64_t __attribute__ ((visibility ("internal"))) oggGetPos(void)
+OCP_INTERNAL ogg_int64_t oggGetPos (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-#warning TODO, deduct devp buffer!
-	return (oggpos+ogglen-ringbuffer_get_tail_available_samples(oggbufpos))%ogglen;
+	return (ogglen + ogglen + oggpos - cpifaceSession->ringbufferAPI->get_tail_available_samples(oggbufpos) - cpifaceSession->plrDevAPI->Idle())%ogglen;
 }
 
-void __attribute__ ((visibility ("internal"))) oggGetInfo(struct ogginfo *info)
+OCP_INTERNAL void oggGetInfo (struct cpifaceSessionAPI_t *cpifaceSession, struct ogginfo *info)
 {
 	static int lastsafe=0;
-	info->pos=oggGetPos();
+	info->pos = oggGetPos (cpifaceSession);
 	info->len=ogglen;
 	info->rate=oggrate;
 	info->stereo=oggstereo;
@@ -1062,174 +840,18 @@ void __attribute__ ((visibility ("internal"))) oggGetInfo(struct ogginfo *info)
 	info->opt50=opt50;
 }
 
-void __attribute__ ((visibility ("internal"))) oggSetPos(ogg_int64_t pos)
+OCP_INTERNAL void oggSetPos (struct cpifaceSessionAPI_t *cpifaceSession, ogg_int64_t pos)
 {
 	pos=(pos+ogglen)%ogglen;
 
 	oggneedseek=1;
 	oggpos=pos;
-	ringbuffer_reset(oggbufpos);
+	cpifaceSession->ringbufferAPI->reset (oggbufpos);
 }
 
-static ov_callbacks callbacks =
-{
-	read_func,
-	seek_func,
-	close_func,
-	tell_func
-};
-int __attribute__ ((visibility ("internal"))) oggOpenPlayer(struct ocpfilehandle_t *oggf)
-{
-	int result;
-	struct vorbis_info *vi;
-
-	if (!plrPlay)
-		return 0;
-
-	oggf->seek_set (oggf, 0);
-	if (oggfile)
-	{
-		oggfile->unref (oggfile);
-		oggfile = 0;
-	}
-	oggfile = oggf;
-	oggfile->ref (oggfile);
-	if ((result = ov_open_callbacks(oggfile, &ov, NULL, 0, callbacks)))
-	{
-		switch (result)
-		{
-			case OV_EREAD:      fprintf (stderr, "ov_open_callbacks(): A read from media returned an error.\n"); break;
-			case OV_ENOTVORBIS: fprintf (stderr, "ov_open_callbacks(): Bitstream does not contain any Vorbis data.\n"); break;
-			case OV_EVERSION:   fprintf (stderr, "ov_open_callbacks(): Vorbis version mismatch.\n"); break;
-			case OV_EBADHEADER: fprintf (stderr, "ov_open_callbacks(): Invalid Vorbis bitstream header.\n"); break;
-			case OV_EFAULT:     fprintf (stderr, "ov_open_callbacks(): Internal logic fault; indicates a bug or heap/stack corruption.\n"); break;
-			default:            fprintf (stderr, "ov_open_callbacks(): Unknown error %d\n", result); break;
-		}
-		return 0; /* we don't bother to do more exact */
-	}
-
-	vi=ov_info(&ov,-1);
-	oggstereo=(vi->channels>=2);
-	oggrate=vi->rate;
-
-	plrSetOptions(oggrate, (PLR_SIGNEDOUT|PLR_16BIT)|PLR_STEREO);
-	stereo=!!(plrOpt&PLR_STEREO);
-	bit16=!!(plrOpt&PLR_16BIT);
-	signedout=!!(plrOpt&PLR_SIGNEDOUT);
-	reversestereo=!!(plrOpt&PLR_REVERSESTEREO);
-	samprate=plrRate;
-
-	oggbufrate=imuldiv(65536, oggrate, samprate);
-
-	ogglen=ov_pcm_total(&ov, -1);
-	if (!ogglen)
-		return 0;
-
-	oggbuf=malloc(1024 * 128);
-	if (!oggbuf)
-		return 0;
-	oggbufpos = ringbuffer_new_samples (RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_16BIT | RINGBUFFER_FLAGS_SIGNED, 1024*32);
-	if (!oggbufpos)
-	{
-		free(oggbuf);
-		oggbuf = 0;
-		return 0;
-	}
-	oggbuffpos=0;
-	current_section=0;
-	oggneedseek=0;
-
-	{
-		int i;
-
-		vorbis_comment *vf = 0;
-		vf = ov_comment (&ov, -1);
-		if (vf)
-		{
-			for (i=0; i < vf->comments; i++)
-			{
-				add_comment(vf->user_comments[i]);
-			}
-		}
-	}
-
-	if (!plrOpenPlayer(&plrbuf, &buflen, plrBufSize * plrRate / 1000, oggf))
-	{
-		ringbuffer_free (oggbufpos);
-		oggbufpos = 0;
-
-		free(oggbuf);
-		oggbuf = 0;
-		return 0;
-	}
-
-	inpause=0;
-	looped=0;
-
-	buf16=malloc(sizeof(uint16_t)*buflen*2);
-	if (!buf16)
-	{
-		plrClosePlayer();
-
-		ringbuffer_free (oggbufpos);
-		oggbufpos = 0;
-
-		free(oggbuf);
-		oggbuf = 0;
-		return 0;
-	}
-	bufpos=0;
-
-	if (!pollInit(oggIdle))
-	{
-		plrClosePlayer();
-
-		free (buf16);
-		buf16 = 0;
-
-		ringbuffer_free (oggbufpos);
-		oggbufpos = 0;
-
-		free(oggbuf);
-		oggbuf = 0;
-
-		return 0;
-	}
-
-	_SET=mcpSet;
-	_GET=mcpGet;
-	mcpSet=SET;
-	mcpGet=GET;
-
-	mcpNormalize (mcpNormalizeDefaultPlayP);
-
-	active=1;
-	opt25_50 = 0;
-	opt25[0] = 0;
-	opt50[0] = 0;
-
-	return 1;
-}
-
-void __attribute__ ((visibility ("internal"))) oggClosePlayer(void)
+static void oggFreeComments (void)
 {
 	int i, j;
-	active=0;
-
-	pollClose();
-
-	plrClosePlayer();
-
-	ringbuffer_free (oggbufpos);
-	oggbufpos = 0;
-
-	free(oggbuf);
-	oggbuf=NULL;
-
-	free(buf16);
-	buf16=NULL;
-
-	ov_clear(&ov);
 
 	for (i=0; i < ogg_comments_count; i++)
 	{
@@ -1253,22 +875,159 @@ void __attribute__ ((visibility ("internal"))) oggClosePlayer(void)
 	free (ogg_pictures);
 	ogg_pictures = 0;
 	ogg_pictures_count = 0;
+}
+
+static ov_callbacks callbacks =
+{
+	read_func,
+	seek_func,
+	close_func,
+	tell_func
+};
+OCP_INTERNAL int oggOpenPlayer (struct ocpfilehandle_t *oggf, struct cpifaceSessionAPI_t *cpifaceSession)
+{
+	enum plrRequestFormat format;
+	int result;
+	int retval;
+	struct vorbis_info *vi;
+
+	if (!cpifaceSession->plrDevAPI)
+	{
+		return errPlay;
+	}
+
+	oggfile = oggf;
+	oggfile->seek_set (oggfile, 0);
+	oggfile->ref (oggfile);
+	if ((result = ov_open_callbacks(oggfile, &ov, NULL, 0, callbacks)))
+	{
+		switch (result)
+		{
+			case OV_EREAD:      cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): A read from media returned an error.\n"); break;
+			case OV_ENOTVORBIS: cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): Bitstream does not contain any Vorbis data.\n"); break;
+			case OV_EVERSION:   cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): Vorbis version mismatch.\n"); break;
+			case OV_EBADHEADER: cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): Invalid Vorbis bitstream header.\n"); break;
+			case OV_EFAULT:     cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): Internal logic fault; indicates a bug or heap/stack corruption.\n"); break;
+			default:            cpifaceSession->cpiDebug (cpifaceSession, "[OGG] ov_open_callbacks(): Unknown error %d\n", result); break;
+		}
+		retval = errFormStruc;
+		goto error_out_file;
+	}
+
+	vi=ov_info(&ov,-1);
+	oggstereo=(vi->channels>=2);
+	oggrate=vi->rate;
+
+	oggRate=oggrate;
+	format=PLR_STEREO_16BIT_SIGNED;
+	if (!cpifaceSession->plrDevAPI->Play (&oggRate, &format, oggfile, cpifaceSession))
+	{
+		retval = errPlay;
+		goto error_out_file;
+	}
+
+	oggbufrate=imuldiv(65536, oggrate, oggRate);
+
+	oggpos=0;
+	ogglen=ov_pcm_total(&ov, -1);
+	if (!ogglen)
+	{
+		retval = errFormStruc;
+		goto error_out_plrDevAPI_Play;
+	}
+
+	oggbuf=malloc(1024 * 128);
+	if (!oggbuf)
+	{
+		retval = errAllocMem;
+		goto error_out_plrDevAPI_Play;
+	}
+	oggbufpos = cpifaceSession->ringbufferAPI->new_samples (RINGBUFFER_FLAGS_STEREO | RINGBUFFER_FLAGS_16BIT | RINGBUFFER_FLAGS_SIGNED, 1024*32);
+	if (!oggbufpos)
+	{
+		retval = errAllocMem;
+		goto error_out_oggbuf;
+	}
+	oggbuffpos=0;
+	current_section=0;
+	oggneedseek=0;
+
+	{
+		int i;
+
+		vorbis_comment *vf = 0;
+		vf = ov_comment (&ov, -1);
+		if (vf)
+		{
+			for (i=0; i < vf->comments; i++)
+			{
+				add_comment (cpifaceSession, vf->user_comments[i]);
+			}
+		}
+	}
+
+	ogg_inpause=0;
+	ogg_looped=0;
+
+	cpifaceSession->mcpSet = oggSet;
+	cpifaceSession->mcpGet = oggGet;
+
+	cpifaceSession->Normalize (cpifaceSession, mcpNormalizeDefaultPlayP);
+
+	active=1;
+	opt25_50 = 0;
+	opt25[0] = 0;
+	opt50[0] = 0;
+
+	return errOk;
+
+	//cpifaceSession->ringbufferAPI->free (oggbufpos);
+	//oggbufpos = 0;
+
+error_out_oggbuf:
+	free(oggbuf);
+	oggbuf = 0;
+
+error_out_plrDevAPI_Play:
+	cpifaceSession->plrDevAPI->Stop (cpifaceSession);
+
+error_out_file:
+	ov_clear(&ov);
+
+	oggFreeComments ();
 
 	if (oggfile)
 	{
 		oggfile->unref (oggfile);
 		oggfile = 0;
 	}
-
-	if (_SET)
-	{
-		mcpSet = _SET;
-		_SET = 0;
-	}
-	if (_GET)
-	{
-		mcpGet = _GET;
-		_GET = 0;
-	}
+	return retval;
 }
 
+OCP_INTERNAL void oggClosePlayer (struct cpifaceSessionAPI_t *cpifaceSession)
+{
+	if (active)
+	{
+		cpifaceSession->plrDevAPI->Stop (cpifaceSession);
+	}
+	active=0;
+
+	if (oggbufpos)
+	{
+		cpifaceSession->ringbufferAPI->free (oggbufpos);
+		oggbufpos = 0;
+	}
+
+	free(oggbuf);
+	oggbuf=NULL;
+
+	ov_clear(&ov);
+
+	oggFreeComments ();
+
+	if (oggfile)
+	{
+		oggfile->unref (oggfile);
+		oggfile = 0;
+	}
+}
