@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2020-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2020-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Glue logic for using libsidplayfp playback engine
  *
@@ -18,8 +18,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-
-
 #include "libsidplayfp-api.h"
 #include <iostream>
 #include <fstream>
@@ -32,14 +30,28 @@
 
 extern "C"
 {
+#ifdef DPACKAGE_NAME
+#undef DPACKAGE_NAME
+#endif
+
+#ifdef VERSION
+#undef VERSION
+#endif
+
+#ifdef PACKAGE_VERSION
+#undef PACKAGE_VERSION
+#endif
+
+#include "../config.h"
+#include <stdio.h>
+#include <unistd.h>
+#include "types.h"
 #include "boot/psetting.h"
 #include "filesel/dirdb.h"
-#include "filesel/filesystem-unix.h"
-#include <stdio.h>
+#include "filesel/filesystem-drive.h"
+#include "filesel/filesystem.h"
 }
 
-#include <unistd.h>
-
 namespace libsidplayfp
 {
 #if 0
@@ -127,7 +139,7 @@ namespace libsidplayfp
 		}
 	}
 
-	ConsolePlayer::ConsolePlayer (const unsigned int rate) :
+	ConsolePlayer::ConsolePlayer (const unsigned int rate, const struct configAPI_t *configAPI, const struct dirdbAPI_t *dirdbAPI, struct dmDrive *dmFile) :
 		m_tune(nullptr),
 		m_state(playerStopped),
 		selected_track(0),
@@ -138,7 +150,7 @@ namespace libsidplayfp
 		m_engCfg = sidplayer.config();
 		m_engCfg.powerOnDelay = 10000;
 
-		const char *defaultC64model = cfGetProfileString("libsidplayfp", "defaultC64", "PAL");
+		const char *defaultC64model = configAPI->GetProfileString("libsidplayfp", "defaultC64", "PAL");
 		if (!strcasecmp(defaultC64model, "PAL"))
 		{
 			//fprintf (stderr, "defaultC64=PAL\n");
@@ -168,10 +180,10 @@ namespace libsidplayfp
 			m_engCfg.defaultC64Model = SidConfig::PAL;
 		}
 
-		m_engCfg.forceC64Model = cfGetProfileBool("libsidplayfp", "forceC64", 0, 0);
+		m_engCfg.forceC64Model = configAPI->GetProfileBool("libsidplayfp", "forceC64", 0, 0);
 		//fprintf (stderr, "forceC64Model=%d\n", m_engCfg.forceC64Model);
 
-		const char *defaultSIDmodel = cfGetProfileString("libsidplayfp", "defaultSID", "MOS6581");
+		const char *defaultSIDmodel = configAPI->GetProfileString("libsidplayfp", "defaultSID", "MOS6581");
 		if (!strcasecmp(defaultSIDmodel, "MOS6581"))
 		{
 			//fprintf (stderr, "defaultSID=MOS6581\n");
@@ -185,10 +197,10 @@ namespace libsidplayfp
 			m_engCfg.defaultSidModel = SidConfig::MOS6581;
 		}
 
-		m_engCfg.forceSidModel = cfGetProfileBool("libsidplayfp", "forceSID", 0, 0);
+		m_engCfg.forceSidModel = configAPI->GetProfileBool("libsidplayfp", "forceSID", 0, 0);
 		//fprintf (stderr, "forceSIDModel=%d\n", m_engCfg.forceSidModel);
 
-		const char *CIAmodel = cfGetProfileString("libsidplayfp", "CIA", "MOS6526");
+		const char *CIAmodel = configAPI->GetProfileString("libsidplayfp", "CIA", "MOS6526");
 		if (!strcasecmp(CIAmodel, "MOS6526"))
 		{
 			//fprintf (stderr, "CIA=MOS6526\n");
@@ -210,10 +222,10 @@ namespace libsidplayfp
 
 		m_engCfg.playback = SidConfig::STEREO;
 
-		m_filter.enabled = cfGetProfileBool("libsidplayfp", "filter", 1, 0);
+		m_filter.enabled = configAPI->GetProfileBool("libsidplayfp", "filter", 1, 0);
 		//fprintf (stderr, "filter=%d\n", m_filter.enabled);
 
-		const char *bias = cfGetProfileString("libsidplayfp", "filterbias", "0.0");
+		const char *bias = configAPI->GetProfileString("libsidplayfp", "filterbias", "0.0");
 		m_filter.bias = strtod(bias, &endptr);
 		if ((*endptr != 0) || (bias == endptr))
 		{
@@ -223,7 +235,7 @@ namespace libsidplayfp
 			//fprintf (stderr, "filterbias=%lf\n", m_filter.bias);
 		}
 
-		const char *curve6581 = cfGetProfileString("libsidplayfp", "filtercurve6581", "0.5");
+		const char *curve6581 = configAPI->GetProfileString("libsidplayfp", "filtercurve6581", "0.5");
 		m_filter.filterCurve6581 = strtod(curve6581, &endptr);
 		if ((*endptr != 0) || (curve6581 == endptr))
 		{
@@ -233,7 +245,7 @@ namespace libsidplayfp
 			//fprintf (stderr, "filtercurve6581=%lf\n", m_filter.filterCurve6581);
 		}
 
-		const char *curve8580 = cfGetProfileString("libsidplayfp", "filtercurve8580", "0.5");
+		const char *curve8580 = configAPI->GetProfileString("libsidplayfp", "filtercurve8580", "0.5");
 		m_filter.filterCurve8580 = strtod(curve8580, &endptr);
 		if ((*endptr != 0) || (curve8580 == endptr))
 		{
@@ -243,42 +255,44 @@ namespace libsidplayfp
 			//fprintf (stderr, "filtercurve8580=%lf\n", m_filter.filterCurve8580);
 		}
 
-		m_engCfg.digiBoost = cfGetProfileBool("libsidplayfp", "digiboost", 0, 0);
+		m_engCfg.digiBoost = configAPI->GetProfileBool("libsidplayfp", "digiboost", 0, 0);
 		//fprintf (stderr, "digiboost=%d\n", m_engCfg.digiBoost);
 
 		// TODO, add check of this return value
-		createSidEmu();
+		createSidEmu (configAPI);
 
-		const char *kernal_string  = cfGetProfileString("libsidplayfp", "kernal",  "KERNAL.ROM");
-		const char *basic_string   = cfGetProfileString("libsidplayfp", "basic",   "BASIC.ROM");
-		const char *chargen_string = cfGetProfileString("libsidplayfp", "chargen", "CHARGEN.ROM");
+		const char *kernal_string  = configAPI->GetProfileString("libsidplayfp", "kernal",  "KERNAL.ROM");
+		const char *basic_string   = configAPI->GetProfileString("libsidplayfp", "basic",   "BASIC.ROM");
+		const char *chargen_string = configAPI->GetProfileString("libsidplayfp", "chargen", "CHARGEN.ROM");
 
-		uint32_t dirdb_base = cfConfigDir_dirdbref; /* should be the parent_dir of the file you want to load */
+		uint32_t dirdb_base = dirdbAPI->ResolvePathWithBaseAndRef (dmFile->basedir->dirdb_ref, configAPI->DataHomeDir, 0, dirdb_use_dir); /* should be the parent_dir of the file you want to load */
 		uint32_t kernal_ref;
 		uint32_t basic_ref;
 		uint32_t chargen_ref;
 
 #ifdef __W32__
-		kernal_ref  = dirdbResolvePathWithBaseAndRef (dirdb_base, kernal_string,  DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME | DIRDB_RESOLVE_WINDOWS_SLASH);
-		basic_ref   = dirdbResolvePathWithBaseAndRef (dirdb_base, basic_string,   DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME | DIRDB_RESOLVE_WINDOWS_SLASH);
-		chargen_ref = dirdbResolvePathWithBaseAndRef (dirdb_base, chargen_string, DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME | DIRDB_RESOLVE_WINDOWS_SLASH);
+		kernal_ref  = dirdbAPI->ResolvePathWithBaseAndRef (dirdb_base, kernal_string,  DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME | DIRDB_RESOLVE_WINDOWS_SLASH);
+		basic_ref   = dirdbAPI->ResolvePathWithBaseAndRef (dirdb_base, basic_string,   DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME | DIRDB_RESOLVE_WINDOWS_SLASH);
+		chargen_ref = dirdbAPI->ResolvePathWithBaseAndRef (dirdb_base, chargen_string, DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME | DIRDB_RESOLVE_WINDOWS_SLASH);
 
 #else
-		kernal_ref  = dirdbResolvePathWithBaseAndRef (dirdb_base, kernal_string,  DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
-		basic_ref   = dirdbResolvePathWithBaseAndRef (dirdb_base, basic_string,   DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
-		chargen_ref = dirdbResolvePathWithBaseAndRef (dirdb_base, chargen_string, DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
+		kernal_ref  = dirdbAPI->ResolvePathWithBaseAndRef (dirdb_base, kernal_string,  DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
+		basic_ref   = dirdbAPI->ResolvePathWithBaseAndRef (dirdb_base, basic_string,   DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
+		chargen_ref = dirdbAPI->ResolvePathWithBaseAndRef (dirdb_base, chargen_string, DIRDB_RESOLVE_DRIVE | DIRDB_RESOLVE_TILDE_HOME, dirdb_use_file);
 
 #endif
-		uint8_t *kernalRom = loadRom(kernal_ref, 8192);
-		uint8_t *basicRom = loadRom(basic_ref, 8192);
-		uint8_t *chargenRom = loadRom(chargen_ref, 4096);
+		uint8_t *kernalRom  = loadRom ( kernal_ref, 8192, dirdbAPI);
+		uint8_t *basicRom   = loadRom (  basic_ref, 8192, dirdbAPI);
+		uint8_t *chargenRom = loadRom (chargen_ref, 4096, dirdbAPI);
 
-		dirdbUnref (kernal_ref, dirdb_use_file);
-		dirdbUnref (basic_ref, dirdb_use_file);
-		dirdbUnref (chargen_ref, dirdb_use_file);
+		dirdbAPI->Unref ( kernal_ref, dirdb_use_file);
+		dirdbAPI->Unref (  basic_ref, dirdb_use_file);
+		dirdbAPI->Unref (chargen_ref, dirdb_use_file);
 
-		sidplayer.setKernal(kernalRom);
-		sidplayer.setBasic(basicRom);
+		dirdbAPI->Unref ( dirdb_base, dirdb_use_dir);
+
+		sidplayer.setKernal ( kernalRom);
+		sidplayer.setBasic  (  basicRom);
 		sidplayer.setChargen(chargenRom);
 
 		delete [] kernalRom;
@@ -292,14 +306,14 @@ namespace libsidplayfp
 		delete &sidplayer;
 	}
 
-	uint8_t* ConsolePlayer::loadRom(uint32_t dirdb_ref, const int size)
+	uint8_t* ConsolePlayer::loadRom(uint32_t dirdb_ref, const int size, const struct dirdbAPI_t *dirdbAPI)
 	{
 		char *romPath = 0;
 #ifdef __W32__
 		#error we need to make flags, so we can reverse the slashes
-		dirdbGetFullname_malloc (dirdb_ref, &romPath, DIRDB_FULLNAME_DRIVE);
+		dirdbAPI->GetFullname_malloc (dirdb_ref, &romPath, DIRDB_FULLNAME_DRIVE);
 #else
-		dirdbGetFullname_malloc (dirdb_ref, &romPath, DIRDB_FULLNAME_NODRIVE);
+		dirdbAPI->GetFullname_malloc (dirdb_ref, &romPath, DIRDB_FULLNAME_NODRIVE);
 #endif
 		std::ifstream is(romPath, std::ios::binary);
 
@@ -338,12 +352,12 @@ namespace libsidplayfp
 
 
 	// Create the sid emulation
-	bool ConsolePlayer::createSidEmu (/*SIDEMUS emu*/)
+	bool ConsolePlayer::createSidEmu (const struct configAPI_t *configAPI /*SIDEMUS emu*/)
 	{
 		// Remove old driver and emulation - should be no-op
 		clearSidEmu ();
 
-		int use_residfp = !strcmp(cfGetProfileString("libsidplayfp", "emulator", "residfp"), "residfp");
+		int use_residfp = !strcmp(configAPI->GetProfileString("libsidplayfp", "emulator", "residfp"), "residfp");
 
 		//fprintf (stderr, "use_residfp=%d\n", use_residfp);
 
