@@ -1,3 +1,23 @@
+/* OpenCP Module Player
+ * copyright (c) 2022-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
+ *
+ * Timidity config (setup:) editor
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
 #include "config.h"
 #include <dirent.h>
 #include <stdio.h>
@@ -12,17 +32,17 @@
 #include "filesel/dirdb.h"
 #include "filesel/filesystem.h"
 #include "filesel/filesystem-drive.h"
-#include "filesel/filesystem-file-mem.h"
+#include "filesel/filesystem-file-dev.h"
 #include "filesel/filesystem-setup.h"
 #include "filesel/pfilesel.h"
 #include "stuff/compat.h"
 #include "stuff/err.h"
-#include "stuff/framelock.h"
 #include "stuff/poutput.h"
-#include "timidity-git/timidity/sysdep.h"
+#include "timidityconfig.h"
 #include "timidity-git/timidity/timidity.h"
+#include "timidity-git/timidity/sysdep.h"
 
-#ifdef TIMIDITY_DEBUG
+#ifdef PLAYTIMIDITY_DEBUG
 # define PRINT(fmt, args...) fprintf(stderr, "%s %s: " fmt, __FILE__, __func__, ##args)
 #else
 # define PRINT(a, ...) do {} while(0)
@@ -292,17 +312,12 @@ static void refresh_configfiles (void)
 	}
 }
 
-static int timidityConfigInit (struct moduleinfostruct *info, struct ocpfilehandle_t *f, const struct interfaceparameters *ip)
-{
-	return 1;
-}
-
-static void timidityConfigFileSelectDraw (int dsel)
+static void timidityConfigFileSelectDraw (int dsel, const struct DevInterfaceAPI_t *API)
 {
 	int mlWidth = 75;
-	int mlHeight = (plScrHeight >= 35) ? 33 : 23;
-	int mlTop = (plScrHeight - mlHeight) / 2 ;
-	int mlLeft = (plScrWidth - mlWidth) / 2;
+	int mlHeight = (API->console->TextHeight >= 35) ? 33 : 23;
+	int mlTop = (API->console->TextHeight - mlHeight) / 2 ;
+	int mlLeft = (API->console->TextWidth - mlWidth) / 2;
 	int mlLine;
 	int half;
 	int skip;
@@ -340,85 +355,85 @@ static void timidityConfigFileSelectDraw (int dsel)
 		dot = skip * (mlHeight - LINES_NOT_AVAILABLE) / (contentheight - (mlHeight - LINES_NOT_AVAILABLE));
 	}
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xda%*C\xc4\xbf", mlWidth - 2);
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o Please select a new configuration file using the arrow keys and press%*C %0.9o\xb3", mlWidth - 72);
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%0.15o <ENTER>%0.7o when done, or %0.15o<ESC>%0.7o to cancel.%*C %.9o\xb3", mlWidth - 41);
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xc3%*C\xc4\xb4", mlWidth - 2);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xda%*C\xc4\xbf", mlWidth - 2);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o Please select a new configuration file using the arrow keys and press%*C %0.9o\xb3", mlWidth - 72);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%0.15o <ENTER>%0.7o when done, or %0.15o<ESC>%0.7o to cancel.%*C %.9o\xb3", mlWidth - 41);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xc3%*C\xc4\xb4", mlWidth - 2);
 
 	for (mlLine = 4; (mlLine + 1) < mlHeight; mlLine++)
 	{
 		int masterindex = mlLine - 4 + skip;
 
-		displaychr  (mlTop, mlLeft,               0x09, '\xb3', 1);
-		displaychr  (mlTop, mlLeft + mlWidth - 1, 0x09, ((mlLine - 4) == dot) ? '\xdd' : '\xb3', 1);
+		API->console->Driver->DisplayChr  (mlTop, mlLeft,               0x09, '\xb3', 1);
+		API->console->Driver->DisplayChr  (mlTop, mlLeft + mlWidth - 1, 0x09, ((mlLine - 4) == dot) ? '\xdd' : '\xb3', 1);
 
 		if (masterindex == 0)
 		{
-			displaystr  (mlTop++, mlLeft + 1, 0x03, "System default", mlWidth - 2);
+			API->console->Driver->DisplayStr  (mlTop++, mlLeft + 1, 0x03, "System default", mlWidth - 2);
 			continue;
 		}
 		if (masterindex == 1)
 		{
-			displaychr (mlTop, mlLeft + 1, (dsel==0)?0x8a:0x0a, ' ', 2);
+			API->console->Driver->DisplayChr (mlTop, mlLeft + 1, (dsel==0)?0x8a:0x0a, ' ', 2);
 			if (have_default_timidity)
 			{
 				int pos = strlen (default_timidity_path) + 3;
-				displaystr (mlTop, mlLeft + 3, (dsel==0)?0x8a:0x0a, default_timidity_path, strlen (default_timidity_path));
+				API->console->Driver->DisplayStr (mlTop, mlLeft + 3, (dsel==0)?0x8a:0x0a, default_timidity_path, strlen (default_timidity_path));
 				if (have_user_timidity)
 				{
 					int len = (mlWidth - pos - 1) < 26 ? (mlWidth - pos - 1) : 26;
-					displaystr (mlTop, mlLeft + pos, (dsel==0)?0x87:0x07, " with user overrides from ", len);
+					API->console->Driver->DisplayStr (mlTop, mlLeft + pos, (dsel==0)?0x87:0x07, " with user overrides from ", len);
 					pos += len;
 					if (pos < (mlWidth - 2))
 					{
-						displaystr_utf8 (mlTop, mlLeft + pos, (dsel==0)?0x8f:0x0f, user_timidity_path, mlWidth - pos - 1);
+						API->console->Driver->DisplayStr_utf8 (mlTop, mlLeft + pos, (dsel==0)?0x8f:0x0f, user_timidity_path, mlWidth - pos - 1);
 					}
 				} else {
-					displaystr (mlTop, mlLeft + pos, (dsel==0)?0x87:0x07, " (with no user overrides)", mlWidth - 1 - pos);
+					API->console->Driver->DisplayStr (mlTop, mlLeft + pos, (dsel==0)?0x87:0x07, " (with no user overrides)", mlWidth - 1 - pos);
 				}
 			} else {
-				displaystr (mlTop, mlLeft + 1, (dsel==0)?0x8c:0x0c, " No global configuration file found", mlWidth - 2);
+				API->console->Driver->DisplayStr (mlTop, mlLeft + 1, (dsel==0)?0x8c:0x0c, " No global configuration file found", mlWidth - 2);
 			}
 			mlTop++;
 			continue;
 		}
 		if (masterindex == 3)
 		{
-			displaystr  (mlTop++, mlLeft + 1, 0x03, "Global configuration files:", mlWidth - 2);
+			API->console->Driver->DisplayStr  (mlTop++, mlLeft + 1, 0x03, "Global configuration files:", mlWidth - 2);
 			continue;
 		}
 		if ((masterindex == 4) && (!global_timidity_count))
 		{
-			displaystr  (mlTop++, mlLeft + 1, 0x0c, " No configuration files found", mlWidth - 2);
+			API->console->Driver->DisplayStr  (mlTop++, mlLeft + 1, 0x0c, " No configuration files found", mlWidth - 2);
 			continue;
 		}
 		if ((masterindex >= 4) && (masterindex < (global_timidity_count + 4)))
 		{
-			display_nprintf (mlTop++, mlLeft + 1, (dsel==(masterindex - 4 + 1))?0x8f:0x0f, mlWidth - 2, " %.*S", mlWidth - 3, global_timidity_path[masterindex - 4]);
+			API->console->DisplayPrintf (mlTop++, mlLeft + 1, (dsel==(masterindex - 4 + 1))?0x8f:0x0f, mlWidth - 2, " %.*S", mlWidth - 3, global_timidity_path[masterindex - 4]);
 			continue;
 		}
 		if (masterindex == (global_timidity_count + 5))
 		{
-			displaystr  (mlTop++, mlLeft + 1,           0x03, "Global SF2 files:", mlWidth - 2);
+			API->console->Driver->DisplayStr  (mlTop++, mlLeft + 1,           0x03, "Global SF2 files:", mlWidth - 2);
 			continue;
 		}
 		if ((masterindex == (global_timidity_count + 6)) && (!sf2_files_count))
 		{
-			displaystr  (mlTop++, mlLeft + 1,           0x0c, " No soundfonts found", mlWidth - 2);
+			API->console->Driver->DisplayStr  (mlTop++, mlLeft + 1,           0x0c, " No soundfonts found", mlWidth - 2);
 			continue;
 		}
 		if ((masterindex >= (global_timidity_count + 6)) && (masterindex < (global_timidity_count + sf2_files_count + 6)))
 		{
-			display_nprintf (mlTop++, mlLeft + 1, (dsel==(masterindex - 6 + 1))?0x8f:0x0f, mlWidth - 2, " %.*S", mlWidth - 3, sf2_files_path[masterindex - global_timidity_count - 6]);
+			API->console->DisplayPrintf (mlTop++, mlLeft + 1, (dsel==(masterindex - 6 + 1))?0x8f:0x0f, mlWidth - 2, " %.*S", mlWidth - 3, sf2_files_path[masterindex - global_timidity_count - 6]);
 			continue;
 		}
-		displayvoid (mlTop++, mlLeft + 1, mlWidth - 2);
+		API->console->Driver->DisplayVoid (mlTop++, mlLeft + 1, mlWidth - 2);
 	}
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xc0%*C\xc4\xd9", mlWidth - 2);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xc0%*C\xc4\xd9", mlWidth - 2);
 }
 
-static void ConfigDrawItems (const int lineno, int xpos, const int width, const char **list, const int listlength, const int selected, const int active)
+static void ConfigDrawItems (const int lineno, int xpos, const int width, const char **list, const int listlength, const int selected, const int active, const struct DevInterfaceAPI_t *API)
 {
 	int i;
 	int origxpos = xpos;
@@ -427,16 +442,16 @@ static void ConfigDrawItems (const int lineno, int xpos, const int width, const
 		int l = strlen (list[i]);
 		if (selected == i)
 		{
-			display_nprintf (lineno, xpos, (active)?0x09:0x01, l + 2, "[%.*o%s%.*o]", (active)?0x0f:0x07, list[i], (active)?0x09:0x01);
+			API->console->DisplayPrintf (lineno, xpos, (active)?0x09:0x01, l + 2, "[%.*o%s%.*o]", (active)?0x0f:0x07, list[i], (active)?0x09:0x01);
 		} else {
-			display_nprintf (lineno, xpos, 0x00, l + 2, " %.*o%s%.0o ", (active)?0x07:0x08, list[i]);
+			API->console->DisplayPrintf (lineno, xpos, 0x00, l + 2, " %.*o%s%.0o ", (active)?0x07:0x08, list[i]);
 		}
 		xpos += l + 2;
 	}
-	displaychr (lineno, xpos, 0x07, ' ', width - xpos + origxpos);
+	API->console->Driver->DisplayChr (lineno, xpos, 0x07, ' ', width - xpos + origxpos);
 }
 
-static void ConfigDrawBar (const int lineno, int xpos, int width, int level, int maxlevel, const int active)
+static void ConfigDrawBar (const int lineno, int xpos, int width, int level, int maxlevel, const int active, const struct DevInterfaceAPI_t *API)
 {
 	char temp[7];
 	int tw = width - 8;
@@ -468,15 +483,15 @@ static void ConfigDrawBar (const int lineno, int xpos, int width, int level, int
 		p3 -= p2;
 		p2 -= p1;
 	}
-	displaystr (lineno, xpos,                         (active)?0x07:0x08, "[", 1);
-	displaychr (lineno, xpos + 1,                     (active)?0x01:0x08, '\xfe', p1);
-	displaychr (lineno, xpos + 1 + p1,                (active)?0x09:0x08, '\xfe', p2);
-	displaychr (lineno, xpos + 1 + p1 + p2,           (active)?0x0b:0x08, '\xfe', p3);
-	displaychr (lineno, xpos + 1 + p1 + p2 + p3,      (active)?0x0f:0x08, '\xfe', p4);
-	displaychr (lineno, xpos + 1 + p1 + p2 + p3 + p4, (active)?0x07:0x08, '\xfa', tw - p1 - p2 - p3 - p4);
+	API->console->Driver->DisplayStr (lineno, xpos,                         (active)?0x07:0x08, "[", 1);
+	API->console->Driver->DisplayChr (lineno, xpos + 1,                     (active)?0x01:0x08, '\xfe', p1);
+	API->console->Driver->DisplayChr (lineno, xpos + 1 + p1,                (active)?0x09:0x08, '\xfe', p2);
+	API->console->Driver->DisplayChr (lineno, xpos + 1 + p1 + p2,           (active)?0x0b:0x08, '\xfe', p3);
+	API->console->Driver->DisplayChr (lineno, xpos + 1 + p1 + p2 + p3,      (active)?0x0f:0x08, '\xfe', p4);
+	API->console->Driver->DisplayChr (lineno, xpos + 1 + p1 + p2 + p3 + p4, (active)?0x07:0x08, '\xfa', tw - p1 - p2 - p3 - p4);
 
 	snprintf (temp, sizeof (temp), "]%5d", level);
-	displaystr (lineno, xpos + width - 8, (active)?0x07:0x08, temp, 8);
+	API->console->Driver->DisplayStr (lineno, xpos + width - 8, (active)?0x07:0x08, temp, 8);
 }
 
 static int DefaultReverbMode;
@@ -487,20 +502,20 @@ static int DefaultPredelayFactor;
 static int DefaultDelayMode;
 static int DefaultDelay;
 static int DefaultChorus;
-static void timidityConfigDraw (int EditPos)
+static void timidityConfigDraw (int EditPos, const struct DevInterfaceAPI_t *API)
 {
 	int large;
 	int mlWidth, mlHeight, mlTop, mlLeft;
-	const char *configfile = cfGetProfileString ("timidity", "configfile", "");
+	const char *configfile = API->configAPI->GetProfileString ("timidity", "configfile", "");
 	const char *reverbs[] = {"disable", "original", "global-original", "freeverb", "global-freeverb"};
 	const char *effect_lr_modes[] = {"disable", "left", "right", "both"};
 	const char *disable_enable[] = {"disable", "enable"};
 
-	if (plScrHeight >= 43)
+	if (API->console->TextHeight >= 43)
 	{
 		large = 2;
 		mlHeight = 41;
-	} else if (plScrHeight >= 35)
+	} else if (API->console->TextHeight >= 35)
 	{
 		large = 1;
 		mlHeight = 33;
@@ -510,132 +525,132 @@ static void timidityConfigDraw (int EditPos)
 	}
 
 	mlWidth = 70;
-	mlTop = (plScrHeight - mlHeight) / 2;
-	mlLeft = (plScrWidth - mlWidth) / 2;
+	mlTop = (API->console->TextHeight - mlHeight) / 2;
+	mlLeft = (API->console->TextWidth - mlWidth) / 2;
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xda%*C\xc4\xbf", mlWidth - 2);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xda%*C\xc4\xbf", mlWidth - 2);
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o Navigate with arrows and hit %.15o<ESC>%.7o to save and exit.%*C %.9o\xb3", mlWidth - 55);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o Navigate with arrows and hit %.15o<ESC>%.7o to save and exit.%*C %.9o\xb3", mlWidth - 55);
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 1. TiMdity+ Configfile/Soundfont: %.15o<ENTER>%.7o to change%*C %.9o\xb3", mlWidth - 53 - 1);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 1. TiMdity+ Configfile/Soundfont: %.15o<ENTER>%.7o to change%*C %.9o\xb3", mlWidth - 53 - 1);
 
 	if (!configfile[0])
 	{
-		display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.3o    (Global default)%*C %.9o\xb3", mlWidth - 21 - 1);
-		display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.3o    %*.7oSelect another file%0.7o%*C %.9o\xb3", (EditPos==0)?8:0, mlWidth - 23 - 2);
+		API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.3o    (Global default)%*C %.9o\xb3", mlWidth - 21 - 1);
+		API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.3o    %*.7oSelect another file%0.7o%*C %.9o\xb3", (EditPos==0)?8:0, mlWidth - 23 - 2);
 	} else {
 		if ((strlen(configfile) > 4) && !strcmp (configfile + strlen (configfile) - 4, ".sf2"))
 		{
-			display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.3o    (SF2 sound font)%*C %.9o\xb3", mlWidth - 21 - 1);
+			API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.3o    (SF2 sound font)%*C %.9o\xb3", mlWidth - 21 - 1);
 		} else {
-			display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.3o    (Specific config file)%*C %0.9o\xb3", mlWidth - 27 - 1);
+			API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.3o    (Specific config file)%*C %0.9o\xb3", mlWidth - 27 - 1);
 		}
-		display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o    %*o%*S%0.9o\xb3", (EditPos==0)?8:0, mlWidth - 6, configfile);
+		API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o    %*o%*S%0.9o\xb3", (EditPos==0)?8:0, mlWidth - 6, configfile);
 	}
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 2. Default Reverb Mode:%*C %.9o\xb3", mlWidth - 25 - 1);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 2. Default Reverb Mode:%*C %.9o\xb3", mlWidth - 25 - 1);
 
-	if (large >= 2) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large >= 2) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop, mlLeft, 0x09, mlWidth, "\xb3%.7o   ");
-	ConfigDrawItems (mlTop, mlLeft + 4, mlWidth - 4 - 1, reverbs, 5, DefaultReverbMode, EditPos==1);
-	displaychr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
+	API->console->DisplayPrintf (mlTop, mlLeft, 0x09, mlWidth, "\xb3%.7o   ");
+	ConfigDrawItems (mlTop, mlLeft + 4, mlWidth - 4 - 1, reverbs, 5, DefaultReverbMode, EditPos==1, API);
+	API->console->Driver->DisplayChr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 3. Default Reverb Level:%*C %.9o\xb3", mlWidth - 26 - 1);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 3. Default Reverb Level:%*C %.9o\xb3", mlWidth - 26 - 1);
 
-	if (large >= 2) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large >= 2) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop, mlLeft, 0x09, 5, "\xb3%.7o    ");
-	ConfigDrawBar (mlTop, mlLeft + 5, mlWidth - 5 - 1, DefaultReverbLevel, 127, EditPos==2);
-	displaychr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
+	API->console->DisplayPrintf (mlTop, mlLeft, 0x09, 5, "\xb3%.7o    ");
+	ConfigDrawBar (mlTop, mlLeft + 5, mlWidth - 5 - 1, DefaultReverbLevel, 127, EditPos==2, API);
+	API->console->Driver->DisplayChr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 4. Default Scale Room:%*C %.9o\xb3", mlWidth - 24 - 1);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 4. Default Scale Room:%*C %.9o\xb3", mlWidth - 24 - 1);
 
-	if (large >= 2) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large >= 2) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop, mlLeft, 0x09, 5, "\xb3%.7o    ");
-	ConfigDrawBar (mlTop, mlLeft + 5, mlWidth - 5 - 1, DefaultScaleRoom, 1000, EditPos==3);
-	displaychr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
+	API->console->DisplayPrintf (mlTop, mlLeft, 0x09, 5, "\xb3%.7o    ");
+	ConfigDrawBar (mlTop, mlLeft + 5, mlWidth - 5 - 1, DefaultScaleRoom, 1000, EditPos==3, API);
+	API->console->Driver->DisplayChr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 5. Default Offset Room:%*C %.9o\xb3", mlWidth - 25 - 1);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 5. Default Offset Room:%*C %.9o\xb3", mlWidth - 25 - 1);
 
-	if (large >= 2) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large >= 2) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop, mlLeft, 0x09, 5, "\xb3%.7o    ");
-	ConfigDrawBar (mlTop, mlLeft + 5, mlWidth - 5 - 1, DefaultOffsetRoom, 1000, EditPos==4);
-	displaychr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
+	API->console->DisplayPrintf (mlTop, mlLeft, 0x09, 5, "\xb3%.7o    ");
+	ConfigDrawBar (mlTop, mlLeft + 5, mlWidth - 5 - 1, DefaultOffsetRoom, 1000, EditPos==4, API);
+	API->console->Driver->DisplayChr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 6. Default Predelay Factor:%*C %.9o\xb3", mlWidth - 29 - 1);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 6. Default Predelay Factor:%*C %.9o\xb3", mlWidth - 29 - 1);
 
-	if (large >= 2) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large >= 2) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop, mlLeft, 0x09, 5, "\xb3%.7o    ");
-	ConfigDrawBar (mlTop, mlLeft + 5, mlWidth - 5 - 1, DefaultPredelayFactor, 1000, EditPos==5);
-	displaychr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
+	API->console->DisplayPrintf (mlTop, mlLeft, 0x09, 5, "\xb3%.7o    ");
+	ConfigDrawBar (mlTop, mlLeft + 5, mlWidth - 5 - 1, DefaultPredelayFactor, 1000, EditPos==5, API);
+	API->console->Driver->DisplayChr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 7. Default Delay Mode:%*C %.9o\xb3", mlWidth - 24 - 1);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 7. Default Delay Mode:%*C %.9o\xb3", mlWidth - 24 - 1);
 
-	if (large >= 2) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large >= 2) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop, mlLeft, 0x09, mlWidth, "\xb3%.7o   ");
-	ConfigDrawItems (mlTop, mlLeft + 4, mlWidth - 4 - 1, effect_lr_modes, 4, DefaultDelayMode, EditPos==6);
-	displaychr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
+	API->console->DisplayPrintf (mlTop, mlLeft, 0x09, mlWidth, "\xb3%.7o   ");
+	ConfigDrawItems (mlTop, mlLeft + 4, mlWidth - 4 - 1, effect_lr_modes, 4, DefaultDelayMode, EditPos==6, API);
+	API->console->Driver->DisplayChr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 8. Default Delay (ms):%*C %.9o\xb3", mlWidth - 24 - 1);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 8. Default Delay (ms):%*C %.9o\xb3", mlWidth - 24 - 1);
 
-	if (large >= 2) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large >= 2) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop, mlLeft, 0x09, 5, "\xb3%.7o    ");
-	ConfigDrawBar (mlTop, mlLeft + 5, mlWidth - 5 - 1, DefaultDelay, 1000, EditPos==7);
-	displaychr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
+	API->console->DisplayPrintf (mlTop, mlLeft, 0x09, 5, "\xb3%.7o    ");
+	ConfigDrawBar (mlTop, mlLeft + 5, mlWidth - 5 - 1, DefaultDelay, 1000, EditPos==7, API);
+	API->console->Driver->DisplayChr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 9. Default Chorus:%*C %.9o\xb3", mlWidth - 20 - 1);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%.7o 9. Default Chorus:%*C %.9o\xb3", mlWidth - 20 - 1);
 
-	if (large >= 2) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large >= 2) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop, mlLeft, 0x09, mlWidth, "\xb3%.7o   ");
-	ConfigDrawItems (mlTop, mlLeft + 4, mlWidth - 4 - 1, disable_enable, 2, DefaultChorus, EditPos==8);
-	displaychr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
+	API->console->DisplayPrintf (mlTop, mlLeft, 0x09, mlWidth, "\xb3%.7o   ");
+	ConfigDrawItems (mlTop, mlLeft + 4, mlWidth - 4 - 1, disable_enable, 2, DefaultChorus, EditPos==8, API);
+	API->console->Driver->DisplayChr (mlTop++, mlLeft + mlWidth - 1, 0x09, '\xb3', 1);
 
-	if (large) display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
+	if (large) API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xb3%*C \xb3", mlWidth - 2);
 
-	display_nprintf (mlTop++, mlLeft, 0x09, mlWidth, "\xc0%*C\xc4\xd9", mlWidth - 2);
+	API->console->DisplayPrintf (mlTop++, mlLeft, 0x09, mlWidth, "\xc0%*C\xc4\xd9", mlWidth - 2);
 }
 
-static interfaceReturnEnum timidityConfigRun (void)
+static void timidityConfigRun (void **token, const struct DevInterfaceAPI_t *API)
 {
 	int esel = 0;
 
 	refresh_configfiles ();
 
-	DefaultReverbMode     = cfGetProfileInt ("timidity", "reverbmode",       2, 10);
-	DefaultReverbLevel    = cfGetProfileInt ("timidity", "reverblevel",     40, 10);
-	DefaultScaleRoom      = cfGetProfileInt ("timidity", "scaleroom",       28, 10);
-	DefaultOffsetRoom     = cfGetProfileInt ("timidity", "offsetroom",      70, 10);
-	DefaultPredelayFactor = cfGetProfileInt ("timidity", "predelayfactor", 100, 10);
-	DefaultDelayMode      = cfGetProfileInt ("timidity", "delaymode",       -1, 10) + 1;
-	DefaultDelay          = cfGetProfileInt ("timidity", "delay",           25, 10);
-	DefaultChorus         = cfGetProfileInt ("timidity", "chorusenabled",    1, 10);
+	DefaultReverbMode     = API->configAPI->GetProfileInt ("timidity", "reverbmode",       2, 10);
+	DefaultReverbLevel    = API->configAPI->GetProfileInt ("timidity", "reverblevel",     40, 10);
+	DefaultScaleRoom      = API->configAPI->GetProfileInt ("timidity", "scaleroom",       28, 10);
+	DefaultOffsetRoom     = API->configAPI->GetProfileInt ("timidity", "offsetroom",      70, 10);
+	DefaultPredelayFactor = API->configAPI->GetProfileInt ("timidity", "predelayfactor", 100, 10);
+	DefaultDelayMode      = API->configAPI->GetProfileInt ("timidity", "delaymode",       -1, 10) + 1;
+	DefaultDelay          = API->configAPI->GetProfileInt ("timidity", "delay",           25, 10);
+	DefaultChorus         = API->configAPI->GetProfileInt ("timidity", "chorusenabled",    1, 10);
 	if (DefaultReverbMode     <    0) DefaultReverbMode     =    0;
 	if (DefaultReverbLevel    <    0) DefaultReverbLevel    =    0;
 	if (DefaultScaleRoom      <    0) DefaultScaleRoom      =    0;
@@ -655,11 +670,11 @@ static interfaceReturnEnum timidityConfigRun (void)
 
 	while (1)
 	{
-		fsDraw();
-		timidityConfigDraw (esel);
-		while (ekbhit())
+		API->fsDraw();
+		timidityConfigDraw (esel, API);
+		while (API->console->KeyboardHit())
 		{
-			int key = egetch();
+			int key = API->console->KeyboardGetChar();
 			static uint32_t lastpress = 0;
 			static int repeat;
 			if ((key != KEY_LEFT) && (key != KEY_RIGHT))
@@ -667,8 +682,8 @@ static interfaceReturnEnum timidityConfigRun (void)
 				lastpress = 0;
 				repeat = 1;
 			} else {
-				uint32_t newpress = dos_clock();
-				if ((newpress-lastpress) > 0x4000) /* 250 ms */
+				uint32_t newpress = clock_ms();
+				if ((newpress-lastpress) > 250) /* 250 ms */
 				{
 					repeat = 1;
 				} else {
@@ -688,7 +703,7 @@ static interfaceReturnEnum timidityConfigRun (void)
 						int dsel = 0;
 						int inner = 1;
 						{ /* preselect dsel */
-							const char *configfile = cfGetProfileString ("timidity", "configfile", "");
+							const char *configfile = API->configAPI->GetProfileString ("timidity", "configfile", "");
 							int i;
 							if (configfile[0])
 							{
@@ -715,11 +730,11 @@ static interfaceReturnEnum timidityConfigRun (void)
 
 						while (inner)
 						{
-							fsDraw();
-							timidityConfigFileSelectDraw (dsel);
-							while (inner && ekbhit())
+							API->fsDraw();
+							timidityConfigFileSelectDraw (dsel, API);
+							while (inner && API->console->KeyboardHit())
 							{
-								int key = egetch();
+								int key = API->console->KeyboardGetChar();
 								switch (key)
 								{
 									case KEY_DOWN:
@@ -734,24 +749,25 @@ static interfaceReturnEnum timidityConfigRun (void)
 											dsel--;
 										}
 										break;
+									case KEY_EXIT:
 									case KEY_ESC:
 										inner = 0;
 										break;
 									case _KEY_ENTER:
 										if (!dsel)
 										{
-											cfSetProfileString ("timidity", "configfile", "");
+											API->configAPI->SetProfileString ("timidity", "configfile", "");
 										} else if (dsel < (global_timidity_count + 1))
 										{
-											cfSetProfileString ("timidity", "configfile", global_timidity_path[dsel - 1]);
+											API->configAPI->SetProfileString ("timidity", "configfile", global_timidity_path[dsel - 1]);
 										} else {
-											cfSetProfileString ("timidity", "configfile", sf2_files_path[dsel - 1 - global_timidity_count]);
+											API->configAPI->SetProfileString ("timidity", "configfile", sf2_files_path[dsel - 1 - global_timidity_count]);
 										}
 										inner = 0;
 										break;
 								}
 							}
-							framelock ();
+							API->console->FrameLock ();
 						}
 					}
 					break;
@@ -902,68 +918,58 @@ static interfaceReturnEnum timidityConfigRun (void)
 						esel--;
 					}
 					break;
+				case KEY_EXIT:
 				case KEY_ESC:
-
-					cfStoreConfig();
+					API->configAPI->StoreConfig();
 					goto superexit;
 					break;
 			}
 		}
-		framelock ();
+		API->console->FrameLock ();
 	}
 
 superexit:
 	reset_configfiles ();
 
-	cfSetProfileInt ("timidity", "reverbmode",     DefaultReverbMode,     10);
-	cfSetProfileInt ("timidity", "reverblevel",    DefaultReverbLevel,    10);
-	cfSetProfileInt ("timidity", "scaleroom",      DefaultScaleRoom,      10);
-	cfSetProfileInt ("timidity", "offsetroom",     DefaultOffsetRoom,     10);
-	cfSetProfileInt ("timidity", "predelayfactor", DefaultPredelayFactor, 10);
-	cfSetProfileInt ("timidity", "delaymode",      DefaultDelayMode - 1,  10);
-	cfSetProfileInt ("timidity", "delay",          DefaultDelay,          10);
-	cfSetProfileInt ("timidity", "chorusenabled",  DefaultChorus,         10);
-	cfStoreConfig ();
-
-	return interfaceReturnNextAuto;
+	API->configAPI->SetProfileInt ("timidity", "reverbmode",     DefaultReverbMode,     10);
+	API->configAPI->SetProfileInt ("timidity", "reverblevel",    DefaultReverbLevel,    10);
+	API->configAPI->SetProfileInt ("timidity", "scaleroom",      DefaultScaleRoom,      10);
+	API->configAPI->SetProfileInt ("timidity", "offsetroom",     DefaultOffsetRoom,     10);
+	API->configAPI->SetProfileInt ("timidity", "predelayfactor", DefaultPredelayFactor, 10);
+	API->configAPI->SetProfileInt ("timidity", "delaymode",      DefaultDelayMode - 1,  10);
+	API->configAPI->SetProfileInt ("timidity", "delay",          DefaultDelay,          10);
+	API->configAPI->SetProfileInt ("timidity", "chorusenabled",  DefaultChorus,         10);
+	API->configAPI->StoreConfig ();
 }
 
-static struct ocpfile_t      *timidityconfig; // needs to overlay an dialog above filebrowser, and after that the file is "finished"   Special case of DEVv
+static struct ocpfile_t *timidityconfig; // needs to overlay an dialog above filebrowser, and after that the file is "finished"   Special case of DEVv
+static void timidityConfigRun (void **token, const struct DevInterfaceAPI_t *API);
 
-static int                    timidityConfigInit (struct moduleinfostruct *info, struct ocpfilehandle_t *f, const struct interfaceparameters *ip);
-static interfaceReturnEnum    timidityConfigRun  (void);
-static struct interfacestruct timidityConfigIntr = {timidityConfigInit, timidityConfigRun, 0, "TiMidity+ Config" INTERFACESTRUCT_TAIL};
-
-static int timidity_config_init (void)
+OCP_INTERNAL int timidity_config_init (struct PluginInitAPI_t *API)
 {
-	struct moduleinfostruct m;
-	uint32_t mdbref;
+	timidityconfig = API->dev_file_create (
+		API->dmSetup->basedir,
+		"timidityconfig.dev",
+		"TiMidity+ Configuration",
+		"",
+		0, /* token */
+		0, /* Init */
+		timidityConfigRun,
+		0, /* Close */
+		0  /* Destructor */
+	);
 
-	timidityconfig= mem_file_open (dmSetup->basedir, dirdbFindAndRef (dmSetup->basedir->dirdb_ref, "timidityconfig.dev", dirdb_use_file), strdup (timidityConfigIntr.name), strlen (timidityConfigIntr.name));
-	dirdbUnref (timidityconfig->dirdb_ref, dirdb_use_file);
-	mdbref = mdbGetModuleReference2 (timidityconfig->dirdb_ref, strlen (timidityConfigIntr.name));
-	mdbGetModuleInfo (&m, mdbref);
-	m.modtype.integer.i = MODULETYPE("DEVv");
-	strcpy (m.title, "TiMidity+ Configuration");
-	mdbWriteModuleInfo (mdbref, &m);
-	filesystem_setup_register_file (timidityconfig);
-	plRegisterInterface (&timidityConfigIntr);
+	API->filesystem_setup_register_file (timidityconfig);
 
 	return errOk;
 }
 
-static void timidity_config_done (void)
+OCP_INTERNAL void timidity_config_done (struct PluginCloseAPI_t *API)
 {
-	plUnregisterInterface (&timidityConfigIntr);
 	if (timidityconfig)
 	{
-		filesystem_setup_unregister_file (timidityconfig);
+		API->filesystem_setup_unregister_file (timidityconfig);
+		timidityconfig->unref (timidityconfig);
 		timidityconfig = 0;
 	}
 }
-
-#ifndef SUPPORT_STATIC_PLUGINS
-char *dllinfo = "";
-#endif
-
-DLLEXTINFO_PREFIX struct linkinfostruct dllextinfo = {.name = "timidityconfig", .desc = "OpenCP UNIX TiMidity+ configuration (c) 2022 Stian Skjelstad", .ver = DLLVERSION, .size = 0, .Init = timidity_config_init, .Close = timidity_config_done};
