@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * XMPlay - Module player for XM/MOD and affiliate formats
  *
@@ -272,11 +272,11 @@ static int panrange(int x)
 }
 
 
-static void ReadQue()
+static void ReadQue (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int type,val1,val2,t;
 	int i;
-	int time=mcpGet(-1, mcpGTimer);
+	int time = cpifaceSession->mcpGet (cpifaceSession, -1, mcpGTimer);
 	while (1)
 	{
 		if (querpos==quewpos)
@@ -353,7 +353,7 @@ static void putque(int type, int val1, int val2)
 }
 
 
-static void PlayNote(struct channel *ch)
+static void PlayNote(struct cpifaceSessionAPI_t *cpifaceSession, struct channel *ch)
 {
 	int portatmp=0;
 	int delaytmp;
@@ -446,7 +446,7 @@ static void PlayNote(struct channel *ch)
 
 			frq=48*256-(((procnot-1)<<8)-ch->chCurNormNote);
 			if (!linearfreq)
-				frq=mcpGetFreq6848(frq);
+				frq=cpifaceSession->mcpAPI->GetFreq6848(frq);
 			ch->chPitch=frq;
 			ch->chFinalPitch=frq;
 			ch->chPortaToPitch=frq;
@@ -471,7 +471,7 @@ static void PlayNote(struct channel *ch)
 		} else {
 			int32_t frq=48*256-(((procnot-1)<<8)-ch->chCurNormNote);
 			if (!linearfreq)
-				frq=mcpGetFreq6848(frq);
+				frq=cpifaceSession->mcpAPI->GetFreq6848(frq);
 			ch->chPortaToPitch=frq;
 		}
 	}
@@ -505,20 +505,26 @@ static void PlayNote(struct channel *ch)
 
 static uint16_t notetab[16]={32768,30929,29193,27554,26008,24548,23170,21870,20643,19484,18390,17358,16384,15464,14596,13777};
 
-static void xmpPlayTick(void)
+static void xmpPlayTick (struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int i;
 	struct xmpsample *sm;
 	int vol, pan;
 
+	if ((!looping) && looped)
+	{
+		cpifaceSession->mcpSet (cpifaceSession, -1, mcpMasterPause, 1);
+		return;
+	}
+
 	if (firstspeed)
 	{
-		mcpSet(-1, mcpGSpeed, firstspeed);
+		cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, firstspeed);
 		firstspeed=0;
 	}
 
-	cmdtime=mcpGet(-1, mcpGCmdTimer);
-	ReadQue();
+	cmdtime = cpifaceSession->mcpGet (cpifaceSession, -1, mcpGCmdTimer);
+	ReadQue (cpifaceSession);
 
 	tick0=0;
 	for (i=0; i<nchan; i++)
@@ -632,6 +638,10 @@ static void xmpPlayTick(void)
 			procvol=patptr[nchan*currow+i][2];
 			proccmd=patptr[nchan*currow+i][3];
 			procdat=patptr[nchan*currow+i][4];
+			if (procnot)
+			{
+				ch->chDelayNote = procnot;
+			}
 
 			if (!patdelay)
 			{
@@ -643,7 +653,7 @@ static void xmpPlayTick(void)
 					ch->chCurIns=procins;
 				}
 				if (procins<=ninst)
-					PlayNote(ch);
+					PlayNote(cpifaceSession, ch);
 			}
 
 			ch->chVCommand=procvol>>4;
@@ -785,7 +795,7 @@ static void xmpPlayTick(void)
 					if (procdat>=0x20)
 					{
 						curbpm=procdat;
-						mcpSet(-1, mcpGSpeed, 256*2*curbpm/5);
+						cpifaceSession->mcpSet (cpifaceSession, -1, mcpGSpeed, 256*2*curbpm/5);
 						putque(queTempo, -1, curbpm);
 					} else {
 						curtempo=procdat;
@@ -838,8 +848,9 @@ static void xmpPlayTick(void)
 						if (ch->cursamp->panenv<nenv)
 							if (ch->chPanEnvPos>envelopes[ch->cursamp->panenv].len)
 								ch->chPanEnvPos=envelopes[ch->cursamp->panenv].len;
-					} else
+					} else {
 						fprintf(stderr, __FILE__ " CmdEnvPos ch->cursamp not set\n");
+					}
 					break;
 				case xmpCmdPanSlide:
 					if (procdat)
@@ -948,8 +959,6 @@ static void xmpPlayTick(void)
 						patdelay=procdat+1;
 					break;
 				case xmpCmdDelayNote:
-					if (procnot)
-						ch->chDelayNote=procnot;
 					ch->fx=xfxDelay;
 					ch->notefx=xfxNXDelay;
 					ch->chDelayIns=procins;
@@ -1028,7 +1037,7 @@ static void xmpPlayTick(void)
 					{
 						ch->chFinalPitch=((ch->chPitch+ch->chCurNormNote+0x80)&~0xFF)-ch->chCurNormNote;
 					} else {
-						ch->chFinalPitch=mcpGetFreq6848(((mcpGetNote6848(ch->chPitch)+ch->chCurNormNote+0x80)&~0xFF)-ch->chCurNormNote);
+						ch->chFinalPitch=cpifaceSession->mcpAPI->GetFreq6848(((cpifaceSession->mcpAPI->GetNote6848(ch->chPitch)+ch->chCurNormNote+0x80)&~0xFF)-ch->chCurNormNote);
 					}
 				} else
 					ch->chFinalPitch=ch->chPitch;
@@ -1075,7 +1084,7 @@ static void xmpPlayTick(void)
 					if (linearfreq)
 						ch->chFinalPitch=((ch->chPitch+ch->chCurNormNote+0x80)&~0xFF)-ch->chCurNormNote;
 					else
-						ch->chFinalPitch=mcpGetFreq6848(((mcpGetNote6848(ch->chPitch)+ch->chCurNormNote+0x80)&~0xFF)-ch->chCurNormNote);
+						ch->chFinalPitch=cpifaceSession->mcpAPI->GetFreq6848(((cpifaceSession->mcpAPI->GetNote6848(ch->chPitch)+ch->chCurNormNote+0x80)&~0xFF)-ch->chCurNormNote);
 				} else
 					ch->chFinalPitch=ch->chPitch;
 				break;
@@ -1114,7 +1123,7 @@ static void xmpPlayTick(void)
 					if (linearfreq)
 						ch->chFinalPitch=((ch->chPitch+ch->chCurNormNote+0x80)&~0xFF)-ch->chCurNormNote;
 					else
-						ch->chFinalPitch=mcpGetFreq6848(((mcpGetNote6848(ch->chPitch)+ch->chCurNormNote+0x80)&~0xFF)-ch->chCurNormNote);
+						ch->chFinalPitch=cpifaceSession->mcpAPI->GetFreq6848(((cpifaceSession->mcpAPI->GetNote6848(ch->chPitch)+ch->chCurNormNote+0x80)&~0xFF)-ch->chCurNormNote);
 				} else
 					ch->chFinalPitch=ch->chPitch;
 
@@ -1248,7 +1257,7 @@ static void xmpPlayTick(void)
 				proccmd=0;
 				procdat=0;
 				procvol=0;
-				PlayNote(ch);
+				PlayNote(cpifaceSession, ch);
 				switch (ch->chDelayVol>>4)
 				{
 					case xmpVCmdVol0x: case xmpVCmdVol1x: case xmpVCmdVol2x: case xmpVCmdVol3x:
@@ -1266,7 +1275,7 @@ static void xmpPlayTick(void)
 
 		if (!ch->cursamp)
 		{
-			mcpSet(i, mcpCStatus, 0);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCStatus, 0);
 			continue;
 		}
 
@@ -1347,55 +1356,55 @@ static void xmpPlayTick(void)
 		}
 
 		if (ch->nextstop)
-			mcpSet(i, mcpCStatus, 0);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCStatus, 0);
 		if (ch->nextsamp!=(unsigned)-1)
-			mcpSet(i, mcpCInstrument, ch->nextsamp);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCInstrument, ch->nextsamp);
 		if (ch->nextpos!=(unsigned)-1)
 		{
-			mcpSet(i, mcpCPosition, ch->nextpos);
-			mcpSet(i, mcpCLoop, 1);
-			mcpSet(i, mcpCDirect, 0);
-			mcpSet(i, mcpCStatus, 1);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCPosition, ch->nextpos);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCLoop, 1);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCDirect, 0);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCStatus, 1);
 		}
 		if (linearfreq)
-			mcpSet(i, mcpCPitch, -ch->chFinalPitch);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCPitch, -ch->chFinalPitch);
 		else
-			mcpSet(i, mcpCPitch6848, ch->chFinalPitch);
-		mcpSet(i, mcpCVolume, (looping||!looped)?vol:0);
-		mcpSet(i, mcpCPanning, pan);
-		mcpSet(i, mcpCMute, mutech[i]);
+			cpifaceSession->mcpSet (cpifaceSession, i, mcpCPitch6848, ch->chFinalPitch);
+		cpifaceSession->mcpSet (cpifaceSession, i, mcpCVolume, (looping||!looped)?vol:0);
+		cpifaceSession->mcpSet (cpifaceSession, i, mcpCPanning, pan);
+		cpifaceSession->mcpSet (cpifaceSession, i, mcpCMute, mutech[i]);
 	}
 	putque(quePos, -1, curtick|(curord<<16)|(currow<<8));
 }
 
-int __attribute__ ((visibility ("internal"))) xmpGetRealPos(void)
+OCP_INTERNAL int xmpGetRealPos (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	ReadQue();
+	ReadQue (cpifaceSession);
 	return realpos;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpChanActive(int ch)
+OCP_INTERNAL int xmpChanActive (struct cpifaceSessionAPI_t *cpifaceSession, int ch)
 {
-	return mcpGet(ch, mcpCStatus)&&channels[ch].cursamp&&channels[ch].chVol&&channels[ch].chFadeVol;
+	return cpifaceSession->mcpGet (cpifaceSession, ch, mcpCStatus)&&channels[ch].cursamp&&channels[ch].chVol&&channels[ch].chFadeVol;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpGetChanIns(int ch)
+OCP_INTERNAL int xmpGetChanIns (int ch)
 {
 	return channels[ch].chCurIns;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpGetChanSamp(int ch)
+OCP_INTERNAL int xmpGetChanSamp (int ch)
 {
 	if (!channels[ch].cursamp)
 		return 0xFFFF;
 	return channels[ch].cursamp-samples;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpGetDotsData(int ch, int *smp, int *frq, int *voll, int *volr, int *sus)
+OCP_INTERNAL int xmpGetDotsData (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int *smp, int *frq, int *voll, int *volr, int *sus)
 {
 	struct channel *c;
 
-	if (!mcpGet(ch, mcpCStatus))
+	if (!cpifaceSession->mcpGet (cpifaceSession, ch, mcpCStatus))
 		return 0;
 	c=&channels[ch];
 	if (!c->cursamp||!c->chVol||!c->chFadeVol)
@@ -1404,23 +1413,23 @@ int __attribute__ ((visibility ("internal"))) xmpGetDotsData(int ch, int *smp, i
 	if (linearfreq)
 		*frq=60*256+c->cursamp->normnote-freqrange(c->chFinalPitch);
 	else
-		*frq=60*256+c->cursamp->normnote+mcpGetNote8363(6848*8363/freqrange(c->chFinalPitch));
-	mcpGetRealVolume(ch, voll, volr);
+		*frq=60*256+c->cursamp->normnote+cpifaceSession->mcpAPI->GetNote8363(6848*8363/freqrange(c->chFinalPitch));
+	cpifaceSession->mcpGetRealVolume (ch, voll, volr);
 	*sus=c->chSustain;
 	return 1;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpGetRealVolume(int ch, int *voll, int *volr)
+OCP_INTERNAL void xmpGetRealVolume (struct cpifaceSessionAPI_t *cpifaceSession, int ch, int *voll, int *volr)
 {
-	mcpGetRealVolume(ch, voll, volr);
+	cpifaceSession->mcpGetRealVolume (ch, voll, volr);
 }
 
-uint16_t __attribute__ ((visibility ("internal"))) xmpGetPos(void)
+OCP_INTERNAL uint16_t xmpGetPos (void)
 {
 	return (curord<<8)|currow;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpSetPos(int ord, int row)
+OCP_INTERNAL void xmpSetPos (struct cpifaceSessionAPI_t *cpifaceSession, int ord, int row)
 {
 	int i;
 
@@ -1447,7 +1456,7 @@ void __attribute__ ((visibility ("internal"))) xmpSetPos(int ord, int row)
 			row=0;
 	}
 	for (i=0; i<nchan; i++)
-		mcpSet(i, mcpCReset, 0);
+		cpifaceSession->mcpSet (cpifaceSession, i, mcpCReset, 0);
 	jumptoord=ord;
 	jumptorow=row;
 	curtick=curtempo;
@@ -1459,32 +1468,28 @@ void __attribute__ ((visibility ("internal"))) xmpSetPos(int ord, int row)
 	realpos=(curord<<16)|(currow<<8);
 }
 
-int __attribute__ ((visibility ("internal"))) xmpGetLChanSample(unsigned int ch, int16_t *b, unsigned int len, uint32_t rate, int opt)
-{
-	return mcpGetChanSample(ch, b, len, rate, opt);
-}
-
-void __attribute__ ((visibility ("internal"))) xmpMute(int i, int m)
+OCP_INTERNAL void xmpMute (struct cpifaceSessionAPI_t *cpifaceSession, int i, int m)
 {
+	cpifaceSession->MuteChannel[i] = m;
 	mutech[i]=m;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpLoop(void)
+OCP_INTERNAL int xmpLoop (void)
 {
 	return looped;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpSetLoop(int x)
+OCP_INTERNAL void xmpSetLoop (int x)
 {
 	looping=x;
 }
 
-int __attribute__ ((visibility ("internal"))) xmpLoadSamples(struct xmodule *m)
+OCP_INTERNAL int xmpLoadSamples (struct cpifaceSessionAPI_t *cpifaceSession, struct xmodule *m)
 {
-	return mcpLoadSamples(m->sampleinfos, m->nsampi);
+	return cpifaceSession->mcpDevAPI->LoadSamples (cpifaceSession, m->sampleinfos, m->nsampi);
 }
 
-int __attribute__ ((visibility ("internal"))) xmpPlayModule(struct xmodule *m, struct ocpfilehandle_t *file)
+OCP_INTERNAL int xmpPlayModule (struct xmodule *m, struct ocpfilehandle_t *file, struct cpifaceSessionAPI_t *cpifaceSession)
 {
 	int i;
 
@@ -1528,7 +1533,9 @@ int __attribute__ ((visibility ("internal"))) xmpPlayModule(struct xmodule *m, s
 	quelen=100;
 	que=malloc(sizeof(int)*quelen*4);
 	if (!que)
-		return 0;
+	{
+		return errAllocMem;
+	}
 	querpos=0;
 	quewpos=0;
 
@@ -1536,102 +1543,36 @@ int __attribute__ ((visibility ("internal"))) xmpPlayModule(struct xmodule *m, s
 	realtempo=m->inibpm;
 	realspeed=m->initempo;
 	firstspeed=256*2*curbpm/5;
-	if (!mcpOpenPlayer(nchan, xmpPlayTick, file))
-		return 0;
-
-	mcpNormalize (mcpNormalizeDefaultPlayW);
-
-	if (nchan!=mcpNChan)
+	if (!cpifaceSession->mcpDevAPI->OpenPlayer(nchan, xmpPlayTick, file, cpifaceSession))
 	{
-		mcpClosePlayer();
-		return 0;
+		return errPlay;
 	}
 
-	return 1;
+	cpifaceSession->Normalize (cpifaceSession, mcpNormalizeDefaultPlayW);
+
+	if (nchan != cpifaceSession->PhysicalChannelCount)
+	{
+		cpifaceSession->mcpDevAPI->ClosePlayer (cpifaceSession);
+		return errFormStruc;
+	}
+
+	return errOk;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpStopModule(void)
+OCP_INTERNAL void xmpStopModule (struct cpifaceSessionAPI_t *cpifaceSession)
 {
-	mcpClosePlayer();
+	cpifaceSession->mcpDevAPI->ClosePlayer (cpifaceSession);
 	free(que);
 }
 
-void __attribute__ ((visibility ("internal"))) xmpGetGlobInfo(int *tmp, int *bpm, int *gvol)
+OCP_INTERNAL void xmpGetGlobInfo (int *tmp, int *bpm, int *gvol)
 {
 	*tmp=realspeed;
 	*bpm=realtempo;
 	*gvol=realgvol;
 }
 
-/*
-int __attribute__ ((visibility ("internal"))) xmpGetTime(void)
-{
-	return mcpGet(-1, mcpGTimer);
-}*/
-
-/*int __attribute__ ((visibility ("internal"))) xmpGetSync(int ch, int *time)
-{
-	ReadQue();
-	if ((ch<0)||(ch>=nchan))
-	{
-		*time=xmpGetTime()-realsynctime;
-		return realsync;
-	} else {
-		*time=xmpGetTime()-channels[ch].chSyncTime;
-		return channels[ch].chSync;
-	}
-}*/
-
-/*int __attribute__ ((visibility ("internal"))) xmpGetTickTime(void)
-{
-	ReadQue();
-	return 65536*5/(2*realtempo);
-}*/
-
-/*int __attribute__ ((visibility ("internal"))) xmpGetRowTime(void)
-{
-	ReadQue();
-	return 65536*5*realspeed/(2*realtempo);
-}*/
-
-/*void __attribute__ ((visibility ("internal"))) xmpSetEvPos(int ch, int pos, int modtype, int mod)
-{
-	struct channel *c;
-	if ((ch<0)||(ch>=nchan))
-		return;
-	c=&channels[ch];
-	c->evpos0=pos;
-	c->evmodtype=modtype;
-	c->evmod=mod;
-	c->evmodpos=0;
-	c->evpos=-1;
-	c->evtime=-1;
-}*/
-
-/*int __attribute__ ((visibility ("internal"))) xmpGetEvPos(int ch, int *time)
-{
-	ReadQue();
-	if ((ch<0)||(ch>=nchan))
-	{
-		*time=-1;
-		return -1;
-	}
-	*time=xmpGetTime()-channels[ch].evtime;
-	return channels[ch].evpos;
-}*/
-
-/*int __attribute__ ((visibility ("internal"))) xmpFindEvPos(int pos, int *time)
-{
-	int i;
-	ReadQue();
-	for (i=0; i<nchan; i++)
-		if (channels[i].evpos==pos)
-			break;
-	*time=xmpGetTime()-channels[i].evtime;
-	return channels[i].evpos;
-}*/
-
-void __attribute__ ((visibility ("internal"))) xmpGetChanInfo(unsigned char ch, struct xmpchaninfo *ci)
+OCP_INTERNAL void xmpGetChanInfo (unsigned char ch, struct xmpchaninfo *ci)
 {
 	const struct channel *t=&channels[ch];
 	ci->note=t->curnote+11;
@@ -1649,7 +1590,7 @@ void __attribute__ ((visibility ("internal"))) xmpGetChanInfo(unsigned char ch,
 	ci->fx=t->fx;
 }
 
-void __attribute__ ((visibility ("internal"))) xmpGetGlobInfo2(struct xmpglobinfo *gi)
+OCP_INTERNAL void xmpGetGlobInfo2 (struct xmpglobinfo *gi)
 {
 	gi->globvol=globalvol;
 	gi->globvolslide=globalfx;
