@@ -1,6 +1,6 @@
 /* OpenCP Module Player
  * copyright (c) 1994-'10 Niklas Beisert <nbeisert@physik.tu-muenchen.de>
- * copyright (c) 2004-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2004-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Some functions not present in POSIX that is needed
  *
@@ -72,13 +72,6 @@
 #include <Threads.h>
 #endif
 
-time_t dos_clock(void)
-{
-	struct timeval t;
-	gettimeofday(&t, NULL);
-	return t.tv_sec*0x10000+t.tv_usec*1024/15625;
-}
-
 void getext_malloc(const char *src, char **ext)
 {
 	const char *ref1;
@@ -87,14 +80,14 @@ void getext_malloc(const char *src, char **ext)
 	/* clear the target points, for now */
 	if (ext) *ext = 0;
 
-	if ((ref1 = rindex (src, '/')))
+	if ((ref1 = strrchr (src, '/')))
 	{
 		src = ref1+1;
 	}
 
 	len = strlen(src);
 
-	ref1 = rindex (src, '.');
+	ref1 = strrchr (src, '.');
 	if (!ref1)
 	{
 		ref1 = src + len;
@@ -168,7 +161,7 @@ int splitpath4_malloc(const char *src, char **drive, char **path, char **file, c
 		goto error_out;
 	}
 
-	if ((ref1=rindex(src, '/')))
+	if ((ref1=strrchr(src, '/')))
 	{
 		if (path)
 		{
@@ -197,7 +190,7 @@ int splitpath4_malloc(const char *src, char **drive, char **path, char **file, c
 
 	len = strlen(src);
 
-	ref1 = rindex (src, '.');
+	ref1 = strrchr (src, '.');
 	if (!ref1)
 	{
 		ref1 = src + len;
@@ -307,7 +300,7 @@ int splitpath_malloc(const char *src, char **drive, char **path, char **filename
 		goto error_out;
 	}
 
-	if ((ref1=rindex(src, '/')))
+	if ((ref1=strrchr(src, '/')))
 	{
 		if (path)
 		{
@@ -394,8 +387,8 @@ int makepath_malloc(char **dst, const char *drive, const char *path, const char
 			fprintf (stderr, "makepath_malloc(): drive starts with :\n");
 			return -1;
 		}
-		pos1 = index (drive, ':');
-		pos2 = rindex (drive, ':');
+		pos1 = strchr (drive, ':');
+		pos2 = strrchr (drive, ':');
 		if (!pos1)
 		{
 			fprintf (stderr, "makepath_malloc(): drive does not contain:\n");
@@ -435,7 +428,7 @@ int makepath_malloc(char **dst, const char *drive, const char *path, const char
 	if (file)
 	{
 		/* Assertion test */
-		if (index (file, '/'))
+		if (strchr (file, '/'))
 		{
 			fprintf (stderr, "makepath_malloc(): file contains /\n");
 			return -1;
@@ -446,7 +439,7 @@ int makepath_malloc(char **dst, const char *drive, const char *path, const char
 	if (ext)
 	{
 		/* Assertion tests */
-		if (index (ext, '/'))
+		if (strchr (ext, '/'))
 		{
 			fprintf (stderr, "makepath_malloc(): ext contains /\n");
 			return -1;
@@ -496,24 +489,6 @@ int makepath_malloc(char **dst, const char *drive, const char *path, const char
 	return 0;
 }
 
-#ifndef HAVE_STRUPR
-
-char *strupr(char *src)
-{
-	char *retval = src;
-	if (src)
-	{
-		while (*src)
-		{
-			*src=toupper(*src);
-			src++;
-		}
-	}
-	return retval;
-}
-
-#endif
-
 #ifndef HAVE_MEMMEM
 void *memmem(const void *haystack, size_t haystacklen,
              const void *needle, size_t needlelen)
@@ -1369,3 +1344,37 @@ void strreplace (char *dst, char old, char replacement)
 		dst++;
 	}
 }
+
+/* Detect MacOS / OS-X */
+#if defined(__APPLE__) && defined(__MACH__)
+# if !defined(MAC_OS_X_VERSION_10_12) || MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_12
+#  include <mach/clock.h>
+#  include <mach/mach_host.h>
+int clock_gettime(clockid_t clk_id, struct timespec *tv)
+{
+	int retval;
+	static clock_serv_t cclock;
+	static int initialized = 0;
+	mach_timespec_t mts;
+
+	if (!initialized)
+	{
+		host_get_clock_service(mach_host_self(), clk_id, &cclock);
+		initialized = 1;
+	}
+	retval = clock_get_time(cclock, &mts);
+	if (retval)
+	{
+		tv->tv_sec = 0;
+		tv->tv_nsec = 0;
+		return retval;
+	}
+	//mach_port_deallocate(mach_task_self(), cclock);
+
+	tv->tv_sec = mts.tv_sec;
+	tv->tv_nsec = mts.tv_nsec;
+
+	return 0;
+}
+# endif
+#endif
