@@ -1,5 +1,5 @@
 /* OpenCP Module Player
- * copyright (c) 2020-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2020-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * Basic glue for the different console implementations for unix
  *
@@ -44,12 +44,11 @@ static int font_entries_8x16_fill;
 static int font_entries_8x16_allocated;
 
 static TTF_Font *unifont_bmp;
-#ifdef UNIFONT_CSUR_TTF
+#if defined(UNIFONT_CSUR_TTF) || defined(UNIFONT_CSUR_OTF)
 static TTF_Font *unifont_csur;
 #endif
 static TTF_Font *unifont_upper;
 
-
 struct font_entry_8x8_t  cp437_8x8 [256];
 struct font_entry_8x16_t cp437_8x16[256];
 
@@ -144,6 +143,7 @@ static void fontengine_8x8_append (struct font_entry_8x8_t *entry)
 			fprintf (stderr, "fontengine_8x8_append: malloc() failure....\n");
 			return;
 		}
+
 		font_entries_8x8 = newentries;
 		font_entries_8x8_allocated = newallocated;
 	}
@@ -169,6 +169,7 @@ static void fontengine_8x16_append (struct font_entry_8x16_t *entry)
 			fprintf (stderr, "fontengine_8x16_append: malloc() failure....\n");
 			return;
 		}
+
 		font_entries_8x16 = newentries;
 		font_entries_8x16_allocated = newallocated;
 	}
@@ -205,11 +206,199 @@ void fontengine_8x8_iterate (void)
 	}
 }
 
+int fontengine_8x8_forceunifont (uint32_t codepoint, int *width, uint8_t data[16])
+{
+	TTF_Surface *text_surface = 0;
+
+	if (codepoint == 0)
+	{
+		codepoint = ' ';
+	}
+
+	       if (                            (codepoint <= 0x0d8ff)  ||
+	            ((codepoint >= 0x0f900) && (codepoint <= 0x0ffff)) )
+	{
+		text_surface = unifont_bmp ? TTF_RenderGlyph32_Shaded (unifont_bmp, codepoint) : 0;
+#if defined(UNIFONT_CSUR_TTF) || defined(UNIFONT_CSUR_OTF)
+	} else if ( ((codepoint >= 0x0e000) && (codepoint <= 0x0f8ff)) )
+	{
+		text_surface = unifont_csur ? TTF_RenderGlyph32_Shaded (unifont_csur, codepoint) : 0;
+#endif
+	} else if ( ((codepoint >= 0x10000) && (codepoint <= 0x1ffff)) ||
+	            ((codepoint >= 0xe0000) && (codepoint <= 0xeffff)) )
+	{
+		text_surface = unifont_upper ? TTF_RenderGlyph32_Shaded (unifont_upper, codepoint) : 0;
+#if defined(UNIFONT_CSUR_TTF) || defined(UNIFONT_CSUR_OTF)
+	} else if ( ((codepoint >= 0xf0000) && (codepoint >= 0xffffd)) )
+	{
+		text_surface = unifont_csur ? TTF_RenderGlyph32_Shaded (unifont_csur, codepoint) : 0;
+#endif
+	}
+
+	if (text_surface && ((text_surface->w == 8) || (text_surface->w == 16)) && (text_surface->h == 16))
+	{
+		int x, y, o=0, i=0;
+		uint8_t data8[32];
+		*width = text_surface->w;
+
+		for (y=0; y < text_surface->h; y++)
+		{
+			for (x=0; x < text_surface->w; x+=8)
+			{
+				data8[o] = 0;
+				if (((uint8_t*)text_surface->pixels)[i++]) data8[o] |= 0x80;
+				if (((uint8_t*)text_surface->pixels)[i++]) data8[o] |= 0x40;
+				if (((uint8_t*)text_surface->pixels)[i++]) data8[o] |= 0x20;
+				if (((uint8_t*)text_surface->pixels)[i++]) data8[o] |= 0x10;
+				if (((uint8_t*)text_surface->pixels)[i++]) data8[o] |= 0x08;
+				if (((uint8_t*)text_surface->pixels)[i++]) data8[o] |= 0x04;
+				if (((uint8_t*)text_surface->pixels)[i++]) data8[o] |= 0x02;
+				if (((uint8_t*)text_surface->pixels)[i++]) data8[o] |= 0x01;
+				o++;
+			}
+			i -= text_surface->w;
+			i += text_surface->pitch;
+		}
+
+		/* SCALE */
+		if (*width == 16)
+		{
+			int iter = 0;
+			int height = 16;
+			uint16_t data16[16];
+			memcpy (data16, data8, 32);
+			while (height > 8)
+			{
+				int i;
+				iter=!iter;
+				if (data16[0] == 0) /* first is blank */
+				{
+					memmove (data16, data16 + 1, (height - 1)<<1);
+					height--;
+					if (height == 8) break;
+				}
+
+				if ((data16[height-2] == 0) && (data16[height-1])) /* two last are blank */
+				{
+					height--;
+					continue;
+				}
+
+				if (iter)
+				{
+					for (i=0; i < (height - 1); i++) /* find duplicates */
+					{
+						if (data16[i] == data16[i+1])
+						{
+							memmove (data16 + i, data16 + i + 1, (height - i - 1)<<1);
+							height--;
+							i = 20;
+							break;
+						}
+					}
+				} else {
+					for (i=height - 2; i >= 0; i--) /* find duplicates */
+					{
+						if (data16[i] == data16[i+1])
+						{
+							memmove (data16 + i, data16 + i + 1, (height - i - 1)<<1);
+							height--;
+							i = 20;
+							break;
+						}
+					}
+				}
+				if (i == 20) continue;
+
+				if ((data16[height-1])) /* last last is blank */
+				{
+					height--;
+					continue;
+				}
+
+				for (i=0;;i++) /* merge lines */
+				{
+					data16[height - i - 1] |= data16[height - i];
+					memmove (data16 + height - i - 1, data16 + height - i, (i)<<1);
+					height--;
+					if (height <= 8) break;
+				}
+			}
+			memcpy (data, data16, 16);
+		} else {
+			int iter = 0;
+			int height = 16;
+			while (height > 8)
+			{
+				int i;
+				iter=!iter;
+				if (data8[0] == 0) /* first is blank */
+				{
+					memmove (data8, data8 + 1, height - 1);
+					height--;
+					if (height == 8) break;
+				}
+
+				if ((data8[height-2] == 0) && (data8[height-1])) /* two last are blank */
+				{
+					height--;
+					continue;
+				}
+
+				if (iter)
+				{
+					for (i=0; i < (height - 1); i++) /* find duplicates */
+					{
+						if (data8[i] == data8[i+1])
+						{
+							memmove (data8 + i, data8 + i + 1, height - i - 1);
+							height--;
+							i = 20;
+							break;
+						}
+					}
+				} else {
+					for (i=height - 2; i >= 0; i--) /* find duplicates */
+					{
+						if (data8[i] == data8[i+1])
+						{
+							memmove (data8 + i, data8 + i + 1, height - i - 1);
+							height--;
+							i = 20;
+							break;
+						}
+					}
+				}
+				if (i == 20) continue;
+
+				if ((data8[height-1])) /* last last is blank */
+				{
+					height--;
+					continue;
+				}
+
+				for (i=0;;i++) /* merge lines */
+				{
+					data8[height - i - 1] |= data8[height - i];
+					memmove (data8 + height - i - 1, data8 + height - i, i);
+					height--;
+					if (height <= 8) break;
+				}
+			}
+			memcpy (data, data8, 8);
+		}
+		return 0;
+	} else {
+		*width = 8;
+		memset (data, 0, 16);
+		return 1;
+	}
+}
+
 /* width will be set to 8 or 16, depending on the glyph */
 uint8_t *fontengine_8x8(uint32_t codepoint, int *width)
 {
 	int i;
-	TTF_Surface *text_surface = 0;
 	struct font_entry_8x8_t *entry = 0;
 
 	if (codepoint == 0)
@@ -228,72 +417,14 @@ uint8_t *fontengine_8x8(uint32_t codepoint, int *width)
 		}
 	}
 
-#warning TODO, we need to extract and scale these
-#if 0
-	       if (                            (codepoint <= 0x0d8ff)  ||
-	            ((codepoint >= 0x0f900) && (codepoint <= 0x0ffff)) )
-	{
-		text_surface = unifont_bmp ? TTF_RenderGlyph32_Shaded (unifont_bmp, codepoint) : 0;
-	} else if ( ((codepoint >= 0x0e000) && (codepoint <= 0x0f8ff)) )
-	{
-		text_surface = unifont_csur ? TTF_RenderGlyph32_Shaded (unifont_csur, codepoint) : 0;
-	} else if ( ((codepoint >= 0x10000) && (codepoint <= 0x1ffff)) ||
-	            ((codepoint >= 0xe0000) && (codepoint <= 0xeffff)) )
-	{
-		text_surface = unifont_upper ? TTF_RenderGlyph32_Shaded (unifont_upper, codepoint) : 0;
-	} else if ( ((codepoint >= 0xf0000) && (codepoint >= 0xffffd)) )
-	{
-		text_surface = unifont_csur ? TTF_RenderGlyph32_Shaded (unifont_csur, codepoint) : 0;
-	}
-#else
-	fprintf (stderr, "TODO scale glyph U+%X\n", codepoint);
-#endif
-
 	entry = malloc (sizeof (*entry));
-	if (text_surface)
-	{
-		if ((text_surface->w != 8) && (text_surface->w != 16))
-		{
-			memset (entry->data, 0xaa, sizeof (entry->data));
-			entry->width = 8;
-			fprintf (stderr, "TTF + unifont + U+%X: gave invalid width: %d\n", (int)codepoint, (int)text_surface->w);
-		} if (text_surface->h != 16)
-		{
-			memset (entry->data, 0x42, sizeof (entry->data));
-			entry->width = 8;
-			fprintf (stderr, "TTF + unifont + U+%X: gave invalid height: %d\n", (int)codepoint, (int)text_surface->h);
-		} else {
-			int x, y, o=0, i=0;
-			entry->width = text_surface->w;
-			for (y=0; y < text_surface->h; y++)
-			{
-				for (x=0; x < text_surface->w; x+=8)
-				{
-					entry->data[o] = 0;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x80;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x40;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x20;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x10;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x08;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x04;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x02;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x01;
-					o++;
-				}
-				i -= text_surface->w;
-				i += text_surface->pitch;
-			}
-		}
-	} else {
-		memset (entry->data, 0x18, sizeof (entry->data));
-		entry->width = 8;
-		fprintf (stderr, "TTF + unifont + U+%X: did not find a glyph\n", (int)codepoint);
-	}
+	fontengine_8x8_forceunifont (codepoint, width, entry->data);
+
+	entry->width = *width;
 	entry->codepoint = codepoint;
 	entry->score=0;
 	fontengine_8x8_append(entry);
 
-	*width = entry->width;
 	return entry->data;
 }
 
@@ -320,11 +451,70 @@ void fontengine_8x16_iterate (void)
 	}
 }
 
+int fontengine_8x16_forceunifont (uint32_t codepoint, int *width, uint8_t data[32])
+{
+	TTF_Surface *text_surface = 0;
+
+	if (codepoint == 0)
+	{
+		codepoint = ' ';
+	}
+
+	       if (                            (codepoint <= 0x0d8ff)  ||
+	            ((codepoint >= 0x0f900) && (codepoint <= 0x0ffff)) )
+	{
+		text_surface = unifont_bmp ? TTF_RenderGlyph32_Shaded (unifont_bmp, codepoint) : 0;
+#if defined(UNIFONT_CSUR_TTF) || defined(UNIFONT_CSUR_OTF)
+	} else if ( ((codepoint >= 0x0e000) && (codepoint <= 0x0f8ff)) )
+	{
+		text_surface = unifont_csur ? TTF_RenderGlyph32_Shaded (unifont_csur, codepoint) : 0;
+#endif
+	} else if ( ((codepoint >= 0x10000) && (codepoint <= 0x1ffff)) ||
+	            ((codepoint >= 0xe0000) && (codepoint <= 0xeffff)) )
+	{
+		text_surface = unifont_upper ? TTF_RenderGlyph32_Shaded (unifont_upper, codepoint) : 0;
+#if defined(UNIFONT_CSUR_TTF) || defined(UNIFONT_CSUR_OTF)
+	} else if ( ((codepoint >= 0xf0000) && (codepoint >= 0xffffd)) )
+	{
+		text_surface = unifont_csur ? TTF_RenderGlyph32_Shaded (unifont_csur, codepoint) : 0;
+#endif
+	}
+
+	if (text_surface && ((text_surface->w == 8) || (text_surface->w == 16)) && (text_surface->h == 16))
+	{
+		int x, y, o=0, i=0;
+		*width = text_surface->w;
+		for (y=0; y < text_surface->h; y++)
+		{
+			for (x=0; x < text_surface->w; x+=8)
+			{
+				data[o] = 0;
+				if (((uint8_t*)text_surface->pixels)[i++]) data[o] |= 0x80;
+				if (((uint8_t*)text_surface->pixels)[i++]) data[o] |= 0x40;
+				if (((uint8_t*)text_surface->pixels)[i++]) data[o] |= 0x20;
+				if (((uint8_t*)text_surface->pixels)[i++]) data[o] |= 0x10;
+				if (((uint8_t*)text_surface->pixels)[i++]) data[o] |= 0x08;
+				if (((uint8_t*)text_surface->pixels)[i++]) data[o] |= 0x04;
+				if (((uint8_t*)text_surface->pixels)[i++]) data[o] |= 0x02;
+				if (((uint8_t*)text_surface->pixels)[i++]) data[o] |= 0x01;
+				o++;
+			}
+			i -= text_surface->w;
+			i += text_surface->pitch;
+		}
+		return 0;
+	} else {
+		*width = 8;
+		memset (data, 0, 32);
+		return 1;
+	}
+}
+
 /* width will be set to 8 or 16, depending on the glyph */
 uint8_t *fontengine_8x16(uint32_t codepoint, int *width)
 {
 	int i;
-	TTF_Surface *text_surface = 0;
+//	TTF_Surface *text_surface = 0;
 	struct font_entry_8x16_t *entry = 0;
 
 	if (codepoint == 0)
@@ -343,77 +533,24 @@ uint8_t *fontengine_8x16(uint32_t codepoint, int *width)
 		}
 	}
 
-	       if (                            (codepoint <= 0x0d8ff)  ||
-	            ((codepoint >= 0x0f900) && (codepoint <= 0x0ffff)) )
-	{
-		text_surface = unifont_bmp ? TTF_RenderGlyph32_Shaded (unifont_bmp, codepoint) : 0;
-#ifdef UNIFONT_CSUR_TTF
-	} else if ( ((codepoint >= 0x0e000) && (codepoint <= 0x0f8ff)) )
-	{
-		text_surface = unifont_csur ? TTF_RenderGlyph32_Shaded (unifont_csur, codepoint) : 0;
-#endif
-	} else if ( ((codepoint >= 0x10000) && (codepoint <= 0x1ffff)) ||
-	            ((codepoint >= 0xe0000) && (codepoint <= 0xeffff)) )
-	{
-		text_surface = unifont_upper ? TTF_RenderGlyph32_Shaded (unifont_upper, codepoint) : 0;
-#ifdef UNIFONT_CSUR_TTF
-	} else if ( ((codepoint >= 0xf0000) && (codepoint >= 0xffffd)) )
-	{
-		text_surface = unifont_csur ? TTF_RenderGlyph32_Shaded (unifont_csur, codepoint) : 0;
-#endif
-	}
-
 	entry = malloc (sizeof (*entry));
-	if (text_surface)
-	{
-		if ((text_surface->w != 8) && (text_surface->w != 16))
-		{
-			memset (entry->data, 0xaa, sizeof (entry->data));
-			entry->width = 8;
-			fprintf (stderr, "TTF + unifont + U+%X: gave invalid width: %d\n", (int)codepoint, (int)text_surface->w);
-		} if (text_surface->h != 16)
-		{
-			memset (entry->data, 0x42, sizeof (entry->data));
-			entry->width = 8;
-			fprintf (stderr, "TTF + unifont + U+%X: gave invalid height: %d\n", (int)codepoint, (int)text_surface->h);
-		} else {
-			int x, y, o=0, i=0;
-			entry->width = text_surface->w;
-			for (y=0; y < text_surface->h; y++)
-			{
-				for (x=0; x < text_surface->w; x+=8)
-				{
-					entry->data[o] = 0;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x80;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x40;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x20;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x10;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x08;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x04;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x02;
-					if (((uint8_t*)text_surface->pixels)[i++]) entry->data[o] |= 0x01;
-					o++;
-				}
-				i -= text_surface->w;
-				i += text_surface->pitch;
-			}
-		}
-	} else {
-		memset (entry->data, 0x18, sizeof (entry->data));
-		entry->width = 8;
-		fprintf (stderr, "TTF + unifont + U+%X: did not find a glyph\n", (int)codepoint);
-	}
+	fontengine_8x16_forceunifont (codepoint, width, entry->data);
+
+	entry->width = *width;
 	entry->codepoint = codepoint;
 	entry->score=0;
 	fontengine_8x16_append(entry);
 
-	*width = entry->width;
 	return entry->data;
 }
 
 int fontengine_init (void)
 {
 	int i;
+
+	char error_ttf[256];
+	char error_otf[256];
+
 	if ( TTF_Init() < 0)
 	{
 		fprintf (stderr, "[TTF] Unable to init truetype-font library: %s\n", TTF_GetError());
@@ -425,22 +562,64 @@ int fontengine_init (void)
 	unifont_bmp = TTF_OpenFontFilename(UNIFONT_TTF, 16, 0, 0, 0);
 	if (!unifont_bmp)
 	{
-		fprintf (stderr, "TTF_OpenFont(\"" UNIFONT_TTF "\") failed: %s\n", TTF_GetError());
+		snprintf (error_ttf, sizeof (error_ttf), "TTF_OpenFont(\"" UNIFONT_TTF "\") failed: %s\n", TTF_GetError());
 		TTF_ClearError();
+
+		unifont_bmp = TTF_OpenFontFilename(UNIFONT_OTF, 16, 0, 0, 0);
+		if (!unifont_bmp)
+		{
+			snprintf (error_otf, sizeof (error_otf), "TTF_OpenFont(\"" UNIFONT_OTF "\") failed: %s\n", TTF_GetError());
+			TTF_ClearError();
+
+			fputs (error_ttf, stderr);
+			fputs (error_otf, stderr);
+		}
 	}
+
 #ifdef UNIFONT_CSUR_TTF
 	unifont_csur = TTF_OpenFontFilename(UNIFONT_CSUR_TTF, 16, 0, 0, 0);
 	if (!unifont_csur)
 	{
-		fprintf (stderr, "TTF_OpenFont(\"" UNIFONT_CSUR_TTF "\") failed: %s\n", TTF_GetError());
+		snprintf (error_ttf, sizeof (error_ttf), "TTF_OpenFont(\"" UNIFONT_CSUR_TTF "\") failed: %s\n", TTF_GetError());
 		TTF_ClearError();
 	}
 #endif
+#ifdef UNIFONT_CSUR_OTF
+	if (!unifont_csur)
+	{
+		unifont_csur = TTF_OpenFontFilename(UNIFONT_CSUR_OTF, 16, 0, 0, 0);
+		if (!unifont_csur)
+		{
+			snprintf (error_otf, sizeof (error_otf), "TTF_OpenFont(\"" UNIFONT_CSUR_OTF "\") failed: %s\n", TTF_GetError());
+			TTF_ClearError();
+		}
+	}
+#endif
+	if (!unifont_csur)
+	{
+#ifdef UNIFONT_CSUR_TTF
+		fputs (error_ttf, stderr);
+#endif
+#ifdef UNIFONT_CSUR_OTF
+		fputs (error_otf, stderr);
+#endif
+	}
+
 	unifont_upper = TTF_OpenFontFilename(UNIFONT_UPPER_TTF , 16, 0, 0, 0);
 	if (!unifont_upper)
 	{
-		fprintf (stderr, "TTF_OpenFont(\"" UNIFONT_UPPER_TTF "\") failed: %s\n", TTF_GetError());
+		snprintf (error_ttf, sizeof (error_ttf), "TTF_OpenFont(\"" UNIFONT_UPPER_TTF "\") failed: %s\n", TTF_GetError());
 		TTF_ClearError();
+
+		unifont_upper = TTF_OpenFontFilename(UNIFONT_UPPER_OTF , 16, 0, 0, 0);
+		if (!unifont_upper)
+		{
+			snprintf (error_otf, sizeof (error_otf), "TTF_OpenFont(\"" UNIFONT_UPPER_OTF "\") failed: %s\n", TTF_GetError());
+			TTF_ClearError();
+
+			fputs (error_ttf, stderr);
+			fputs (error_otf, stderr);
+		}
 	}
 
 	for (i=0; i < 256; i++)
@@ -533,7 +712,7 @@ void fontengine_done (void)
 		TTF_CloseFont(unifont_bmp);
 		unifont_bmp = 0;
 	}
-#ifdef UNIFONT_CSUR_TTF
+#if defined(UNIFONT_CSUR_TTF) || defined(UNIFONT_CSUR_OTF)
 	if (unifont_csur)
 	{
 		TTF_CloseFont(unifont_csur);
