@@ -1,5 +1,5 @@
 /* opencp module Player
- * copyright (c) 2018-'22 Stian Skjelstad <stian.skjelstad@gmail.com>
+ * copyright (c) 2018-'23 Stian Skjelstad <stian.skjelstad@gmail.com>
  *
  * SDL2 graphic driver
  *
@@ -22,17 +22,20 @@
 #include "config.h"
 #include <assert.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <time.h>
 #include <SDL.h>
 #include "types.h"
 #include "boot/console.h"
 #include "boot/psetting.h"
 #include "cpiface/cpiface.h"
-#include "framelock.h"
-#include "poutput.h"
-#include "poutput-sdl2.h"
-#include "poutput-fontengine.h"
-#include "poutput-swtext.h"
+#include "stuff/framelock.h"
+#include "stuff/imsrtns.h"
+#include "stuff/poutput.h"
+#include "stuff/poutput-fontengine.h"
+#include "stuff/poutput-keyboard.h"
+#include "stuff/poutput-sdl2.h"
+#include "stuff/poutput-swtext.h"
 
 typedef enum
 {
@@ -61,7 +64,7 @@ static const mode_gui_data_t mode_gui_data[] =
 	{-1, -1, -1}
 };
 
-static FontSizeEnum plCurrentFontWanted;
+static FontSizeEnum sdl2_CurrentFontWanted;
 
 typedef struct
 {
@@ -91,16 +94,22 @@ static SDL_Texture *current_texture = NULL;
 static int last_text_height;
 static int last_text_width;
 
+static int cachemode = -1;
+static int no_window_resize = 0;
+
 static void (*set_state)(int fullscreen, int width, int height) = 0;
-static int do_fullscreen = 0;
+static int current_fullsceen = 0;
 static int plScrRowBytes = 0;
 static int ekbhit_sdl2dummy(void);
-static int ___valid_key(uint16_t key);
-static void sdl2_gflushpal(void);
-static void sdl2_gupdatepal(unsigned char color, unsigned char _red, unsigned char _green, unsigned char _blue);
+static int sdl2_SetGraphMode (int high);
+static int sdl2_HasKey (uint16_t key);
+static void sdl2_gFlushPal (void);
+static void sdl2_gUpdatePal (uint8_t color, uint8_t _red, uint8_t _green, uint8_t _blue);
 
-static void *SDL2ScrTextGUIOverlayAddBGRA(unsigned int x, unsigned int y, unsigned int width, unsigned int height, unsigned int pitch, uint8_t *data_bgra);
-static void SDL2ScrTextGUIOverlayRemove(void *handle);
+static void *sdl2_TextOverlayAddBGRA (unsigned int x, unsigned int y, unsigned int width, unsigned int height, unsigned int pitch, uint8_t *data_bgra);
+static void sdl2_TextOverlayRemove (void *handle);
+
+static const struct consoleDriver_t sdl2ConsoleDriver;
 
 static uint32_t sdl2_palette[256] = {
 	0xff000000,
@@ -125,6 +134,9 @@ static uint8_t *virtual_framebuffer = 0;
 
 static void sdl2_close_window(void)
 {
+#ifdef SDL2_DEBUG
+	fprintf (stderr, "[SDL2-video] sdl_close_window()");
+#endif
 	if (current_texture)
 	{
 		SDL_DestroyTexture (current_texture);
@@ -282,6 +294,9 @@ static void sdl2_dump_renderer (void)
 
 static void set_state_textmode(int fullscreen, int width, int height)
 {
+#ifdef SDL2_DEBUG
+	fprintf (stderr, " set_state_textmode(fullscreen=%d, width=%d, height=%d\n", fullscreen, width, height);
+#endif
 	/* texture WILL for sure resize, so just get rid of it! */
 	if (current_texture)
 	{
@@ -292,86 +307,111 @@ static void set_state_textmode(int fullscreen, int width, int height)
 	if (virtual_framebuffer)
 	{
 		free (virtual_framebuffer);
-		plVidMem = virtual_framebuffer = 0;
+		Console.VidMem = virtual_framebuffer = 0;
 	}
 
-	if (fullscreen != do_fullscreen)
+	if (fullscreen != current_fullsceen)
 	{
+#ifdef SDL2_DEBUG
+		fprintf (stderr, " fullscreen =! current_fullsceen\n");
+#endif
 		if (fullscreen)
 		{
-			last_text_width  = plScrLineBytes;
-			last_text_height = plScrLines;
+			last_text_width  = Console.GraphBytesPerLine;
+			last_text_height = Console.GraphLines;
+#ifdef SDL2_DEBUG
+			fprintf (stderr, " store previous resolution into last_text_width and last_text_height\n");
+#endif
 		} else {
 			width = last_text_width;
 			height = last_text_height;
+#ifdef SDL2_DEBUG
+			fprintf (stderr, " restore and override width and height\n");
+#endif
 		}
 	}
 
-	if ((fullscreen != do_fullscreen) || (!current_window))
+	if (!width)
 	{
-		sdl2_close_window();
+#ifdef SDL2_DEBUG
+		fprintf (stderr, "[SDL2-video] set_state_textmode() width==0 ???\n");
+#endif
+		width = 640;
+	}
 
-		do_fullscreen = fullscreen;
+	if (!height)
+	{
+#ifdef SDL2_DEBUG
+		fprintf (stderr, "[SDL2-video] set_state_textmode() height==0 ???\n");
+#endif
+		height = 480;
+	}
+
+	if (!no_window_resize)
+	{
+
+		current_fullsceen = fullscreen;
 
 		if (fullscreen)
 		{
-			current_window = SDL_CreateWindow ("Open Cubic Player",
-			                                   SDL_WINDOWPOS_UNDEFINED,
-			                                   SDL_WINDOWPOS_UNDEFINED,
-			                                   0, 0,
-		                                           SDL_WINDOW_FULLSCREEN_DESKTOP);
-			if (current_window)
+			if (!current_window)
 			{
-				SDL_GetWindowSize (current_window, &width, &height);
+				current_window = SDL_CreateWindow ("Open Cubic Player",
+				                                   SDL_WINDOWPOS_UNDEFINED,
+				                                   SDL_WINDOWPOS_UNDEFINED,
+				                                   0, 0,
+				                                   SDL_WINDOW_FULLSCREEN_DESKTOP);
+			} else {
+				SDL_SetWindowFullscreen (current_window, SDL_WINDOW_FULLSCREEN_DESKTOP);
 			}
 		} else {
-			if (!width)
+			if (!current_window)
 			{
-#ifdef SDL2_DEBUG
-				fprintf (stderr, "[SDL2-video] set_state_textmode() width==0 ???\n");
-#endif
-				width = 640;
+				current_window = SDL_CreateWindow ("Open Cubic Player",
+				                                   SDL_WINDOWPOS_UNDEFINED,
+				                                   SDL_WINDOWPOS_UNDEFINED,
+				                                   width, height,
+				                                   SDL_WINDOW_RESIZABLE);
+			} else {
+				SDL_SetWindowFullscreen (current_window, 0);
+				SDL_SetWindowResizable (current_window, SDL_TRUE);
+				SDL_SetWindowSize (current_window, width, height);
 			}
-			if (!height)
-			{
-#ifdef SDL2_DEBUG
-				fprintf (stderr, "[SDL2-video] set_state_textmode() height==0 ???\n");
-#endif
-				height = 480;
-			}
-			current_window = SDL_CreateWindow ("Open Cubic Player",
-			                                   SDL_WINDOWPOS_UNDEFINED,
-			                                   SDL_WINDOWPOS_UNDEFINED,
-		                                           width, height,
-		                                           SDL_WINDOW_RESIZABLE);
-		}
-
-		if (!current_window)
-		{
-			fprintf (stderr, "[SDL2-video]: SDL_CreateWindow: %s (fullscreen=%d %dx%d)\n", SDL_GetError(), fullscreen, width, height);
-			SDL_ClearError();
 		}
+	} else {
+		no_window_resize = 0;
 	}
 
-	while ( ((width/FontSizeInfo[plCurrentFont].w) < 80) || ((height/FontSizeInfo[plCurrentFont].h) < 25))
+	if (!current_window)
+	{
+		fprintf (stderr, "[SDL2-video]: SDL_CreateWindow: %s (fullscreen=%d %dx%d)\n", SDL_GetError(), fullscreen, width, height);
+		SDL_ClearError();
+		exit(1);
+	}
+
+	SDL_GetWindowSize (current_window, &width, &height);
+
+	while ( ((width/FontSizeInfo[Console.CurrentFont].w) < 80) || ((height/FontSizeInfo[Console.CurrentFont].h) < 25))
 	{
 #ifdef SDL2_DEBUG
 		fprintf (stderr, "[SDL2-video] find a smaller font, since (%d/%d)=%d < 80   or   (%d/%d)=%d < 25\n",
-				width, FontSizeInfo[plCurrentFont].w, width/FontSizeInfo[plCurrentFont].w,
-				height, FontSizeInfo[plCurrentFont].h, width/FontSizeInfo[plCurrentFont].h);
+				width,  FontSizeInfo[Console.CurrentFont].w, width/FontSizeInfo[Console.CurrentFont].w,
+				height, FontSizeInfo[Console.CurrentFont].h, width/FontSizeInfo[Console.CurrentFont].h);
 #endif
-		switch (plCurrentFont)
+		switch (Console.CurrentFont)
 		{
 			case _8x16:
-				plCurrentFont = _8x8;
+				Console.CurrentFont = _8x8;
 				break;
 			case _8x8:
 			default:
 				if (!fullscreen)
 				{
+#ifdef SDL2_DEBUG
 					fprintf(stderr, "[SDL2-video] unable to find a small enough font for %d x %d, increasing window size\n", width, height);
-					width = FontSizeInfo[plCurrentFont].w * 80;
-					height = FontSizeInfo[plCurrentFont].h * 25;
+#endif
+					width  = FontSizeInfo[Console.CurrentFont].w * 80;
+					height = FontSizeInfo[Console.CurrentFont].h * 25;
 					SDL_SetWindowSize (current_window, width, height);
 				} else {
 					fprintf(stderr, "[SDL2-video] unable to find a small enough font for %d x %d\n", width, height);
@@ -381,12 +421,12 @@ static void set_state_textmode(int fullscreen, int width, int height)
 		}
 	}
 
-	plScrWidth     = width/FontSizeInfo[plCurrentFont].w;
-	plScrHeight    = height/FontSizeInfo[plCurrentFont].h;
-	plScrLineBytes = width;
-	plScrLines     = height;
+	Console.TextWidth         = width /FontSizeInfo[Console.CurrentFont].w;
+	Console.TextHeight        = height/FontSizeInfo[Console.CurrentFont].h;
+	Console.GraphBytesPerLine = width;
+	Console.GraphLines        = height;
 
-	plScrRowBytes = plScrWidth*2;
+	plScrRowBytes = Console.TextWidth * 2;
 
 	if (!current_renderer)
 	{
@@ -410,7 +450,9 @@ static void set_state_textmode(int fullscreen, int width, int height)
 		current_texture = SDL_CreateTexture (current_renderer, SDL_PIXELFORMAT_ARGB8888, SDL_TEXTUREACCESS_STREAMING, width, height);
 		if (!current_texture)
 		{
+#ifdef SDL2_DEBUG
 			fprintf (stderr, "[SDL2-video]: SDL_CreateTexture: %s\n", SDL_GetError());
+#endif
 			SDL_ClearError();
 			current_texture = SDL_CreateTexture (current_renderer, SDL_PIXELFORMAT_RGB888, SDL_TEXTUREACCESS_STREAMING, width, height);
 			if (!current_texture)
@@ -422,61 +464,84 @@ static void set_state_textmode(int fullscreen, int width, int height)
 		}
 	}
 
-	plVidMem = virtual_framebuffer = calloc (plScrLineBytes, plScrLines);
+	Console.VidMem = virtual_framebuffer = calloc (Console.GraphBytesPerLine, Console.GraphLines);
 
-	sdl2_gflushpal();
+	sdl2_gFlushPal ();
 
 	___push_key(VIRT_KEY_RESIZE);
 }
 
-static void plSetTextMode(unsigned char x)
+static void sdl2_SetTextMode(unsigned char x)
 {
+#ifdef SDL2_DEBUG
+	fprintf (stderr, "sdl2_SetTextMode(%d)\n", x);
+#endif
 	set_state = set_state_textmode;
 
-	___setup_key(ekbhit_sdl2dummy, ekbhit_sdl2dummy);
-	_validkey=___valid_key;
-
-	if ((x==plScrMode) && (current_window))
+	if ((x == Console.CurrentMode) && (current_window))
 	{
-		memset(virtual_framebuffer, 0, plScrLineBytes * plScrLines);
+		memset (virtual_framebuffer, 0, Console.GraphBytesPerLine * Console.GraphLines);
 		return;
 	}
 
-	_plSetGraphMode(-1); /* closes the window, so gdb helper below will have a clear screen */
-
 	if (x==255)
 	{
-		plScrMode=255;
+#ifdef SDL2_DEBUG
+		fprintf (stderr, "gdb helper, plScrMode=255\n");
+#endif
+		sdl2_SetGraphMode (-1); /* closes the window, so gdb helper below will have a clear screen */
+
+		Console.CurrentMode = 255;
 		return; /* gdb helper */
 	}
 
+	if (cachemode >= 0)
+	{
+		sdl2_SetGraphMode (-1); /* closes the window */
+	}
+
 	/* if invalid mode, set it to custom */
 	if (x>7)
 	{
+#ifdef SDL2_DEBUG
+		fprintf (stderr, "custom request\n");
+#endif
 		x=8;
 
 		set_state_textmode(
-			do_fullscreen,
+			current_fullsceen,
 			last_text_width,
 			last_text_height);
 	} else {
-		plCurrentFont = mode_tui_data[x].font;
+#ifdef SDL2_DEBUG
+		fprintf (stderr, "plCurrentFont=%d (mode_tui_data[%d].font))\n", mode_tui_data[x].font, x);
+		fprintf (stderr, "mode_tui_data[%d].gui_mode=%d\n", x, mode_tui_data[x].gui_mode);
+		fprintf (stderr, "mode_gui_data[%d].width=%d\n", mode_tui_data[x].gui_mode, mode_gui_data[mode_tui_data[x].gui_mode].width);
+		fprintf (stderr, "mode_gui_data[%d].height=%d", mode_tui_data[x].gui_mode, mode_gui_data[mode_tui_data[x].gui_mode].height);
+#endif
+
+		Console.CurrentFont = mode_tui_data[x].font;
 
 		set_state_textmode(
-			do_fullscreen,
+			current_fullsceen,
 			mode_gui_data[mode_tui_data[x].gui_mode].width,
 			mode_gui_data[mode_tui_data[x].gui_mode].height);
 	}
 
-	plScrType = plScrMode = x;
+	Console.LastTextMode = Console.CurrentMode = x;
+#ifdef SDL2_DEBUG
+	fprintf (stderr, "[SDL2-video] plScrType = plScrMode = %d\n", Console.LastTextMode );
+#endif
 }
 
-static int cachemode = -1;
-
 static void set_state_graphmode(int fullscreen, int width, int height)
 {
 	mode_gui_t mode;
 
+#ifdef SDL2_DEBUG
+	fprintf (stderr, "[SDL2-video] set_state_graphmode(fullscreen=%d, width=%d, height=%d)", fullscreen, width, height);
+#endif
+
 	/* texture WILL for sure resize, so just get rid of it! */
 	if (current_texture)
 	{
@@ -487,15 +552,15 @@ static void set_state_graphmode(int fullscreen, int width, int height)
 	switch (cachemode)
 	{
 		case 13:
-			plScrMode=13;
+			Console.CurrentMode = 13;
 			mode = MODE_320_200;
 			break;
 		case 0:
-			plScrMode=100;
+			Console.CurrentMode = 100;
 			mode = MODE_640_480;
 			break;
 		case 1:
-			plScrMode=101;
+			Console.CurrentMode = 101;
 			mode = MODE_1024_768;
 			break;
 		default:
@@ -505,28 +570,45 @@ static void set_state_graphmode(int fullscreen, int width, int height)
 	width = mode_gui_data[mode].width;
 	height = mode_gui_data[mode].height;
 
-	if ((fullscreen != do_fullscreen) || (!current_window))
+	if (!no_window_resize)
 	{
-		sdl2_close_window();
-
-		do_fullscreen = fullscreen;
+		current_fullsceen = fullscreen;
 
 		if (fullscreen)
 		{
-			current_window = SDL_CreateWindow ("Open Cubic Player",
-			                                   SDL_WINDOWPOS_UNDEFINED,
-			                                   SDL_WINDOWPOS_UNDEFINED,
-		                                           0, 0,
-		                                           SDL_WINDOW_FULLSCREEN_DESKTOP);
+			if (!current_window)
+			{
+				current_window = SDL_CreateWindow ("Open Cubic Player",
+				                                   SDL_WINDOWPOS_UNDEFINED,
+				                                   SDL_WINDOWPOS_UNDEFINED,
+			                                           0, 0,
+			                                           SDL_WINDOW_FULLSCREEN_DESKTOP);
+			} else {
+				SDL_SetWindowFullscreen (current_window, SDL_WINDOW_FULLSCREEN_DESKTOP);
+			}
 		} else {
-			current_window = SDL_CreateWindow ("Open Cubic Player",
-		                                           SDL_WINDOWPOS_UNDEFINED,
-		                                           SDL_WINDOWPOS_UNDEFINED,
-		                                           width, height,
-		                                           0);
+			if (!current_window)
+			{
+				current_window = SDL_CreateWindow ("Open Cubic Player",
+			                                           SDL_WINDOWPOS_UNDEFINED,
+			                                           SDL_WINDOWPOS_UNDEFINED,
+			                                           width, height,
+			                                           0);
+			} else {
+				SDL_SetWindowFullscreen (current_window, 0);
+				SDL_SetWindowResizable (current_window, SDL_FALSE);
+				SDL_SetWindowSize (current_window, width, height);
+			}
 		}
 	}
 
+	if (!current_window)
+	{
+		fprintf (stderr, "[SDL2-video]: SDL_CreateWindow: %s (fullscreen=%d %dx%d)\n", SDL_GetError(), fullscreen, width, height);
+		SDL_ClearError();
+		exit(1);
+	}
+
 	if (!current_renderer)
 	{
 		current_renderer = SDL_CreateRenderer (current_window, -1, 0);
@@ -546,7 +628,9 @@ static void set_state_graphmode(int fullscreen, int width, int height)
 		current_texture = SDL_CreateTexture (current_renderer, SDL_PIXELFORMAT_ARGB8888, SDL_TEXTUREACCESS_STREAMING, width, height);
 		if (!current_texture)
 		{
+#ifdef SDL2_DEBUG
 			fprintf (stderr, "[SDL2-video]: SDL_CreateTexture: %s\n", SDL_GetError());
+#endif
 			SDL_ClearError();
 			current_texture = SDL_CreateTexture (current_renderer, SDL_PIXELFORMAT_RGB888, SDL_TEXTUREACCESS_STREAMING, width, height);
 			if (!current_texture)
@@ -558,21 +642,25 @@ static void set_state_graphmode(int fullscreen, int width, int height)
 		}
 	}
 
-	plScrLineBytes=width;
-	plScrLines=height;
-	plScrWidth=plScrLineBytes/8;
-	plScrHeight=plScrLines/16;
+	Console.GraphBytesPerLine = width;
+	Console.GraphLines        = height;
+	Console.TextWidth         = Console.GraphBytesPerLine/8;
+	Console.TextHeight        = Console.GraphLines/16;
 
-	plScrRowBytes=plScrWidth*2;
+	plScrRowBytes = Console.TextWidth * 2;
 
-	sdl2_gflushpal();
+	sdl2_gFlushPal ();
 
 	___push_key(VIRT_KEY_RESIZE);
 	return;
 }
 
-static int __plSetGraphMode(int high)
+static int sdl2_SetGraphMode (int high)
 {
+#ifdef SDL2_DEBUG
+	fprintf (stderr, "[SDL2-video] sdl2_SetGraphMode(high=%d)", high);
+#endif
+
 	if (high>=0)
 		set_state = set_state_graphmode;
 
@@ -583,83 +671,83 @@ static int __plSetGraphMode(int high)
 	if (virtual_framebuffer)
 	{
 		free (virtual_framebuffer);
-		plVidMem = virtual_framebuffer = 0;
+		Console.VidMem = virtual_framebuffer = 0;
 	}
-	sdl2_close_window();
 
 	if (high<0)
 	{
 		return 0;
 	}
 
-	___setup_key(ekbhit_sdl2dummy, ekbhit_sdl2dummy);
-	_validkey=___valid_key;
+	set_state_graphmode(current_fullsceen, 0, 0);
 
-	set_state_graphmode(do_fullscreen, 0, 0);
-
-	plVidMem = virtual_framebuffer=malloc(plScrLineBytes * plScrLines);
+	Console.VidMem = virtual_framebuffer = malloc (Console.GraphBytesPerLine * Console.GraphLines);
 
 quick:
 	if (virtual_framebuffer)
-		memset(virtual_framebuffer, 0, plScrLineBytes * plScrLines);
+	{
+		memset (virtual_framebuffer, 0, Console.GraphBytesPerLine * Console.GraphLines);
+	}
 
-	sdl2_gflushpal();
+	sdl2_gFlushPal ();
 	return 0;
 }
 
-static void __vga13(void)
+static void sdl2_vga13 (void)
 {
-	_plSetGraphMode(13);
+	sdl2_SetGraphMode (13);
 }
 
-static int conRestore(void)
+static int sdl2_consoleRestore(void)
 {
 	return 0;
 }
-static void conSave(void)
+
+static void sdl2_consoleSave(void)
 {
 }
 
-static void plDisplaySetupTextMode(void)
+static void sdl2_DisplaySetupTextMode(void)
 {
 	while (1)
 	{
 		uint16_t c;
-		memset(virtual_framebuffer, 0, plScrLineBytes * plScrLines);
+		memset (virtual_framebuffer, 0, Console.GraphBytesPerLine * Console.GraphLines);
 		make_title("sdl2-driver setup", 0);
 		swtext_displaystr_cp437(1, 0, 0x07, "1:  font-size:", 14);
-		swtext_displaystr_cp437(1, 15, plCurrentFont == _8x8 ? 0x0f : 0x07, "8x8", 3);
-		swtext_displaystr_cp437(1, 19, plCurrentFont == _8x16 ? 0x0f : 0x07, "8x16", 4);
+		swtext_displaystr_cp437(1, 15, Console.CurrentFont == _8x8 ? 0x0f : 0x07, "8x8", 3);
+		swtext_displaystr_cp437(1, 19, Console.CurrentFont == _8x16 ? 0x0f : 0x07, "8x16", 4);
 /*
 		swtext_displaystr_cp437(2, 0, 0x07, "2:  fullscreen: ", 16);
 		swtext_displaystr_cp437(3, 0, 0x07, "3:  resolution in fullscreen:", 29);*/
 
-		swtext_displaystr_cp437(plScrHeight-1, 0, 0x17, "  press the number of the item you wish to change and ESC when done", plScrWidth);
+		swtext_displaystr_cp437(Console.TextHeight - 1, 0, 0x17, "  press the number of the item you wish to change and ESC when done", Console.TextWidth);
 
-		while (!_ekbhit())
+		while (!ekbhit())
 		{
 				framelock();
 		}
-		c=_egetch();
+		c = egetch();
 
 		switch (c)
 		{
 			case '1':
 				/* we can assume that we are in text-mode if we are here */
-				plCurrentFontWanted = plCurrentFont = (plCurrentFont == _8x8)?_8x16:_8x8;
-				set_state_textmode(do_fullscreen, plScrLineBytes, plScrLines);
-				cfSetProfileInt("x11", "font", plCurrentFont, 10);
+				sdl2_CurrentFontWanted = Console.CurrentFont = (Console.CurrentFont == _8x8)?_8x16:_8x8;
+				set_state_textmode(current_fullsceen, Console.GraphBytesPerLine, Console.GraphLines);
+				cfSetProfileInt(cfScreenSec, "fontsize", Console.CurrentFont, 10);
 				break;
+			case KEY_EXIT:
 			case KEY_ESC: return;
 		}
 	}
 }
 
-static const char *plGetDisplayTextModeName(void)
+static const char *sdl2_GetDisplayTextModeName(void)
 {
 	static char mode[48];
-	snprintf(mode, sizeof(mode), "res(%dx%d), font(%s)%s", plScrWidth, plScrHeight,
-		plCurrentFont == _8x8 ? "8x8" : "8x16", do_fullscreen?" fullscreen":"");
+	snprintf(mode, sizeof(mode), "res(%dx%d), font(%s)%s", Console.TextWidth, Console.TextHeight,
+		Console.CurrentFont == _8x8 ? "8x8" : "8x16", current_fullsceen?" fullscreen":"");
 	return mode;
 }
 
@@ -673,7 +761,7 @@ struct keytranslate_t translate[] =
 {
 	{SDLK_BACKSPACE,    KEY_BACKSPACE},
 	{SDLK_TAB,          KEY_TAB},
-	{SDLK_DELETE,       KEY_DELETE}, /* ??? */
+	{SDLK_DELETE,       KEY_DELETE},
 	{SDLK_RETURN,       _KEY_ENTER},
 	{SDLK_RETURN2,      _KEY_ENTER},
 	/*SDLK_PAUSE*/
@@ -1020,7 +1108,7 @@ struct keytranslate_t translate_alt[] =
 	{-1,                0xffff},
 };
 
-static int ___valid_key(uint16_t key)
+static int sdl2_HasKey (uint16_t key)
 {
 	int index;
 
@@ -1062,7 +1150,7 @@ static struct SDL2ScrTextGUIOverlay_t **SDL2ScrTextGUIOverlays;
 static int                              SDL2ScrTextGUIOverlays_count;
 static int                              SDL2ScrTextGUIOverlays_size;
 
-static void *SDL2ScrTextGUIOverlayAddBGRA(unsigned int x, unsigned int y, unsigned int width, unsigned int height, unsigned int pitch, uint8_t *data_bgra)
+static void *sdl2_TextOverlayAddBGRA (unsigned int x, unsigned int y, unsigned int width, unsigned int height, unsigned int pitch, uint8_t *data_bgra)
 {
 	struct SDL2ScrTextGUIOverlay_t *e = malloc (sizeof (*e));
 	e->x = x;
@@ -1082,7 +1170,7 @@ static void *SDL2ScrTextGUIOverlayAddBGRA(unsigned int x, unsigned int y, unsign
 	return e;
 }
 
-static void SDL2ScrTextGUIOverlayRemove(void *handle)
+static void sdl2_TextOverlayRemove (void *handle)
 {
 	int i;
 	for (i=0; i < SDL2ScrTextGUIOverlays_count; i++)
@@ -1095,7 +1183,7 @@ static void SDL2ScrTextGUIOverlayRemove(void *handle)
 			return;
 		}
 	}
-	fprintf (stderr, "[SDL2] Warning: SDL2ScrTextGUIOverlayRemove, handle %p not found\n", handle);
+	fprintf (stderr, "[SDL2] Warning: sdl2_TextOverlayRemove, handle %p not found\n", handle);
 }
 
 static void RefreshScreenGraph(void)
@@ -1124,10 +1212,14 @@ static void RefreshScreenGraph(void)
 			int j;
 
 			dst = (uint32_t *)dst_line;
-			for (j=0;j<plScrLineBytes;j++)
+			for (j = 0; j < Console.GraphBytesPerLine; j++)
+			{
 				*(dst++)=sdl2_palette[*(src++)];
-			if ((++Y)>=plScrLines)
+			}
+			if ((++Y) >= Console.GraphLines)
+			{
 				break;
+			}
 			dst_line += /*current_surface->*/pitch;
 		}
 
@@ -1141,7 +1233,7 @@ static void RefreshScreenGraph(void)
 				int tx, x;
 				uint8_t *src, *dst;
 
-				if (y >= plScrLines) { break; }
+				if (y >= Console.GraphLines) { break; }
 
 				tx = SDL2ScrTextGUIOverlays[i]->x + SDL2ScrTextGUIOverlays[i]->width;
 				x = (SDL2ScrTextGUIOverlays[i]->x < 0) ? 0 : SDL2ScrTextGUIOverlays[i]->x;
@@ -1151,7 +1243,7 @@ static void RefreshScreenGraph(void)
 
 				for (; x < tx; x++)
 				{
-					if (x >= plScrLineBytes) { break; }
+					if (x >= Console.GraphBytesPerLine) { break; }
 
 					if (src[3] == 0)
 					{
@@ -1186,10 +1278,10 @@ static void RefreshScreenGraph(void)
 	SDL_RenderCopy (current_renderer, current_texture, NULL, NULL);
 	SDL_RenderPresent (current_renderer);
 
-	if (plCurrentFont == _8x8)
+	if (Console.CurrentFont == _8x8)
 	{
 		fontengine_8x8_iterate ();
-	} else if (plCurrentFont == _8x16)
+	} else if (Console.CurrentFont == _8x16)
 	{
 		fontengine_8x16_iterate ();
 	}
@@ -1214,7 +1306,7 @@ static int ekbhit_sdl2dummy(void)
 {
 	SDL_Event event;
 
-	if (plScrMode<=8)
+	if (Console.CurrentMode <= 8)
 	{
 		RefreshScreenText();
 	} else {
@@ -1235,7 +1327,7 @@ static int ekbhit_sdl2dummy(void)
 #ifdef SDL2_DEBUG
 						fprintf(stderr, "[SDL2-video] CLOSE:\n");
 #endif
-						___push_key(KEY_ESC);
+						___push_key(KEY_EXIT);
 						break;
 					}
 					case SDL_WINDOWEVENT_SIZE_CHANGED:
@@ -1259,21 +1351,22 @@ static int ekbhit_sdl2dummy(void)
 #endif
 						if (current_window && (SDL_GetWindowID(current_window) == event.window.windowID))
 						{
-							plCurrentFont = plCurrentFontWanted;
-							if (!do_fullscreen && (plScrMode == plScrType))
+							Console.CurrentFont = sdl2_CurrentFontWanted;
+							if (!current_fullsceen && (Console.CurrentMode == Console.LastTextMode ))
 							{
 								last_text_height = event.window.data2;
 								last_text_width = event.window.data1;
 							}
-							set_state(do_fullscreen, event.window.data1, event.window.data2);
-							if (!do_fullscreen)
+							no_window_resize = 1;
+							set_state(current_fullsceen, event.window.data1, event.window.data2);
+							if (!current_fullsceen)
 							{
-								if (plScrType == plScrMode) /* if we are in text-mode, make it a custom one */
+								if (Console.LastTextMode == Console.CurrentMode) /* if we are in text-mode, make it a custom one */
 								{
 #ifdef SDL2_DEBUG
-									fprintf (stderr, "CUSTOM MODE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
+									fprintf (stderr, "[SDL2-video] CUSTOM MODE plScrType = plScrMode = 8\n");
 #endif
-									plScrType = plScrMode = 8;
+									Console.LastTextMode = Console.CurrentMode = 8;
 								}
 							}
 						} else {
@@ -1302,7 +1395,7 @@ static int ekbhit_sdl2dummy(void)
 						break;
 
 					case 3:
-						set_state(!do_fullscreen, plScrLineBytes, plScrLines);
+						set_state (!current_fullsceen, Console.GraphBytesPerLine, Console.GraphLines);
 						break;
 
 					case 4:
@@ -1340,7 +1433,9 @@ static int ekbhit_sdl2dummy(void)
 #endif
 				if (skipone)
 				{
-					fprintf (stderr, "skipone hit..\n");
+#ifdef SDL2_DEBIG
+					fprintf (stderr, "              skipone hit..\n");
+#endif
 					skipone=0;
 				} else {
 					int i;
@@ -1395,7 +1490,7 @@ static int ekbhit_sdl2dummy(void)
 				skipone = 0;
 
 				if ((!(event.key.keysym.mod & (KMOD_CTRL|KMOD_ALT))) && // ignore shift
-				      (event.key.keysym.sym >= 32) && (event.key.keysym.sym <= 127))
+				      (event.key.keysym.sym >= 32) && (event.key.keysym.sym < 127))
 				{ // We use SDL_TEXTINPUT event to fetch all these now...
 					break;
 				}
@@ -1446,7 +1541,7 @@ static int ekbhit_sdl2dummy(void)
 					/* TODO, handle ALT-ENTER */
 					if (event.key.keysym.sym==SDLK_RETURN)
 					{
-						set_state(!do_fullscreen, plScrLineBytes, plScrLines);
+						set_state (!current_fullsceen, Console.GraphBytesPerLine, Console.GraphLines);
 					} else {
 						for (index=0;translate_alt[index].OCP!=0xffff;index++)
 							if (translate_alt[index].SDL==event.key.keysym.sym)
@@ -1460,8 +1555,13 @@ static int ekbhit_sdl2dummy(void)
 				}
 
 				if (event.key.keysym.mod & (KMOD_CTRL|KMOD_SHIFT|KMOD_ALT))
+				{
 					break;
+				}
 
+#ifdef SDL2_DEBUG
+				fprintf (stderr, "KEY: %d 0x%x\n", event.key.keysym.sym, event.key.keysym.sym);
+#endif
 				for (index=0;translate[index].OCP!=0xffff;index++)
 				{
 					if (translate[index].SDL==event.key.keysym.sym)
@@ -1479,11 +1579,11 @@ static int ekbhit_sdl2dummy(void)
 	return 0;
 }
 
-static void sdl2_gflushpal(void)
+static void sdl2_gFlushPal(void)
 {
 }
 
-static void sdl2_gupdatepal(unsigned char index, unsigned char _red, unsigned char _green, unsigned char _blue)
+static void sdl2_gUpdatePal (uint8_t index, uint8_t _red, uint8_t _green, uint8_t _blue)
 {
 	uint8_t *pal = (uint8_t *)sdl2_palette;
 	pal[(index<<2)+3] = 0xff;
@@ -1551,51 +1651,23 @@ int sdl2_init(void)
 	SDL_EventState (SDL_TEXTEDITING, SDL_ENABLE);
 
 	/* Fill some default font and window sizes */
-	plCurrentFontWanted = plCurrentFont = cfGetProfileInt("x11", "font", _8x16, 10);
-	if (plCurrentFont > _FONT_MAX)
+	sdl2_CurrentFontWanted = Console.CurrentFont = cfGetProfileInt(cfScreenSec, "fontsize", _8x16, 10);
+	if (Console.CurrentFont > _FONT_MAX)
 	{
-		plCurrentFont = _8x16;
+		Console.CurrentFont = _8x16;
 	}
-	last_text_width  = plScrLineBytes = 640;
-	last_text_height = plScrLines     = 480;
-	plScrType = plScrMode = 8;
+	last_text_width  = Console.GraphBytesPerLine = saturate(cfGetProfileInt(cfScreenSec, "winwidth",  1280, 10), 640, 16384);
+	last_text_height = Console.GraphLines        = saturate(cfGetProfileInt(cfScreenSec, "winheight", 1024, 10), 400, 16384);
+	Console.LastTextMode = Console.CurrentMode = 8;
 
 	need_quit = 1;
 
-	_plSetTextMode=plSetTextMode;
-	_plSetGraphMode=__plSetGraphMode;
-	_gdrawstr=generic_gdrawstr;
-	_gdrawchar8=generic_gdrawchar8;
-	_gdrawchar8p=generic_gdrawchar8p;
-	_gdrawcharp=generic_gdrawcharp;
-	_gdrawchar=generic_gdrawchar;
-	_gupdatestr=generic_gupdatestr;
-	_gupdatepal=sdl2_gupdatepal;
-	_gflushpal=sdl2_gflushpal;
-	_vga13=__vga13;
+	Console.Driver = &sdl2ConsoleDriver;
+	___setup_key(ekbhit_sdl2dummy, ekbhit_sdl2dummy);
 
-	_displayvoid=swtext_displayvoid;
-	_displaystrattr=swtext_displaystrattr_cp437;
-	_displaystr=swtext_displaystr_cp437;
-	_displaystr_utf8=swtext_displaystr_utf8;
-	_measurestr_utf8=swtext_measurestr_utf8;
+	Console.TextGUIOverlay = 1;
 
-	_drawbar=swtext_drawbar;
-	_idrawbar=swtext_idrawbar;
-
-	_setcur=swtext_setcur;
-	_setcurshape=swtext_setcurshape;
-	_conRestore=conRestore;
-	_conSave=conSave;
-
-	_plGetDisplayTextModeName = plGetDisplayTextModeName;
-	_plDisplaySetupTextMode = plDisplaySetupTextMode;
-
-	plScrTextGUIOverlay        = 1;
-	plScrTextGUIOverlayAddBGRA = SDL2ScrTextGUIOverlayAddBGRA;
-	plScrTextGUIOverlayRemove  = SDL2ScrTextGUIOverlayRemove;
-
-	plVidType=vidModern;
+	Console.VidType = vidModern;
 
 	return 0;
 error_out:
@@ -1620,7 +1692,7 @@ void sdl2_done(void)
 	if (virtual_framebuffer)
 	{
 		free (virtual_framebuffer);
-		plVidMem = virtual_framebuffer = 0;
+		Console.VidMem = virtual_framebuffer = 0;
 	}
 
 	need_quit = 0;
@@ -1630,3 +1702,41 @@ void sdl2_done(void)
 	SDL2ScrTextGUIOverlays_size = 0;
 	SDL2ScrTextGUIOverlays_count = 0;
 }
+
+static void sdl2_DosShell (void)
+{
+	/* dummy */
+}
+
+static const struct consoleDriver_t sdl2ConsoleDriver =
+{
+	sdl2_vga13,
+	sdl2_SetTextMode,
+	sdl2_DisplaySetupTextMode,
+	sdl2_GetDisplayTextModeName,
+	swtext_measurestr_utf8,
+	swtext_displaystr_utf8,
+	swtext_displaychr_cp437,
+	swtext_displaystr_cp437,
+	swtext_displaystrattr_cp437,
+	swtext_displayvoid,
+	swtext_drawbar,
+	swtext_idrawbar,
+	sdl2_TextOverlayAddBGRA,
+	sdl2_TextOverlayRemove,
+	sdl2_SetGraphMode,
+	generic_gdrawchar,
+	generic_gdrawcharp,
+	generic_gdrawchar8,
+	generic_gdrawchar8p,
+	generic_gdrawstr,
+	generic_gupdatestr,
+	sdl2_gUpdatePal,
+	sdl2_gFlushPal,
+	sdl2_HasKey,
+	swtext_setcur,
+	swtext_setcurshape,
+	sdl2_consoleRestore,
+	sdl2_consoleSave,
+	sdl2_DosShell
+};
